@@ -99,6 +99,8 @@ protected:
 
 public:
     vector<JGuiObject*> mObjects;
+
+    vector<JGuiObject*> mButtons;
     int mCount;
 
     JGuiController(JGE* jge, int id, JGuiListener* listener);
@@ -108,10 +110,10 @@ public:
     virtual void Update(float dt);
     virtual bool CheckUserInput(JButton key);
 
-    void Add(JGuiObject* ctrl);
-    void RemoveAt(int i);
-    void Remove(int id);
-    void Remove(JGuiObject* ctrl);
+    virtual void Add(JGuiObject* ctrl, bool isButton = false);
+    virtual void RemoveAt(int i, bool isButton = false);
+    virtual void Remove(int id);
+    virtual void Remove(JGuiObject* ctrl);
 
     void SetActionButton(JButton button);
     void SetStyle(int style);
