@@ -3,7 +3,9 @@
 //logging facility
 //#define DOLOG
 
-#ifdef DOLOG
+#include <string>
+//The PSP one is to log stuff in JLogger's lastLog, it does not do full log in a text file unless DOLOG is defined
+#if defined(DOLOG) || defined (PSP)
 #define LOG(x) JLogger::Log(x);
 #else
 #define LOG(x) {};
@@ -17,11 +19,15 @@
 class JLogger{
  public:
   static void Log(const char * text);
+  static void Log(std::string text);
 
   JLogger(const char* text);
   ~JLogger();
 
   const char* mText;
+
+  static std::string lastLog;
+  static int lastTime;
 };
 
 #endif
