@@ -8,6 +8,12 @@
 //
 //-------------------------------------------------------------------------------------
 
+ 
+// Should we add PrecompiledHeader.h to more platforms here? PSP Doesn't support it in JGE (erwan 2011/12/11) 
+#if defined (IOS) || defined (ANDROID)
+#include "PrecompiledHeader.h"
+#endif
+
 #include <iostream>
 #include <map>
 #include <set>
@@ -22,6 +28,9 @@
 #include "../include/JFileSystem.h"
 //#include "../include/JParticleSystem.h"
 
+#if defined (IOS)
+#import "wagicAppDelegate.h"
+#endif
 
 //////////////////////////////////////////////////////////////////////////
 #if defined (WIN32)    // WIN32 specific code
@@ -44,7 +53,9 @@ u8 JGE::GetAnalogY()
 
 #elif defined (LINUX)    // Unix specific code
 #include <sys/time.h>
+#ifdef WITH_FMOD
 #include "../Dependencies/include/fmod.h"
+#endif //WITH_FMOD
 
 
 u8 JGE::GetAnalogX()
@@ -315,6 +326,11 @@ JGE::JGE()
     strcpy(mDebuggingMsg, "");
     mCurrentMusic = NULL;
 #endif
+
+#if defined (ANDROID)
+    mJNIEnv = NULL;
+#endif
+
     Init();
 }
 
@@ -347,8 +363,8 @@ void JGE::Init()
     sceCtrlSetSamplingCycle(0);
     sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
 
-    JRenderer::GetInstance();
-    JFileSystem::GetInstance();
+    //JRenderer::GetInstance(); Lazy loading
+    //JFileSystem::GetInstance(); Lazy loading
     //JSoundSystem::GetInstance(); let's do lazy loading
 
     mDone = false;
@@ -441,8 +457,8 @@ void JGE::Init()
     mDone = false;
     mPaused = false;
     mCriticalAssert = false;
-    JRenderer::GetInstance();
-    JFileSystem::GetInstance();
+    //JRenderer::GetInstance(); Lazy loading
+    //JFileSystem::GetInstance(); Lazy loading
     JSoundSystem::GetInstance();
     LeftClickedProcessed();
 }
@@ -540,6 +556,8 @@ void JGE::Pause()
 
     mPaused = true;
     if (mApp != NULL) mApp->Pause();
+
+    JFileSystem::GetInstance()->Pause();
 }
 
 void JGE::Resume()
@@ -567,5 +585,126 @@ void JGE::Scroll(int inXVelocity, int inYVelocity)
     }
 }
 
+void JGE::SendCommand(string command)
+{
+#if defined (ANDROID)
+    sendJNICommand(command);
+#elif defined (IOS)
+    SendCommand(command, "");
+#endif
+}
+
+void JGE::SendCommand(std::string command, std::string parameter)
+{
+#if defined (IOS)
+    // get the app delegate and have it handle the command
+    wagicAppDelegate *delegate = [ [UIApplication sharedApplication] delegate];
+    [delegate handleWEngineCommand:[NSString stringWithCString: command.c_str() encoding: NSUTF8StringEncoding]
+                     withParameter: [NSString stringWithCString: parameter.c_str() encoding:NSUTF8StringEncoding]];
+    
+#endif
+}
+
+// this controls commands meant to modify/interact with UI
+void JGE::SendCommand(std::string command, float& x, float& y, float& width, float& height)
+{
+#ifdef ANDROID
+
+#elif IOS
+    wagicAppDelegate *delegate = [[UIApplication sharedApplication] delegate];
+    [delegate handleWEngineCommand: [NSString stringWithCString: command.c_str() encoding: NSUTF8StringEncoding] withUIParameters: x yCoordinate: y width: width height: height];
+#endif
+    
+}
+
+
+#if defined (ANDROID)
+JNIEnv * JGE::getJNIEnv()
+{
+	JNIEnv *env;
+    int status = mJavaVM->GetEnv((void **)&env, JNI_VERSION_1_4);
+    if(status == JNI_ERR)
+    {
+        DebugTrace("Failed to get JNI environment, assuming native thread");
+        status = mJavaVM->AttachCurrentThread(&env, NULL);
+        if(status == JNI_ERR)
+        {
+            DebugTrace("callback_handler: failed to attach current thread");
+            return NULL;
+        }
+    }
+    else if (status == JNI_EDETACHED)
+    {
+    	DebugTrace("env is detached.  trying to attach it to the current thread");
+        jint attachSuccess = mJavaVM->AttachCurrentThread(&env,NULL);
+        if(attachSuccess == 0)
+        {
+            return env;
+        }
+        else
+        {
+            return NULL;
+        }//end if/els
+    }
+    else if (status == JNI_OK)
+    {
+    	return env;
+    }
+
+    return NULL;
+}
+
+string JGE::getFileSystemLocation()
+{
+    char result[512];
+    JNIEnv * env = getJNIEnv();
+    if (env == NULL)
+    {
+        DebugTrace("An Error Occurred in getting the JNI Environment whie trying to get the system folder location.  Defaulting to /mnt/sdcard/net.wagic.app/Wagic"); 
+    	return "/mnt/sdcard/Wagic";
+    };
+
+    jclass jniClass = env->FindClass("org/libsdl/app/SDLActivity");
+    jmethodID methodId = env->GetStaticMethodID( jniClass, "getSystemFolderPath", "()Ljava/lang/String;");
+
+    if (methodId == 0)
+    {
+        DebugTrace("An Error Occurred in getting the JNI methodID for getSystemFolderPath. Defaulting to /mnt/sdcard/Wagic"); 
+    	return "/mnt/sdcard/Wagic";
+    };
+
+    jstring systemPath = (jstring) env->CallStaticObjectMethod(jniClass, methodId);
+
+    // Now convert the Java String to C++ char array
+	const char* cstr = env->GetStringUTFChars(systemPath, 0); 
+	string retVal (cstr);
+	env->ReleaseStringUTFChars(systemPath, cstr); 
+	env->DeleteLocalRef(systemPath);
+
+    return retVal;
+}
+
+/// Access to JNI Environment
+void JGE::SetJNIEnv(JNIEnv * env, jclass cls)
+{
+    mJNIEnv = env;
+    mJNIClass = cls;
+    midSendCommand = mJNIEnv->GetStaticMethodID(mJNIClass,"jgeSendCommand","(Ljava/lang/String;)V");
+}
+
+void JGE::setJVM( JavaVM *vm)
+{
+	mJavaVM = vm;
+}
+
+void JGE::sendJNICommand(string command)
+{
+    if (midSendCommand) {
+        mJNIEnv->CallStaticVoidMethod(mJNIClass, midSendCommand, mJNIEnv->NewStringUTF(command.c_str()));
+    }       
+}
+#endif 
+
+
 std::queue< pair< pair<LocalKeySym, JButton>, bool> > JGE::keyBuffer;
 std::multimap<LocalKeySym, JButton> JGE::keyBinds;
