@@ -13,8 +13,6 @@
 
 JGE* JGuiObject::mEngine = NULL;
 
-JGE* JGuiController::mEngine = NULL;
-
 JGuiObject::JGuiObject(int id) :
     mId(id)
 {
@@ -55,11 +53,9 @@ ostream& operator<<(ostream &out, const JGuiObject &j)
     return j.toString(out);
 }
 
-JGuiController::JGuiController(int id, JGuiListener* listener) :
-    mId(id), mListener(listener)
+JGuiController::JGuiController(JGE* jge, int id, JGuiListener* listener) :
+    mEngine(jge), mId(id), mListener(listener)
 {
-    mEngine = JGE::GetInstance();
-
     mBg = NULL;
     mShadingBg = NULL;
 
@@ -82,6 +78,8 @@ JGuiController::~JGuiController()
 {
     for (int i = 0; i < mCount; i++)
         if (mObjects[i] != NULL) delete mObjects[i];
+    for (size_t i = 0; i < mButtons.size(); i++)
+        if (mButtons[i] != NULL) delete mButtons[i];
 
 }
 
@@ -154,33 +152,55 @@ bool JGuiController::CheckUserInput(JButton key)
     }
     else
     { // a dude may have clicked somewhere, we're gonna select the closest object from where he clicked
-        int x, y;
+        int x = -1, y = -1;
         unsigned int distance2;
         unsigned int minDistance2 = -1;
         int n = mCurr;
         if (mEngine->GetLeftClickCoordinates(x, y))
         {
-            for (int i = 0; i < mCount; i++)
+            // first scan the buttons on the screen and then process the other gui elements
+            for (size_t i = 0; i < mButtons.size(); i++)
             {
-                float top, left;
-                if (mObjects[i]->getTopLeft(top, left))
+                if (mButtons[i]->ButtonPressed())
                 {
-                    distance2 = (top - y) * (top - y) + (left - x) * (left - x);
-                    if (distance2 < minDistance2)
-                    {
-                        minDistance2 = distance2;
-                        n = i;
-                    }
+                    mEngine->LeftClickedProcessed();
+                    return true;
                 }
             }
-
-            if (n != mCurr && mObjects[mCurr] != NULL && mObjects[mCurr]->Leaving(JGE_BTN_DOWN))
+            
+            if (mObjects.size())
             {
-                mCurr = n;
-                mObjects[mCurr]->Entering();
+                for (int i = 0; i < mCount; i++)
+                {
+                    float top, left;
+                    if (mObjects[i]->getTopLeft(top, left))
+                    {
+                        distance2 = (top - y) * (top - y) + (left - x) * (left - x);
+                        if (distance2 < minDistance2)
+                        {
+                            minDistance2 = distance2;
+                            n = i;
+                        }
+                        else
+                            break;
+                    }
+                }
+                
+                if (n != mCurr && mObjects[mCurr] != NULL && mObjects[mCurr]->Leaving(JGE_BTN_DOWN))
+                {
+                    mCurr = n;
+                    mObjects[mCurr]->Entering();
+                }
+                // if the same object was selected process click
+                else if (n == mCurr && mObjects[mCurr] != NULL && mObjects[mCurr]->Leaving(JGE_BTN_OK))
+                {
+                    mObjects[mCurr]->Entering();
+                }
+                mEngine->LeftClickedProcessed();
+				mEngine->ResetInput();
+                return true;
             }
             mEngine->LeftClickedProcessed();
-            return true;
         }
     }
     return false;
@@ -189,20 +209,43 @@ bool JGuiController::CheckUserInput(JButton key)
 void JGuiController::Update(float dt)
 {
     for (int i = 0; i < mCount; i++)
-        if (mObjects[i] != NULL) mObjects[i]->Update(dt);
+        if (mObjects[i] != NULL)
+            mObjects[i]->Update(dt);
+    
+    for (size_t i = 0; i < mButtons.size(); i++ )
+        mButtons[i]->Update(dt);
 
-    JButton key = mEngine->ReadButton();
-    CheckUserInput(key);
+    if(mEngine)
+    {
+        JButton key = mEngine->ReadButton();
+        CheckUserInput(key);
+    }
 }
 
-void JGuiController::Add(JGuiObject* ctrl)
+void JGuiController::Add(JGuiObject* ctrl, bool isButton)
 {
-    mObjects.push_back(ctrl);
-    mCount++;
+    if (!isButton)
+    {
+        mObjects.push_back(ctrl);
+        mCount++;
+    }
+    else
+    {
+        mButtons.push_back(ctrl);
+    }
 }
 
-void JGuiController::RemoveAt(int i)
+void JGuiController::RemoveAt(int i, bool isButton)
 {
+    if (isButton)
+    {
+        if (!mButtons[i]) return;
+        mButtons.erase(mButtons.begin() + i);
+        delete mButtons[i];
+        
+        return;
+    }
+    
     if (!mObjects[i]) return;
     mObjects.erase(mObjects.begin() + i);
     delete mObjects[i];
@@ -221,6 +264,15 @@ void JGuiController::Remove(int id)
             return;
         }
     }
+    
+    for (size_t i = 0; i < mButtons.size(); i++)
+    {
+        if (mButtons[i] != NULL && mButtons[i]->GetId() == id)
+        {
+            RemoveAt(i, true);
+            return;
+        }
+    }
 }
 
 void JGuiController::Remove(JGuiObject* ctrl)
@@ -233,6 +285,16 @@ void JGuiController::Remove(JGuiObject* ctrl)
             return;
         }
     }
+
+    
+    for (size_t i = 0; i < mButtons.size(); i++)
+    {
+        if (mButtons[i] != NULL && mButtons[i] == ctrl)
+        {
+            RemoveAt(i, true);
+            return;
+        }
+    }
 }
 
 void JGuiController::SetActionButton(JButton button)
