@@ -144,6 +144,10 @@ JSample *JSoundSystem::LoadSample(const char *fileName)
 }
 
 
+void JSoundSystem::PauseMusic(JMusic *music)
+{
+    StopMP3();
+}
 
 
 void JSoundSystem::PlayMusic(JMusic *music, bool looping)
