@@ -1,13 +1,19 @@
 #import <UIKit/UIKit.h>
 #import <QuartzCore/QuartzCore.h>
-
+#import "AdWhirlDelegateProtocol.h"
+#import "EAGLViewController.h"
+#import "EAGLView.h"
 #import "ESRenderer.h"
 
 // This class wraps the CAEAGLLayer from CoreAnimation into a convenient UIView subclass.
 // The view content is basically an EAGL surface you render your OpenGL scene into.
 // Note that setting the view non-opaque will only work if the EAGL surface has an alpha channel.
-@interface EAGLView : UIView
+@interface EAGLView : UIView<AdWhirlDelegate,UIGestureRecognizerDelegate>
 {    
+    AdWhirlView *adView;
+    //This is a trick, AdMob uses a viewController to display its Ads, trust me, you'll need this
+    EAGLViewController *viewController;
+    
 @private
     id <ESRenderer> renderer;
 
@@ -21,13 +27,19 @@
     id displayLink;
     CGPoint currentLocation;
 }
-
+@property (nonatomic, retain) AdWhirlView *adView;
 @property (readonly, nonatomic, getter=isAnimating) BOOL animating;
 @property (nonatomic) NSInteger animationFrameInterval;
-@property(nonatomic, readwrite) CGPoint currentLocation;
+@property (nonatomic, readwrite) CGPoint currentLocation;
 
 - (void)startAnimation;
 - (void)stopAnimation;
 - (void)drawView:(id)sender;
 
+- (void)updateKeyboard: (NSString *) inputString;
+- (void)removeAds;
+- (void)displayAds;
+
+- (void)destroyGame;
+
 @end
