@@ -1,8 +1,6 @@
 #import "EAGLViewController.h"
 #import "EAGLView.h"
 
-
-
 @interface EAGLViewController (PrivateMethods)
 - (NSString*)interfaceOrientationName:(UIInterfaceOrientation) interfaceOrientation;
 - (NSString*)deviceOrientationName:(UIDeviceOrientation) deviceOrientation;
@@ -10,44 +8,72 @@
 
 @implementation EAGLViewController
 
+@synthesize bannerIsVisible;
+@synthesize eaglView;
+@synthesize inputField;
+
+#pragma mark initialization / deallocation methods
+
 - (id)init {
     self = [super init];
     if (self) {
-        // Custom initialization.
+        inputField = [[UITextField alloc] initWithFrame: CGRectMake(-50, -50, 25, 25)] ;
+        [self.inputField setEnablesReturnKeyAutomatically: YES];
+        [self.inputField setEnabled: YES];
+        [self.inputField setDelegate: self];
+        [inputField setAutoresizingMask:UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight];
+        [inputField setAutocapitalizationType:UITextAutocapitalizationTypeNone];
+        [inputField setAutocorrectionType:UITextAutocorrectionTypeNo];
+        [inputField setKeyboardType: UIKeyboardTypeNamePhonePad];
+        CGRect frame = [[UIScreen mainScreen] applicationFrame];
+        eaglView = [[EAGLView alloc] initWithFrame:frame];
+        [self setView: eaglView];
+        [self.view addSubview: inputField];
+        [inputField release];
     }
     return self;
 }
 
 - (void)dealloc {
+    [eaglView setDelegate: nil];
+    [eaglView release], eaglView = nil;
+    [inputField release], inputField = nil;
     [super dealloc];
 }
 
 
-// Implement loadView to create a view hierarchy programmatically, without using a nib.
-- (void)loadView {
-	NSLog(@"EAGL ViewController - loadView");
-	
-	CGRect frame = [[UIScreen mainScreen] applicationFrame];
-	
-	EAGLView *eaglView = [[[EAGLView alloc] initWithFrame:frame] autorelease];
-	
-	self.view = eaglView;
-}
-
-
-// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
 - (void)viewDidLoad {
 	NSLog(@"EAGL ViewController - view Did Load");
+
 	[super viewDidLoad];
 }
 
 - (void)viewWillAppear:(BOOL)animated {
-	
 	NSLog(@"EAGL ViewController - view Will Appear");
-	
+    [self.view resumeGame];	
 }
 
 
+- (void)viewWillDisappear:(BOOL)animated
+{
+    [self.view pauseGame];
+}
+
+- (void)pauseGame
+{
+    [self.view pauseGame];
+}
+
+- (void)resumeGame
+{
+    [self.view resumeGame];
+}
+
+- (void)endGame
+{
+    [self.view endGame];
+}
+
 - (void)viewDidAppear:(BOOL)animated {
 	
 	NSLog(@"EAGL ViewController - view Did Appear");
@@ -62,41 +88,6 @@
 
 
 
-- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
-    // Overriden to allow any orientation.
-    bool isSmallScreen = (UI_USER_INTERFACE_IDIOM()) == UIUserInterfaceIdiomPhone;
-
-    if ( isSmallScreen && UIInterfaceOrientationIsPortrait(interfaceOrientation))
-        return NO;
-    
-    return YES;
-}
-
-- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
-    
-	UIDeviceOrientation currentDeviceOrientation = [UIDevice currentDevice].orientation;
-	UIInterfaceOrientation currentInterfaceOrientation	= self.interfaceOrientation;
-	
-	NSLog(@"EAGL ViewController - will Rotate To Interface: %@. Current Interface: %@. Current Device: %@", 
-		  [self interfaceOrientationName:toInterfaceOrientation], 
-		  [self interfaceOrientationName:currentInterfaceOrientation], 
-		  [self deviceOrientationName:currentDeviceOrientation]);
-	
-}
-
-- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
-	
-	UIDeviceOrientation currentDeviceOrientation = [UIDevice currentDevice].orientation;
-	UIInterfaceOrientation currentInterfaceOrientation	= self.interfaceOrientation;
-	
-	NSLog(@"EAGL ViewController - did Rotate From Interface: %@. Current Interface: %@. Current Device: %@", 
-		  [self interfaceOrientationName:fromInterfaceOrientation], 
-		  [self interfaceOrientationName:currentInterfaceOrientation], 
-		  [self deviceOrientationName:currentDeviceOrientation]);
-	
-}
-
-
 - (void)didReceiveMemoryWarning {
     // Releases the view if it doesn't have a superview.
     [super didReceiveMemoryWarning];
@@ -111,7 +102,25 @@
     // e.g. self.myOutlet = nil;
 }
 
+#pragma mark - device orientation handlers
 
+- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
+    // Overriden to allow any orientation.
+    bool isSmallScreen = (UI_USER_INTERFACE_IDIOM()) == UIUserInterfaceIdiomPhone;
+
+    if ( isSmallScreen && UIInterfaceOrientationIsPortrait(interfaceOrientation))
+        return NO;
+    
+    return YES;
+}
+
+- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
+    [[eaglView adView] rotateToOrientation: toInterfaceOrientation];
+}
+
+
+#pragma mark -
+#pragma mark Orientation Information
 - (NSString*)interfaceOrientationName:(UIInterfaceOrientation) interfaceOrientation {
 	
 	NSString* result = nil;
@@ -171,6 +180,48 @@
 	return result;
 };
 
+#pragma mark -
+
+#pragma mark UITextFieldDelegate methods
+- (void)toggleKeyboardWithState: (NSString *) initialText
+{
+    UIView *blockerView = [[[UIView alloc] initWithFrame: [self.view frame]] autorelease];
+    [blockerView setBackgroundColor: [UIColor clearColor]];
+    [self.view addSubview: blockerView];
+    if ([[self inputField] becomeFirstResponder])
+    {
+        [inputField setText: initialText];
+        // success in displaying the keyboard
+    }
+}
+
+- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
+{
+    if ([string isEqualToString:@" "])
+        [eaglView updateKeyboard: @"SPACE"];
+    else if ( (string == nil || [string isEqualToString: @""]) && (1 == range.length))
+        [eaglView updateKeyboard: @"DELETE"];
+    else if ( (string == nil || [string isEqualToString: @""]) && (range.location == (range.length-1)))
+        [eaglView updateKeyboard: @"CLEAR"];
+    else
+        [eaglView updateKeyboard: string];
+    
+    return YES;
+}
+
+- (BOOL)textFieldShouldEndEditing:(UITextField *)textField
+{
+    return YES;
+}
 
 
+- (BOOL) textFieldShouldReturn:(UITextField *)textField 
+{
+    [eaglView updateKeyboard: @"SAVE"];
+	[textField resignFirstResponder];
+	return YES;
+}
+
+
+#pragma mark -
 @end
