@@ -6,7 +6,7 @@
 using std::string;
 using std::vector;
 
-#include "AIPlayer.h"
+#include "AIPlayerBaka.h"
 
 class ManaCost;
 class MTGAbility;
@@ -16,6 +16,8 @@ class AIHint
 public:
     string mCondition;
     string mAction;
+    string mCombatAttackTip;
+    vector<string>castOrder;
     int mSourceId;
     AIHint(string line);
 };
@@ -24,7 +26,7 @@ public:
 class AIHints
 {
 protected:
-    AIPlayer * mPlayer;
+    AIPlayerBaka * mPlayer;
     vector<AIHint *> hints;
     AIHint * getByCondition (string condition);
     AIAction * findAbilityRecursive(AIHint * hint, ManaCost * potentialMana);
@@ -34,10 +36,12 @@ protected:
     bool findSource(int sourceId);
     bool abilityMatches(MTGAbility * a, AIHint * hint);
 public:
-    AIHints (AIPlayer * player);
+    AIHints (AIPlayerBaka * player);
     AIAction * suggestAbility(ManaCost * potentialMana);
+    bool HintSaysDontAttack(GameObserver* observer,MTGCardInstance * card = NULL);
+    vector<string> mCastOrder();
     void add(string line);
     ~AIHints();
 };
 
-#endif
\ No newline at end of file
+#endif
