@@ -7,6 +7,7 @@
 #ifndef _ACTIONELEMENT_H_
 #define _ACTIONELEMENT_H_
 #include <JGui.h>
+#include "MTGDefinitions.h"
 
 #define INACTIVE 0
 #define ACTION_REQUESTED 1
@@ -22,11 +23,10 @@ class ActionElement: public JGuiObject
 {
 protected:
     int activeState;
-    int isClone;
-public:
     TargetChooser * tc;
-    int currentPhase;
-    int newPhase;
+public:
+    GamePhase currentPhase;
+    GamePhase newPhase;
     int modal;
     int waitingForAnswer;
     int getActivity();
@@ -48,9 +48,16 @@ public:
     }
     ;
     ActionElement(int id);
+    ActionElement(const ActionElement& copyFromMe);
+    TargetChooser * getActionTc(){return tc;}
+    virtual void setActionTC(TargetChooser * newTc = NULL){this->tc = newTc;}
     virtual ~ActionElement();
     virtual int isReactingToTargetClick(Targetable * card);
     virtual int reactToTargetClick(Targetable * card);
+    virtual int reactToChoiceClick(Targetable * card,int choice = 0,int controlid = 0)
+    {
+        return 0;
+    }
     virtual int isReactingToClick(MTGCardInstance * card, ManaCost * man = NULL)
     {
         return 0;
