@@ -28,14 +28,18 @@ struct CardGui: public PlayGuiObject
 {
 protected:
 
+    static map<string, string>counterGraphics;
+
     /*
     ** Tries to render the Big version of a card picture, backups to text version in case of failure
     */
     static void RenderBig(MTGCard * card, const Pos& pos);
 
-    void RenderCountersBig(const Pos& pos);
+    static void RenderCountersBig(MTGCard * card, const Pos& pos, int drawMode = DrawMode::kNormal);
     static void AlternateRender(MTGCard * card, const Pos& pos);
     static void TinyCropRender(MTGCard * card, const Pos& pos, JQuad * quad);
+    static string FormattedData (string data, string replace, string value);
+    static bool FilterCard (MTGCard * card,string filter);
 
 public:
     static const float Width;
