@@ -9,9 +9,13 @@
 #include "hge/hgeparticle.h"
 #include "DeckMetaData.h"
 #include "TextScroller.h"
+#include "InteractiveButton.h"
 
 class DeckMenu: public JGuiController
 {
+private:
+    InteractiveButton *dismissButton;
+    
 protected:
 
     float mHeight, mWidth, mX, mY;
@@ -42,7 +46,6 @@ protected:
     string getDescription();
     string getMetaInformation();
     DeckMetaData *mSelectedDeck;
-    int mSelectedDeckId;
     bool mShowDetailsScreen;
     bool mAlwaysShowDetailsButton;
     bool mClosed;
@@ -51,7 +54,15 @@ public:
     VerticalTextScroller * mScroller;
     bool mAutoTranslate;
     float mSelectionTargetY;
-
+    
+    int getSelectedDeckId() const 
+    {
+        return mSelectedDeck->getDeckId();
+    }
+    
+    void selectDeck(int deckId, bool isAi);
+    void selectRandomDeck(bool isAi);
+    
     //used for detailed info button
     JQuadPtr pspIcons[8];
     JTexture * pspIconsTexture;
@@ -62,19 +73,13 @@ public:
     DeckMetaData * getSelectedDeck();
     void enableDisplayDetailsOverride();
     bool showDetailsScreen();
-    bool isClosed()
-    {
-        return mClosed;
-    }
-    int getSelectedDeckId()
-    {
-        return mSelectedDeckId;
-    }
+    
+    virtual bool isClosed() const { return mClosed; }
 
-    void Render();
-    void Update(float dt);
-    void Add(int id, const char * Text, string desc = "", bool forceFocus = false, DeckMetaData *deckMetaData = NULL);
-    void Close();
+    virtual void Render();
+    virtual void Update(float dt);
+    virtual void Add(int id, const char * Text, string desc = "", bool forceFocus = false, DeckMetaData *deckMetaData = NULL);
+    virtual void Close();
     void updateScroller();
     void RenderBackground();
     void RenderDeckManaColors();
