@@ -17,6 +17,7 @@
 #include "DeckStats.h"
 #include "WDataSrc.h"
 #include "WGui.h"
+#include "InteractiveButton.h"
 
 #define NO_USER_ACTIVITY_HELP_DELAY 10
 #define NO_USER_ACTIVITY_SHOWCARD_DELAY 0.1
@@ -31,7 +32,8 @@ enum
     STAGE_ONSCREEN_MENU = 5,
     STAGE_WELCOME = 6,
     STAGE_MENU = 7,
-    STAGE_FILTERS = 8
+    STAGE_FILTERS = 8,
+    STAGE_TRANSITION_SELECTED = 9
 };
 
 // TODO: need a better name for MENU_FIRST_MENU, this is reused for the 1st submenu of
@@ -74,13 +76,13 @@ enum DECK_VIEWER_MENU_ITEMS
 #define MED_SPEED 5.0f
 #define LOW_SPEED 1.5
 
-#define MAX_SAVED_FILTERS 8
-#define CARDS_DISPLAYED 7
+#define MAX_SAVED_FILTERS Constants::NB_Colors + 1
+#define CARDS_DISPLAYED 10
 
 class GameStateDeckViewer: public GameState, public JGuiListener
 {
 private:
-    JQuadPtr mIcons[CARDS_DISPLAYED];
+    vector<JQuadPtr> mIcons;
     JQuadPtr pspIcons[8];
     JTexture * pspIconsTexture;
     float last_user_activity;
@@ -93,6 +95,9 @@ private:
     JMusic * bgMusic;
     int lastPos;
     int lastTotal;
+    int mSelected;
+    
+    InteractiveButton *toggleDeckButton, *sellCardButton, *statsPrevButton, *filterButton;
 
     WGuiFilters * filterMenu;
     WSrcDeckViewer * source;
@@ -116,6 +121,11 @@ private:
     void saveDeck(); //Saves the deck and additional necessary information
     void saveAsAIDeck(string deckName); // saves deck as an AI Deck
     int getCurrentPos();
+    void sellCard();
+    void setButtonState(bool state);
+    bool userPressedButton();
+    void RenderButtons();
+    
     pair<float, float> cardsCoordinates[CARDS_DISPLAYED];
 
 public:
