@@ -10,7 +10,6 @@
 class GuiCombat: public GuiLayer
 {
 protected:
-    GameObserver* go;
     DamagerDamaged* active;
     AttackerDamaged* activeAtk;
     static JTexture* ok_tex;
@@ -29,6 +28,9 @@ protected:
     void addOne(DefenserDamaged* blocker, CombatStep);
     void removeOne(DefenserDamaged* blocker, CombatStep);
     void remaskBlkViews(AttackerDamaged* before, AttackerDamaged* after);
+    void shiftLeft();
+    void shiftRight( DamagerDamaged* oldActive );
+    bool didClickOnButton( Pos buttonPosition, int& x, int& y);
     int resolve();
 
 public:
@@ -44,7 +46,6 @@ public:
     virtual bool CheckUserInput(JButton key);
     virtual int receiveEventPlus(WEvent* e);
     virtual int receiveEventMinus(WEvent* e);
-
     typedef vector<AttackerDamaged*>::iterator inner_iterator;
 };
 
