@@ -16,16 +16,18 @@ class GuiLayer
 {
 protected:
     JButton mActionButton;
+    GameObserver* observer;
 public:
     int mCurr;
     vector<JGuiObject *> mObjects;
+    vector<JGuiObject *> manaObjects;
     void Add(JGuiObject * object);
     int Remove(JGuiObject * object);
     int modal;
     bool hasFocus;
     virtual void resetObjects();
     int getMaxId();
-    GuiLayer();
+    GuiLayer(GameObserver *observer);
     virtual ~GuiLayer();
     virtual void Update(float dt);
     virtual bool CheckUserInput(JButton key)
