@@ -10,16 +10,22 @@
 #include "CardSelector.h"
 #include "ManaCost.h"
 
-class OtherAbilitiesEventReceiver: public MTGAbility
+class PermanentAbility: public MTGAbility
+{
+public:
+    int testDestroy() {return 0;};
+    PermanentAbility(GameObserver* observer, int _id);
+};
+
+class OtherAbilitiesEventReceiver: public PermanentAbility
 {
 public:
-    int testDestroy();
     int receiveEvent(WEvent * event);
-    OtherAbilitiesEventReceiver(int _id);
+    OtherAbilitiesEventReceiver(GameObserver* observer, int _id);
     OtherAbilitiesEventReceiver * clone() const;
 };
 
-class MTGEventBonus: public MTGAbility
+class MTGEventBonus: public PermanentAbility
 {
 public:
     int textAlpha;
@@ -55,21 +61,18 @@ public:
 
     int receiveEvent(WEvent * event);
     void grantAward(string awardName,int amount);
-    int testDestroy();
     void Update(float dt);
     void Render();
-    MTGEventBonus(int _id);
+    MTGEventBonus(GameObserver* observer, int _id);
     virtual MTGEventBonus * clone() const;
 };
-
-class MTGPutInPlayRule: public MTGAbility
+class MTGPutInPlayRule: public PermanentAbility
 {
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGPutInPlayRule(int _id);
+    MTGPutInPlayRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "cast card normally";
@@ -82,9 +85,8 @@ class MTGKickerRule: public MTGPutInPlayRule
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGKickerRule(int _id);
+    MTGKickerRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "pay kicker";
@@ -92,7 +94,7 @@ public:
     virtual MTGKickerRule * clone() const;
 };
 
-class MTGAlternativeCostRule: public MTGAbility
+class MTGAlternativeCostRule: public PermanentAbility
 {
 protected:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana, ManaCost *alternateManaCost);
@@ -101,10 +103,8 @@ protected:
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGAlternativeCostRule(int _id);
+    MTGAlternativeCostRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         if(alternativeName.size())
@@ -119,9 +119,8 @@ class MTGBuyBackRule: public MTGAlternativeCostRule
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGBuyBackRule(int _id);
+    MTGBuyBackRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "cast and buy back";
@@ -135,9 +134,8 @@ public:
 
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGFlashBackRule(int _id);
+    MTGFlashBackRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "flash back";
@@ -150,9 +148,8 @@ class MTGRetraceRule: public MTGAlternativeCostRule
 public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGRetraceRule(int _id);
+    MTGRetraceRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "retrace";
@@ -160,15 +157,14 @@ public:
     virtual MTGRetraceRule * clone() const;
 };
 
-class MTGMorphCostRule: public MTGAbility
+class MTGMorphCostRule: public PermanentAbility
 {
 public:
  
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGMorphCostRule(int _id);
+    MTGMorphCostRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "play morphed";
@@ -182,23 +178,14 @@ public:
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int receiveEvent(WEvent *e);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     string suspendmenu;
     virtual ostream& toString(ostream& out) const;
-    MTGSuspendRule(int _id);
-    const char * getMenuText()
-    {
-        suspendmenu = "suspend";
-        //char buffer[20];
-        //sprintf(buffer,"-%i",card->suspendedTime);
-        //suspendmenu.append(buffer);
-        //TODO:make this work so it shows "Suspend-the amount of turns"
-        return suspendmenu.c_str();
-    }
+    MTGSuspendRule(GameObserver* observer, int _id);
+    const char * getMenuText();
     virtual MTGSuspendRule * clone() const;
 };
 
-class MTGAttackRule: public MTGAbility, public Limitor
+class MTGAttackRule: public PermanentAbility, public Limitor
 {
 public:
  
@@ -206,9 +193,8 @@ public:
     virtual bool greyout(Target*);
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGAttackRule(int _id);
+    MTGAttackRule(GameObserver* observer, int _id);
     const char * getMenuText()
     {
         return "Attacker";
@@ -217,70 +203,93 @@ public:
     virtual MTGAttackRule * clone() const;
 };
 
-/* handles combat trigger send recieve events*/
-class MTGCombatTriggersRule: public MTGAbility
+
+class MTGPlaneswalkerAttackRule: public PermanentAbility, public Limitor
 {
 public:
-    MTGCombatTriggersRule(int _id);
+ 
+    virtual bool select(Target*);
+    virtual bool greyout(Target*);
+    int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
+    int reactToClick(MTGCardInstance * card);
+    MTGPlaneswalkerAttackRule(GameObserver* observer, int _id);
+    const char * getMenuText()
+    {
+        return "Attack Planeswalker";
+    }
+    virtual MTGPlaneswalkerAttackRule * clone() const;
+};
+class AAPlaneswalkerAttacked: public InstantAbility
+{
+public:
+    string menuText;
+    MTGCardInstance* attacker;
+    AAPlaneswalkerAttacked(GameObserver* observer, int id, MTGCardInstance * source, MTGCardInstance * target);
+    int resolve();
+    const char* getMenuText();
+    AAPlaneswalkerAttacked * clone() const;
+    ~AAPlaneswalkerAttacked();
+};
+/* handles combat trigger send recieve events*/
+class MTGCombatTriggersRule: public PermanentAbility
+{
+public:
+    MTGCombatTriggersRule(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
     virtual ostream& toString(ostream& out) const;
-    int testDestroy();
     virtual MTGCombatTriggersRule * clone() const;
 };
 
-class MTGBlockRule: public MTGAbility
+class MTGBlockRule: public PermanentAbility
 {
 public:
+    string blockmenu;
+    TargetChooser * tcb;
+    MTGAbility * blocker;
+    MTGAbility * blockAbility;
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
-    int testDestroy();
     virtual ostream& toString(ostream& out) const;
-    MTGBlockRule(int _id);
-    const char * getMenuText()
-    {
-        return "Blocker";
-    }
+    MTGBlockRule(GameObserver* observer, int _id);
+    const char * getMenuText();
     virtual MTGBlockRule * clone() const;
+    ~MTGBlockRule();
 };
 
 /* Persist Rule */
-class MTGPersistRule: public MTGAbility
+class MTGPersistRule: public PermanentAbility
 {
 public:
-    MTGPersistRule(int _id);
+    MTGPersistRule(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
     virtual ostream& toString(ostream& out) const;
-    int testDestroy();
     virtual MTGPersistRule * clone() const;
 };
 /* vampire Rule */
-class MTGVampireRule: public MTGAbility
+class MTGVampireRule: public PermanentAbility
 {
 public:
-    MTGVampireRule(int _id);
-    map<MTGCardInstance*,vector<MTGCardInstance*> > victems;
+    MTGVampireRule(GameObserver* observer, int _id);
+    map<MTGCardInstance*,vector<MTGCardInstance*> > victims;
     int receiveEvent(WEvent * event);
     virtual ostream& toString(ostream& out) const;
-    int testDestroy();
     virtual MTGVampireRule * clone() const;
 };
 //unearths destruction if leaves play effect
-class MTGUnearthRule: public MTGAbility
+class MTGUnearthRule: public PermanentAbility
 {
 public:
-    MTGUnearthRule(int _id);
+    MTGUnearthRule(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
     virtual ostream& toString(ostream& out) const;
-    int testDestroy();
     virtual MTGUnearthRule * clone() const;
 };
-class MTGTokensCleanup: public MTGAbility
+class MTGTokensCleanup: public PermanentAbility
 {
 public:
     vector<MTGCardInstance *> list;
-    MTGTokensCleanup(int _id);
+    MTGTokensCleanup(GameObserver* observer, int _id);
     int receiveEvent(WEvent * event);
-    int testDestroy();
     virtual MTGTokensCleanup * clone() const;
 };
 
@@ -293,7 +302,7 @@ public:
 class MTGLegendRule: public ListMaintainerAbility
 {
 public:
-    MTGLegendRule(int _id);
+    MTGLegendRule(GameObserver* observer, int _id);
     int canBeInList(MTGCardInstance * card);
     int added(MTGCardInstance * card);
     int removed(MTGCardInstance * card);
@@ -304,7 +313,7 @@ public:
 class MTGPlaneWalkerRule: public ListMaintainerAbility
 {
 public:
-    MTGPlaneWalkerRule(int _id);
+    MTGPlaneWalkerRule(GameObserver* observer, int _id);
     int canBeInList(MTGCardInstance * card);
     int added(MTGCardInstance * card);
     int removed(MTGCardInstance * card);
@@ -312,13 +321,22 @@ public:
     virtual ostream& toString(ostream& out) const;
     virtual MTGPlaneWalkerRule * clone() const;
 };
+/* LifeLink */
+class MTGPlaneswalkerDamage: public PermanentAbility
+{
+public:
+    MTGPlaneswalkerDamage(GameObserver* observer, int _id);
 
-class MTGMomirRule: public MTGAbility
+    int receiveEvent(WEvent * event);
+
+    virtual MTGPlaneswalkerDamage * clone() const;
+};
+class MTGMomirRule: public PermanentAbility
 {
 private:
     int genRandomCreatureId(int convertedCost);
-    static vector<int> pool[20];
-    static int initialized;
+    vector<int> pool[20];
+    int initialized;
 
     int textAlpha;
     string text;
@@ -327,10 +345,9 @@ public:
     int alreadyplayed;
     MTGAllCards * collection;
     MTGCardInstance * genCreature(int id);
-    int testDestroy();
     void Update(float dt);
     void Render();
-    MTGMomirRule(int _id, MTGAllCards * _collection);
+    MTGMomirRule(GameObserver* observer, int _id, MTGAllCards * _collection);
     int isReactingToClick(MTGCardInstance * card, ManaCost * mana = NULL);
     int reactToClick(MTGCardInstance * card);
     int reactToClick(MTGCardInstance * card, int id);
@@ -343,17 +360,16 @@ public:
 };
 
 //stone hewer gaint avatar mode
-class MTGStoneHewerRule: public MTGAbility
+class MTGStoneHewerRule: public PermanentAbility
 {
 private:
     int genRandomEquipId(int convertedCost);
-    static vector<int> pool[20];
-    static int initialized;
+    vector<int> pool[20];
+    int initialized;
 public:
 	MTGAllCards * collection;
 	MTGCardInstance * genEquip(int id);
-	int testDestroy();
-	MTGStoneHewerRule(int _id, MTGAllCards * _collection);
+    MTGStoneHewerRule(GameObserver* observer, int _id, MTGAllCards * _collection);
 	int receiveEvent(WEvent * event);
 	const char * getMenuText()
 	{
@@ -363,11 +379,10 @@ public:
 	virtual MTGStoneHewerRule * clone() const;
 };
 //Hermit Druid avatar mode
-class MTGHermitRule: public MTGAbility
+class MTGHermitRule: public PermanentAbility
 {
 public:
-	int testDestroy();
-	MTGHermitRule(int _id);
+    MTGHermitRule(GameObserver* observer, int _id);
 	int receiveEvent(WEvent * event);
 	const char * getMenuText()
 	{
@@ -377,29 +392,26 @@ public:
 };
 //
 /* LifeLink */
-class MTGLifelinkRule: public MTGAbility
+class MTGLifelinkRule: public PermanentAbility
 {
 public:
-    MTGLifelinkRule(int _id);
+    MTGLifelinkRule(GameObserver* observer, int _id);
 
     int receiveEvent(WEvent * event);
 
-    int testDestroy();
-
     virtual ostream& toString(ostream& out) const;
 
     virtual MTGLifelinkRule * clone() const;
 };
 
 /* Deathtouch */
-class MTGDeathtouchRule: public MTGAbility
+class MTGDeathtouchRule: public PermanentAbility
 {
 public:
-    MTGDeathtouchRule(int _id);
+    MTGDeathtouchRule(GameObserver* observer, int _id);
 
     int receiveEvent(WEvent * event);
 
-    int testDestroy();
     const char * getMenuText()
     {
         return "Deathtouch";
@@ -408,6 +420,15 @@ public:
     virtual MTGDeathtouchRule * clone() const;
 };
 
+/* handling parentchild */
+class ParentChildRule: public PermanentAbility
+{
+public:
+    ParentChildRule(GameObserver* observer, int _id);
+    int receiveEvent(WEvent * event);
+    virtual ostream& toString(ostream& out) const;
+    virtual ParentChildRule * clone() const;
+};
 /* HUD Display */
 
 class HUDString
@@ -424,7 +445,7 @@ public:
     ;
 };
 
-class HUDDisplay: public MTGAbility
+class HUDDisplay: public PermanentAbility
 {
 private:
     list<HUDString *> events;
@@ -434,11 +455,10 @@ private:
     float maxWidth;
     int addEvent(string s);
 public:
-    int testDestroy();
     int receiveEvent(WEvent * event);
     void Update(float dt);
     void Render();
-    HUDDisplay(int _id);
+    HUDDisplay(GameObserver* observer, int _id);
     ~HUDDisplay();
     virtual HUDDisplay * clone() const;
 };
