@@ -21,11 +21,12 @@ class ManaCost
     friend std::ostream& operator<<(std::ostream& out, ManaCost& m);
     friend std::ostream& operator<<(std::ostream& out, ManaCost* m);
     friend std::ostream& operator<<(std::ostream& out, ManaCost m);
-
+    
 protected:
-    int8_t cost[Constants::MTG_NB_COLORS+1];
+    std::vector<int8_t> cost;
     std::vector<ManaCostHybrid> hybrids;
 
+    virtual void init();
 
 public:
     enum
@@ -50,12 +51,16 @@ public:
     ManaCost * morph;
     ManaCost * suspend;
     string alternativeName;
+    bool isMulti;
     static ManaCost * parseManaCost(string value, ManaCost * _manacost = NULL, MTGCardInstance * c = NULL);
-    virtual void init();
-    virtual void reinit();
+    virtual void resetCosts();
     void x();
     int hasX();
-    ManaCost(int8_t _cost[], int nb_elems = 1);
+    void specificX(int color = 0);
+    int hasSpecificX();
+    int xColor;
+    int hasAnotherCost();
+    ManaCost(std::vector<int8_t>& _cost, int nb_elems = 1);
     ManaCost();
     ~ManaCost();
     ManaCost(ManaCost * _manaCost);
@@ -76,14 +81,16 @@ public:
     // Extra Costs (sacrifice,counters...)
     //
     int addExtraCost(ExtraCost * _cost);
+    int addExtraCosts(ExtraCosts *_cost);
     int setExtraCostsAction(MTGAbility * action, MTGCardInstance * card);
     int isExtraPaymentSet();
     int canPayExtra();
     int doPayExtra();
+    ExtraCost * getExtraCost(unsigned int i);
 
     int addHybrid(int c1, int v1, int c2, int v2);
-    int tryToPayHybrids(std::vector<ManaCostHybrid>& _hybrids, int _nbhybrids, int8_t diff[]);
-    void randomDiffHybrids(ManaCost * _cost, int8_t diff[]);
+    int tryToPayHybrids(std::vector<ManaCostHybrid>& _hybrids, int _nbhybrids,std::vector<int8_t>& diff);
+    void randomDiffHybrids(ManaCost * _cost, std::vector<int8_t>& diff);
     int add(ManaCost * _cost);
     int remove(ManaCost * _cost);
     int removeAll(int color);
@@ -105,7 +112,7 @@ class ManaPool:public ManaCost{
 protected:
     Player * player;
 public:
-    void init();
+    void Empty();
     ManaPool(Player * player);
     ManaPool(ManaCost * _manaCost, Player * player);
     int remove (int color, int value);
