@@ -15,40 +15,42 @@ class Player: public Damageable
 {
 protected:
     ManaPool * manaPool;
+    JTexture * mAvatarTex;
+    JQuadPtr mAvatar;
+    bool loadAvatar(string file, string resName = "playerAvatar");
+    bool premade;
 
 public:
-    enum ENUM_PLAY_MODE
+    enum Mode
     {
         MODE_TEST_SUITE,
         MODE_HUMAN,
         MODE_AI
     };
 
-    JTexture * mAvatarTex;
-    JQuadPtr mAvatar;
-    int playMode;
+    int deckId;
+    string mAvatarName;
+    Mode playMode;
     bool nomaxhandsize;
-    bool isPoisoned;
     MTGPlayerCards * game;
     MTGDeck * mDeck;
     string deckFile;
     string deckFileSmall;
     string deckName;
     string phaseRing;
-	int offerInterruptOnPhase;
-    Player(string deckFile, string deckFileSmall, MTGDeck * deck = NULL);
+    int offerInterruptOnPhase;
+    int skippingTurn;
+    int extraTurn;
+    vector<MTGCardInstance*>curses;
+    Player(GameObserver *observer, string deckFile, string deckFileSmall, MTGDeck * deck = NULL);
     virtual ~Player();
-
+    virtual void setObserver(GameObserver*g);
     virtual void End();
     virtual int displayStack()
     {
         return 1;
     }
     const string getDisplayName() const;
-    int typeAsTarget()
-    {
-        return TARGET_PLAYER;
-    }
 
     int afterDamage();
 
@@ -56,6 +58,7 @@ public:
     int loseLife(int value);
     int gainOrLoseLife(int value);
 
+    bool isPoisoned() {return (poisonCount > 0);}
     int poisoned();
     int damaged();
     int prevented();
@@ -75,6 +78,11 @@ public:
         return 0;
     }
 
+    bool isHuman()
+    {
+        return (playMode == MODE_HUMAN);
+    }
+
     Player * opponent();
     int getId();
     JQuadPtr getIcon();
@@ -88,23 +96,22 @@ public:
     {
     }
 
-    void loadAvatar(string file);
-
     /**
     ** Returns the path to the stats file of currently selected deck. 
     */
     std::string GetCurrentDeckStatsFile();
+    virtual bool parseLine(const string& s);
+    friend ostream& operator<<(ostream&, const Player&);
+    bool operator<(Player& aPlayer);
+    bool isDead();
 };
 
 class HumanPlayer: public Player
 {
 public:
-    HumanPlayer(string deckFile, string deckFileSmall, MTGDeck * deck = NULL);
-    HumanPlayer(string deckFile);
-
+    HumanPlayer(GameObserver *observer, string deckFile, string deckFileSmall, bool premade = false, MTGDeck * deck = NULL);
+    void End();
+    friend ostream& operator<<(ostream&, const HumanPlayer&);
 };
 
-ostream& operator<<(ostream&, const Player&);
-istream& operator>>(istream& in, Player& p);
-
 #endif
