@@ -19,4 +19,12 @@
 #include "JFileSystem.h"
 #include "JLogger.h"
 
-#endif //PRECOMPILEDHEADER_H
\ No newline at end of file
+#include "GameOptions.h"
+
+#include <boost/shared_ptr.hpp>
+
+#if defined (IOS) || defined (ANDROID) || defined (QT_CONFIG) || defined (SDL_CONFIG)
+#define TOUCH_ENABLED
+#endif
+
+#endif //PRECOMPILEDHEADER_H
