@@ -28,8 +28,7 @@ private:
 
     static JQuadPtr spadeR, spadeL, jewel, side;
     static JTexture *spadeRTex, *spadeLTex, *jewelTex, *sideTex;
-    static WFont* titleFont;
-    static hgeParticleSystem* stars;
+    hgeParticleSystem* stars;
 
     inline void MogrifyJewel();
     void drawHorzPole(float x, float y, float width);
@@ -37,17 +36,21 @@ private:
 
 public:
     bool autoTranslate;
-    SimpleMenu(int id, JGuiListener* listener, int fontId, float x, float y, const char * _title = "", int _maxItems = 7, bool centerHorizontal = true, bool centerVertical = true);
+    bool isMultipleChoice;
+    SimpleMenu(JGE*, int id, JGuiListener* listener, int fontId, float x, float y, const char * _title = "", int _maxItems = 7, bool centerHorizontal = true, bool centerVertical = true);
     virtual ~SimpleMenu();
-    void Render();
-    void Update(float dt);
-    void Add(int id, const char * Text, string desc = "", bool forceFocus = false);
-    void Close();
+    virtual void Render();
+    virtual bool CheckUserInput(JButton key);
+    virtual void Update(float dt);
+    virtual void Add(int id, const char * Text, string desc = "", bool forceFocus = false);
+    int getmCurr(){return mCurr;}
+    float getWidth(){return mWidth; }
+    virtual void Close();
 
     void RecenterMenu();
 
     float selectionTargetY;
-    bool isClosed()
+    virtual bool isClosed() const
     {
         return mClosed;
     }
