@@ -32,7 +32,8 @@ public:
         OPPONENT = -1,
         CONTROLLER = 1,
         TARGET_CONTROLLER = 2,
-        OWNER = 3
+        OWNER = 3,
+        TARGETED_PLAYER = 4
     };
     bool other;
     bool withoutProtections;
@@ -259,6 +260,44 @@ public:
     virtual bool equals(TargetChooser * tc);
 };
 
+class BlockableChooser: public TypeTargetChooser
+{
+public:
+    bool withoutProtections;
+    BlockableChooser(GameObserver *observer, int * _zones, int _nbzones, MTGCardInstance * card = NULL, int _maxtargets = 1, bool other = false, bool targetMin = false) :
+    TypeTargetChooser(observer, "creature",_zones, _nbzones, card, _maxtargets, other, targetMin)
+    {
+    }
+    ;
+    BlockableChooser(GameObserver *observer, MTGCardInstance * card = NULL, int _maxtargets = 1, bool other = false,bool targetMin = false) :
+        TypeTargetChooser(observer, "creature", card, _maxtargets, other,targetMin)
+    {
+    }
+    ;
+    virtual bool canTarget(Targetable * target, bool withoutProtections = false);
+    virtual BlockableChooser * clone() const;
+    virtual bool equals(TargetChooser * tc);
+};
+
+class myCursesChooser: public TypeTargetChooser
+{
+public:
+    bool withoutProtections;
+    myCursesChooser(GameObserver *observer, int * _zones, int _nbzones, MTGCardInstance * card = NULL, int _maxtargets = 1, bool other = false, bool targetMin = false) :
+    TypeTargetChooser(observer, "*",_zones, _nbzones, card, _maxtargets, other, targetMin)
+    {
+    }
+    ;
+    myCursesChooser(GameObserver *observer, MTGCardInstance * card = NULL, int _maxtargets = 1, bool other = false,bool targetMin = false) :
+        TypeTargetChooser(observer, "*", card, _maxtargets, other,targetMin)
+    {
+    }
+    ;
+    virtual bool canTarget(Targetable * target, bool withoutProtections = false);
+    virtual myCursesChooser * clone() const;
+    virtual bool equals(TargetChooser * tc);
+};
+
 class ParentChildChooser: public TypeTargetChooser
 {
 public:
