@@ -1,7 +1,7 @@
 #ifndef _DEBUG_H_
 #define _DEBUG_H_
 
-#if WIN32
+#if (defined WIN32) && (!defined QT_CONFIG)
 #define snprintf sprintf_s
 #endif 
 
@@ -32,6 +32,9 @@
 // Debug options - comment/uncomment as needed
 //#define DEBUG_CACHE
 #ifdef _DEBUG
+#ifdef WIN32
+#define AI_CHANGE_TESTING
+#endif
 //#define RENDER_AI_STATS
 #endif
 
