@@ -6,23 +6,21 @@
 #include "AIStats.h"
 #include "AllAbilities.h"
 
-MTGAbility * AIMomirPlayer::momirAbility = NULL;
-
-AIMomirPlayer::AIMomirPlayer(string file, string fileSmall, string avatarFile, MTGDeck * deck) :
-    AIPlayerBaka(file, fileSmall, avatarFile, deck)
+AIMomirPlayer::AIMomirPlayer(GameObserver *observer, string file, string fileSmall, string avatarFile, MTGDeck * deck) :
+    AIPlayerBaka(observer, file, fileSmall, avatarFile, deck)
 {
     momirAbility = NULL;
     agressivity = 100;
 }
 
-int AIMomirPlayer::getEfficiency(AIAction * action)
+int AIMomirPlayer::getEfficiency(OrderedAIAction * action)
 {
     MTGAbility * ability = action->ability;
-    if (ability->cost && !(ability->cost->isExtraPaymentSet())) return 0; //Does not handle abilities with sacrifice yet
+    ManaCost * cost = ability->getCost();
+    if (cost && !(cost->isExtraPaymentSet())) return 0; //Does not handle abilities with sacrifice yet
     int efficiency = AIPlayerBaka::getEfficiency(action);
 
-    GameObserver * g = GameObserver::GetInstance();
-    if (g->getCurrentGamePhase() < Constants::MTG_PHASE_FIRSTMAIN) return 0;
+    if (observer->getCurrentGamePhase() < MTG_PHASE_FIRSTMAIN) return 0;
     return efficiency;
 }
 
@@ -30,8 +28,7 @@ MTGAbility * AIMomirPlayer::getMomirAbility()
 {
     if (momirAbility) return momirAbility;
 
-    GameObserver * g = GameObserver::GetInstance();
-    momirAbility = g->mLayers->actionLayer()->getAbility(MTGAbility::MOMIR);
+    momirAbility = observer->mLayers->actionLayer()->getAbility(MTGAbility::MOMIR);
     return momirAbility;
 }
 
@@ -45,7 +42,7 @@ int AIMomirPlayer::momir()
     int converted = potentialMana->getConvertedCost();
     SAFE_DELETE(potentialMana);
     int efficiency = 100;
-    int chance = 1 + (WRand() % 100);
+    int chance = 1 + (randomGenerator.random() % 100);
     if (converted == 5 && myCreatures > opponentCreatures && game->hand->nb_cards < 4) efficiency = 5; //Strategy: skip 5 drop
     if (converted == 7 && myCreatures > opponentCreatures && game->hand->nb_cards < 2) efficiency = 50; //Strategy: 7 drops have bad upkeep costs and the AI doesn't handle those right now...
     if (converted > 8) converted = 8;
@@ -53,14 +50,17 @@ int AIMomirPlayer::momir()
 
     if (efficiency >= chance)
     {
-        int8_t _cost[] = { Constants::MTG_COLOR_ARTIFACT, converted };
+        
+        std::vector<int8_t> _cost;
+        _cost.push_back(Constants::MTG_COLOR_ARTIFACT);
+        _cost.push_back(converted);
         ManaCost * cost = NEW ManaCost(_cost);
         MTGAbility * ability = getMomirAbility();
         MTGCardInstance * card = game->hand->cards[0];
         if (ability->isReactingToClick(card, cost))
         {
-            tapLandsForMana(cost);
-            AIAction * a = NEW AIAction(ability, card);
+            payTheManaCost(cost);
+            AIAction * a = NEW AIAction(this, ability, card);
             clickstream.push(a);
             result = 1;
         }
@@ -78,24 +78,24 @@ int AIMomirPlayer::computeActions()
      You skip your Five on the play because it is the weakest drop. There are plenty of serviceable guys there, but very few bombs compared to other drops
      the general rule is this: if you want to get to Eight, you have to skip two drops on the play and one drop on the draw.
      */
-    GameObserver * g = GameObserver::GetInstance();
-    Player * p = g->currentPlayer;
-    if (!(g->currentlyActing() == this)) return 0;
+
+    Player * p = observer->currentPlayer;
+    if (!(observer->currentlyActing() == this)) return 0;
     if (chooseTarget()) return 1;
-    int currentGamePhase = g->getCurrentGamePhase();
-    if (g->isInterrupting == this)
+    int currentGamePhase = observer->getCurrentGamePhase();
+    if (observer->isInterrupting == this)
     { // interrupting
         selectAbility();
         return 1;
     }
-    else if (p == this && g->mLayers->stackLayer()->count(0, NOT_RESOLVED) == 0)
+    else if (p == this && observer->mLayers->stackLayer()->count(0, NOT_RESOLVED) == 0)
     { //standard actions
         CardDescriptor cd;
         MTGCardInstance * card = NULL;
 
         switch (currentGamePhase)
         {
-        case Constants::MTG_PHASE_FIRSTMAIN:
+        case MTG_PHASE_FIRSTMAIN:
         {
             ManaCost * potentialMana = getPotentialMana();
             int converted = potentialMana->getConvertedCost();
@@ -110,8 +110,8 @@ int AIMomirPlayer::computeActions()
                 int canPutLandsIntoPlay = game->playRestrictions->canPutIntoZone(card, game->inPlay);
                 if (card && (canPutLandsIntoPlay == PlayRestriction::CAN_PLAY))
                 {
-                    MTGAbility * putIntoPlay = g->mLayers->actionLayer()->getAbility(MTGAbility::PUT_INTO_PLAY);
-                    AIAction * a = NEW AIAction(putIntoPlay, card); //TODO putinplay action
+                    MTGAbility * putIntoPlay = observer->mLayers->actionLayer()->getAbility(MTGAbility::PUT_INTO_PLAY);
+                    AIAction * a = NEW AIAction(this, putIntoPlay, card); //TODO putinplay action
                     clickstream.push(a);
                     return 1;
                 }
@@ -120,7 +120,7 @@ int AIMomirPlayer::computeActions()
             return 1;
             break;
         }
-        case Constants::MTG_PHASE_SECONDMAIN:
+        case MTG_PHASE_SECONDMAIN:
             selectAbility();
             return 1;
             break;
