@@ -37,6 +37,8 @@ CardPrimitive::CardPrimitive()
 
 CardPrimitive::CardPrimitive(CardPrimitive * source)
 {
+    if(!source)
+        return;
     basicAbilities = source->basicAbilities;
 
     for (size_t i = 0; i < source->types.size(); ++i)
@@ -49,12 +51,12 @@ CardPrimitive::CardPrimitive(CardPrimitive * source)
         manaCost.alternative->alternativeName = source->getManaCost()->alternative->alternativeName;
 
     text = source->text;
+    formattedText = source->formattedText;
     setName(source->name);
 
     power = source->power;
     toughness = source->toughness;
-    restriction = source->restriction;
-    otherrestriction = source->otherrestriction;
+    restrictions = source->restrictions ? source->restrictions->clone() : NULL;
     suspendedTime = source->suspendedTime;
 
     magicText = source->magicText;
@@ -66,6 +68,7 @@ CardPrimitive::CardPrimitive(CardPrimitive * source)
 
 CardPrimitive::~CardPrimitive()
 {
+    SAFE_DELETE(restrictions);
 }
 
 int CardPrimitive::init()
@@ -78,7 +81,7 @@ int CardPrimitive::init()
     magicTexts.clear();
     spellTargetType = "";
     alias = 0;
-    hasRestriction = false;
+    restrictions = NULL;
     return 1;
 }
 
@@ -99,36 +102,44 @@ bool CardPrimitive::isSpell()
 
 void CardPrimitive::setRestrictions(string _restriction)
 {
-    restriction = _restriction;
+    if (!restrictions)
+        restrictions = NEW CastRestrictions();
+
+    restrictions->restriction = _restriction;
 }
 
-void CardPrimitive::getRestrictions()
+const string CardPrimitive::getRestrictions()
 {
-    restriction;
+    if (!restrictions)
+        return "";
+
+    return restrictions->restriction;
 }
 
 void CardPrimitive::setOtherRestrictions(string _restriction)
 {
-    otherrestriction = _restriction;
+    if (!restrictions)
+        restrictions = NEW CastRestrictions();
+
+    restrictions->otherrestriction = _restriction;
 }
 
-void CardPrimitive::getOtherRestrictions()
+const string CardPrimitive::getOtherRestrictions()
 {
-    otherrestriction;
+    if (!restrictions)
+        return "";
+
+    return restrictions->otherrestriction;
 }
 
 void CardPrimitive::setColor(const string& _color, int removeAllOthers)
 {
-    if (_color.compare(Constants::kManaBlue) == 0)
-        return setColor(Constants::MTG_COLOR_BLUE, removeAllOthers);
-    if (_color.compare(Constants::kManaRed) == 0)
-        return setColor(Constants::MTG_COLOR_RED, removeAllOthers);
-    if (_color.compare(Constants::kManaGreen) == 0)
-        return setColor(Constants::MTG_COLOR_GREEN, removeAllOthers);
-    if (_color.compare(Constants::kManaBlack) == 0)
-        return setColor(Constants::MTG_COLOR_BLACK, removeAllOthers);
-    if (_color.compare(Constants::kManaWhite) == 0)
-        return setColor(Constants::MTG_COLOR_WHITE, removeAllOthers);
+    for( size_t i =0 ; i< Constants::MTGColorStrings.size(); i++)
+    {
+        if (_color.compare(Constants::MTGColorStrings[i]) == 0)
+            return setColor(i, removeAllOthers);
+    }
+    //Keep artifact compare, to Remove this we need change all MTG.txt
     if (_color.compare("artifact") == 0)
         return setColor(Constants::MTG_COLOR_ARTIFACT, removeAllOthers);
 }
@@ -151,7 +162,7 @@ int CardPrimitive::getColor()
 {
     if (colors)
     {
-        for (int i = 1; i < Constants::MTG_NB_COLORS; i++)
+        for (int i = 1; i < Constants::NB_Colors; i++)
             if (hasColor(i))
                 return i;
     }
@@ -200,13 +211,15 @@ void CardPrimitive::setSubtype(const string& value)
     int parentType = 0;
     for (size_t i = 0; i < types.size(); ++i)
     {
-        if (Subtypes::subtypesList->isType(types[i]))
+        if (MTGAllCards::isType(types[i]))
         {
             parentType = types[i];
             break;
         }
     }
-    int id = Subtypes::subtypesList->add(value, parentType);
+
+
+    int id = MTGAllCards::add(value, parentType);
     addType(id);
 }
 
@@ -221,7 +234,7 @@ void CardPrimitive::addType(int id)
 int CardPrimitive::removeType(string value, int removeAll)
 {
 
-    int id = Subtypes::subtypesList->find(value);
+    int id = MTGAllCards::findType(value);
     return removeType(id, removeAll);
 }
 
@@ -246,10 +259,32 @@ void CardPrimitive::setText(const string& value)
     text = value;
 }
 
-const string& CardPrimitive::getText()
+/* This alters the card structure, but this is intentional for performance and
+*  space purpose: The only time we get the card text is to render it
+*  on the screen, in a formatted way.
+* Formatting the string every frame is not efficient, especially since we always display it the same way
+* Formatting all strings at startup is inefficient too.
+* Instead, we format when requested, but only once, and cache the result.
+* To avoid memory to blow up, in exchange of the cached result, we erase the original string
+*/
+const vector<string>& CardPrimitive::getFormattedText()
 {
-    return text;
-}
+    if (!text.size())
+        return formattedText;
+
+    std::string::size_type found = text.find_first_of("{}");
+    while (found != string::npos)
+    {
+        text[found] = '/';
+        found = text.find_first_of("{}", found + 1);
+    }
+    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAGIC_FONT);
+    mFont->FormatText(text, formattedText);
+
+    text = "";
+
+    return formattedText;
+};
 
 void CardPrimitive::addMagicText(string value)
 {
@@ -304,19 +339,19 @@ bool CardPrimitive::hasSubtype(int _subtype)
 
 bool CardPrimitive::hasType(const char * _type)
 {
-    int id = Subtypes::subtypesList->find(_type);
+    int id = MTGAllCards::findType(_type);
     return hasType(id);
 }
 
 bool CardPrimitive::hasSubtype(const char * _subtype)
 {
-    int id = Subtypes::subtypesList->find(_subtype);
+    int id = MTGAllCards::findType(_subtype);
     return hasType(id);
 }
 
 bool CardPrimitive::hasSubtype(const string& _subtype)
 {
-    int id = Subtypes::subtypesList->find(_subtype);
+    int id = MTGAllCards::findType(_subtype);
     return hasType(id);
 }
 
@@ -386,4 +421,4 @@ uint8_t CardPrimitive::ConvertColorToBitMask(int inColor)
     }
 
     return value;
-}
\ No newline at end of file
+}
