@@ -8,12 +8,14 @@
 #include "WResourceManager.h"
 #include "GameObserver.h"
 
-Damage::Damage(MTGCardInstance * source, Damageable * target)
+Damage::Damage(GameObserver* observer, MTGCardInstance * source, Damageable * target)
+    : Interruptible(observer)
 {
     init(source, target, source->getPower(), DAMAGE_OTHER);
 }
 
-Damage::Damage(MTGCardInstance * source, Damageable * target, int damage, int _typeOfDamage)
+Damage::Damage(GameObserver* observer, MTGCardInstance * source, Damageable * target, int damage, int _typeOfDamage)
+    : Interruptible(observer)
 {
     init(source, target, damage, _typeOfDamage);
 }
@@ -36,16 +38,15 @@ int Damage::resolve()
     if (damage < 0)
         damage = 0; //Negative damages cannot happen
     state = RESOLVED_OK;
-    GameObserver * g = GameObserver::GetInstance();
     WEvent * e = NEW WEventDamage(this);
     //Replacement Effects
-    e = g->replacementEffects->replace(e);
+    e = observer->replacementEffects->replace(e);
     if (!e)
         return 0;
     WEventDamage * ev = dynamic_cast<WEventDamage*> (e);
     if (!ev)
     {
-        g->receiveEvent(e);
+        observer->receiveEvent(e);
         return 0;
     }
     damage = ev->damage->damage;
@@ -114,6 +115,14 @@ int Damage::resolve()
             }
             damage = 0;
         }
+        if ((_target)->has(Constants::HYDRA))
+        {
+            for (int j = damage; j > 0; j--)
+            {
+                (_target)->counters->removeCounter(1, 1);
+            }
+            damage = 0;
+        }
         if (!damage)
         {
             state = RESOLVED_NOK;
@@ -136,13 +145,14 @@ int Damage::resolve()
         }
         if(_target->toughness <= 0 && _target->has(Constants::INDESTRUCTIBLE))
             _target->controller()->game->putInGraveyard(_target);
-
     }
-    else if (target->type_as_damageable == DAMAGEABLE_PLAYER && source->has(Constants::INFECT))
+    else if (target->type_as_damageable == DAMAGEABLE_PLAYER && (source->has(Constants::INFECT)||source->has(Constants::POISONDAMAGER)))
     {
         // Poison on player
         Player * _target = (Player *) target;
-        _target->poisonCount += damage;//this will be changed to poison counters.
+        if(!_target->inPlay()->hasAbility(Constants::POISONSHROUD))
+            _target->poisonCount += damage;//this will be changed to poison counters.
+        _target->damageCount += damage;
     }
     else if (target->type_as_damageable == DAMAGEABLE_PLAYER && (source->has(Constants::POISONTOXIC) ||
                     source->has(Constants::POISONTWOTOXIC) || source->has(Constants::POISONTHREETOXIC)))
@@ -151,17 +161,20 @@ int Damage::resolve()
         Player * _target = (Player *) target;
         a = target->dealDamage(damage);
         target->damageCount += damage;
-        if (source->has(Constants::POISONTOXIC))
+        if(!_target->inPlay()->hasAbility(Constants::POISONSHROUD))
         {
-            _target->poisonCount += 1;
-        }
-        else if (source->has(Constants::POISONTWOTOXIC))
-        {
-            _target->poisonCount += 2;
-        }
-        else
-        {
-            _target->poisonCount += 3;
+            if (source->has(Constants::POISONTOXIC))
+            {
+                _target->poisonCount += 1;
+            }
+            else if (source->has(Constants::POISONTWOTOXIC))
+            {
+                _target->poisonCount += 2;
+            }
+            else
+            {
+                _target->poisonCount += 3;
+            }
         }
 
     }
@@ -185,13 +198,12 @@ int Damage::resolve()
             }
             target->lifeLostThisTurn += damage;
             WEvent * lifed = NEW WEventLife((Player*)target,-damage);
-            GameObserver * game = GameObserver::GetInstance();
-            game->receiveEvent(lifed);
+            observer->receiveEvent(lifed);
         }
     }
 
     //Send (Damage/Replaced effect) event to listeners
-    g->receiveEvent(e);
+    observer->receiveEvent(e);
     return a;
 }
 void Damage::Render()
@@ -233,7 +245,8 @@ ostream& Damage::toString(ostream& out) const
     return out;
 }
 
-DamageStack::DamageStack()
+DamageStack::DamageStack(GameObserver *observer)
+    : GuiLayer(observer), Interruptible(observer)
 {
     currentState = -1;
     type = ACTION_DAMAGES;
@@ -253,7 +266,7 @@ int DamageStack::resolve()
         if (damage->state == NOT_RESOLVED)
             damage->resolve();
     }
-    GameObserver::GetInstance()->receiveEvent(NEW WEventDamageStackResolved());
+    ((Interruptible*)this)->getObserver()->receiveEvent(NEW WEventDamageStackResolved());
     return 1;
 }
 
@@ -292,3 +305,48 @@ ostream& DamageStack::toString(ostream& out) const
 {
     return (out << "DamageStack ::: currentState : " << currentState);
 }
+
+
+ostream& operator<<(ostream& out, const Damageable& p)
+{
+    out << "life=" << p.life << endl;
+    out << "poisoncount=" << p.poisonCount << endl;
+    out << "damagecount=" << p.damageCount << endl;
+    out << "preventable=" << p.preventable << endl;
+    return out;
+}
+
+
+bool Damageable::parseLine(const string& s)
+{
+    size_t limiter = s.find("=");
+    if (limiter == string::npos) limiter = s.find(":");
+    string areaS;
+    if (limiter != string::npos)
+    {
+        areaS = s.substr(0, limiter);
+        if (areaS.compare("life") == 0)
+        {
+            life = atoi((s.substr(limiter + 1)).c_str());
+        }
+        else if (areaS.compare("poisoncount") == 0)
+        {
+            poisonCount = atoi((s.substr(limiter + 1)).c_str());
+        }
+        else if (areaS.compare("damagecount") == 0)
+        {
+            damageCount = atoi((s.substr(limiter + 1)).c_str());
+        }
+        else if (areaS.compare("preventable") == 0)
+        {
+            preventable = atoi((s.substr(limiter + 1)).c_str());
+        }
+        else
+        {
+            return false;
+        }
+    }
+
+    return true;
+}
+
