@@ -29,6 +29,7 @@ int ExtraCost::setSource(MTGCardInstance * _source)
         tc->source = _source;
         // this keeps the target chooser from being unable to select a creature with shroud/protections.
         tc->targetter = NULL;
+        tc->Owner = source->controller();
     }
     else
     {
@@ -185,6 +186,7 @@ int DiscardRandomCost::doPay()
     MTGCardInstance * _target = (MTGCardInstance *) target;
     if (target)
     {
+        source->storedCard = target->createSnapShot();
         _target->controller()->game->discardRandom(_target->controller()->game->hand, source);
         target = NULL;
         if (tc)
@@ -213,8 +215,9 @@ int DiscardCost::doPay()
     MTGCardInstance * _target = (MTGCardInstance *) target;
     if (target)
     {
+        source->storedCard = target->createSnapShot();
         WEvent * e = NEW WEventCardDiscard(target);
-        GameObserver * game = GameObserver::GetInstance();
+        GameObserver * game = target->owner->getObserver();
         game->receiveEvent(e);
         _target->controller()->game->putInGraveyard(_target);
         target = NULL;
@@ -224,8 +227,40 @@ int DiscardCost::doPay()
     }
     return 0;
 }
-//to library cost
 
+//cycling
+CycleCost * CycleCost::clone() const
+{
+    CycleCost * ec = NEW CycleCost(*this);
+    if (tc)
+        ec->tc = tc->clone();
+    return ec;
+}
+
+CycleCost::CycleCost(TargetChooser *_tc) :
+ExtraCost("Cycle", _tc)
+{
+}
+
+int CycleCost::doPay()
+{
+    MTGCardInstance * _source = (MTGCardInstance *) source;
+    if (_source)
+    {
+        WEvent * e = NEW WEventCardDiscard(target);//cycling sends 2 events one for the discard and one for the specific cycle trigger
+        GameObserver * game = _source->owner->getObserver();
+        game->receiveEvent(e);
+        WEvent * e2 = NEW WEventCardCycle(_source);
+        game->receiveEvent(e2);
+        _source->controller()->game->putInGraveyard(_source);
+        if (tc)
+            tc->initTargets();
+        return 1;
+    }
+    return 0;
+}
+
+//to library cost
 ToLibraryCost * ToLibraryCost::clone() const
 {
     ToLibraryCost * ec = NEW ToLibraryCost(*this);
@@ -244,6 +279,7 @@ int ToLibraryCost::doPay()
     MTGCardInstance * _target = (MTGCardInstance *) target;
     if (target)
     {
+        source->storedCard = target->createSnapShot();
         _target->controller()->game->putInLibrary(target);
         target = NULL;
         if (tc)
@@ -269,7 +305,7 @@ MillCost::MillCost(TargetChooser *_tc)
 
 int MillCost::canPay()
 {
-    MTGGameZone * z = target->controller()->game->library;
+    MTGGameZone * z = target?target->controller()->game->library:source->controller()->game->library;
     int nbcards = z->nb_cards;
     if (nbcards < 1)
         return 0;
@@ -281,6 +317,7 @@ int MillCost::doPay()
     MTGCardInstance * _target = (MTGCardInstance *) target;
     if (target)
     {
+        source->storedCard = (MTGCardInstance*)_target->controller()->game->library->cards[_target->controller()->game->library->nb_cards - 1]->createSnapShot();
         _target->controller()->game->putInZone(
             _target->controller()->game->library->cards[_target->controller()->game->library->nb_cards - 1],
             _target->controller()->game->library, _target->controller()->game->graveyard);
@@ -302,6 +339,7 @@ MillExileCost::MillExileCost(TargetChooser *_tc)
 int MillExileCost::doPay()
 {
     MTGCardInstance * _target = (MTGCardInstance *) target;
+    source->storedCard = (MTGCardInstance*)_target->controller()->game->library->cards[_target->controller()->game->library->nb_cards - 1]->createSnapShot();
     if (target)
     {
         _target->controller()->game->putInZone(
@@ -419,6 +457,7 @@ int TapTargetCost::isPaymentSet()
 int TapTargetCost::doPay()
 {
     MTGCardInstance * _target = (MTGCardInstance *) target;
+    source->storedCard = target->createSnapShot();
     if (target)
     {
         _target->tap();
@@ -430,6 +469,49 @@ int TapTargetCost::doPay()
     return 0;
 }
 
+//untap other as a cost
+UnTapTargetCost * UnTapTargetCost::clone() const
+{
+    UnTapTargetCost * ec = NEW UnTapTargetCost(*this);
+    if (tc)
+        ec->tc = tc->clone();
+    return ec;
+}
+
+UnTapTargetCost::UnTapTargetCost(TargetChooser *_tc)
+    : ExtraCost("Untap Target", _tc)
+{
+}
+
+int UnTapTargetCost::isPaymentSet()
+{
+    if (target && !target->isTapped())
+    {
+        tc->removeTarget(target);
+        target->isExtraCostTarget = false;
+        target = NULL;
+        return 0;
+    }
+    if (target)
+        return 1;
+    return 0;
+}
+
+int UnTapTargetCost::doPay()
+{
+    MTGCardInstance * _target = (MTGCardInstance *) target;
+    source->storedCard = target->createSnapShot();
+    if (target)
+    {
+        _target->untap();
+        target = NULL;
+        if (tc)
+            tc->initTargets();
+        return 1;
+    }
+    return 0;
+}
+
 //exile as cost
 ExileTargetCost * ExileTargetCost::clone() const
 {
@@ -449,6 +531,7 @@ int ExileTargetCost::doPay()
 
     if (target)
     {
+        source->storedCard = target->createSnapShot();
         target->controller()->game->putInExile(target);
         target = NULL;
         if (tc)
@@ -477,6 +560,7 @@ int BounceTargetCost::doPay()
 
     if (target)
     {
+        source->storedCard = target->createSnapShot();
         target->controller()->game->putInHand(target);
         target = NULL;
         if (tc)
@@ -500,12 +584,18 @@ ExtraCost("Select unblocked attacker", _tc)
 {
 }
 
+int Ninja::canPay()
+{
+    if(source->getObserver()->getCurrentGamePhase() != MTG_PHASE_COMBATBLOCKERS)
+        return 0;
+    return 1;
+}
+
 int Ninja::isPaymentSet()
 {
-    GameObserver * g = GameObserver::GetInstance();
-    int currentPhase = g->getCurrentGamePhase();
-    if (target && ((target->isAttacker() && target->blocked) || target->isAttacker() < 1 || currentPhase
-        != Constants::MTG_PHASE_COMBATBLOCKERS))
+    if (target && ((target->isAttacker() && target->isBlocked()) ||
+                   target->isAttacker() < 1 ||
+                   target->getObserver()->getCurrentGamePhase() != MTG_PHASE_COMBATBLOCKERS))
     {
         tc->removeTarget(target);
         target = NULL;
@@ -549,10 +639,12 @@ int SacrificeCost::doPay()
 {
     if (target)
     {
-        WEvent * e = NEW WEventCardSacrifice(target);
-        GameObserver * game = GameObserver::GetInstance();
-        game->receiveEvent(e);
+        MTGCardInstance * beforeCard = target;
+        source->storedCard = target->createSnapShot();
         target->controller()->game->putInGraveyard(target);
+        WEvent * e = NEW WEventCardSacrifice(beforeCard,target);
+        GameObserver * game = target->owner->getObserver();
+        game->receiveEvent(e);
         target = NULL;
         if (tc)
             tc->initTargets();
@@ -568,9 +660,14 @@ CounterCost * CounterCost::clone() const
     CounterCost * ec = NEW CounterCost(*this);
     if (tc)
         ec->tc = tc->clone();
+    
     if (counter)
         ec->counter = NEW Counter(counter->target, counter->name.c_str(), counter->power, counter->toughness);
-    ec->counter->nb = counter->nb;
+
+    //TODO: counter can be NULL at this point, what do we set ec->counter->nb to if it is?
+    if (ec->counter != NULL)
+        ec->counter->nb = counter->nb;
+    
     return ec;
 }
 
@@ -626,9 +723,20 @@ int CounterCost::canPay()
     if (counter->nb >= 0)
         return 1; //add counters always possible
     // otherwise, move on only if target has enough counters
-    Counter * targetCounter = target->counters->hasCounter(counter->name.c_str(), counter->power, counter->toughness);
+    Counter * targetCounter = NULL;
+    if(target)
+    {
+        targetCounter = target->counters->hasCounter(counter->name.c_str(), counter->power, counter->toughness);
+
+    }
+    else
+    {
+        targetCounter = source->counters->hasCounter(counter->name.c_str(), counter->power, counter->toughness);
+    }
     if (targetCounter && targetCounter->nb >= -counter->nb)
+    {
         return 1;
+    }
     return 0;
 }
 
@@ -643,6 +751,9 @@ int CounterCost::doPay()
         {
             target->counters->addCounter(counter->name.c_str(), counter->power, counter->toughness);
         }
+        if (tc)
+            tc->initTargets();
+        target = NULL;
         return 1;
     }
 
@@ -654,8 +765,14 @@ int CounterCost::doPay()
             target->counters->removeCounter(counter->name.c_str(), counter->power, counter->toughness);
         }
         hasCounters = 0;
+        if (tc)
+            tc->initTargets();
+        target = NULL;
         return 1;
     }
+    if (tc)
+        tc->initTargets();
+    target = NULL;
     return 0;
 }
 
@@ -697,6 +814,7 @@ int ExtraCosts::setAction(MTGAbility * _action, MTGCardInstance * _card)
 {
     action = _action;
     source = _card;
+    source->storedCard = NULL;
     for (size_t i = 0; i < costs.size(); i++)
     {
         costs[i]->setSource(_card);
@@ -707,6 +825,7 @@ int ExtraCosts::setAction(MTGAbility * _action, MTGCardInstance * _card)
 int ExtraCosts::reset()
 {
     action = NULL;
+    source->storedCard = NULL;
     source = NULL;
     //TODO set all payments to "unset"
     return 1;
@@ -749,7 +868,8 @@ int ExtraCosts::canPay()
     {
         if (!costs[i]->canPay())
         {
-            costs[i]->target->isExtraCostTarget = false;
+            if(costs[i]->target)
+                costs[i]->target->isExtraCostTarget = false;
             return 0;
         }
     }
@@ -762,7 +882,9 @@ int ExtraCosts::doPay()
     for (size_t i = 0; i < costs.size(); i++)
     {
         if(costs[i]->target)
+        {
             costs[i]->target->isExtraCostTarget = false;
+        }
         result += costs[i]->doPay();
     }
     return result;
