@@ -28,19 +28,22 @@
 #include "WFilter.h"
 #include "Rules.h"
 #include "ModRules.h"
+#include "JFileSystem.h"
+#include "Credits.h"
+#include "AbilityParser.h"
 
 #define DEFAULT_DURATION .25
 
-int GameApp::players[] = { 0, 0 };
-int GameApp::HasMusic = 1;
+PlayerType GameApp::players[] = { PLAYER_TYPE_CPU, PLAYER_TYPE_CPU };
+bool GameApp::HasMusic = true;
 JMusic * GameApp::music = NULL;
 string GameApp::currentMusicFile = "";
 string GameApp::systemError = "";
 
-JQuadPtr manaIcons[7];
+vector<JQuadPtr > manaIcons;
 
-GameState::GameState(GameApp* parent) :
-    mParent(parent)
+GameState::GameState(GameApp* parent, string id) :
+    mParent(parent), mStringID(id)
 {
     mEngine = JGE::GetInstance();
 }
@@ -66,8 +69,8 @@ GameApp::GameApp() :
         mGameStates[i] = NULL;
 
     mShowDebugInfo = false;
-    players[0] = 0;
-    players[1] = 0;
+    players[0] = PLAYER_TYPE_CPU;
+    players[1] = PLAYER_TYPE_CPU;
     gameType = GAME_TYPE_CLASSIC;
 
     mCurrentState = NULL;
@@ -89,14 +92,17 @@ void GameApp::Create()
     _CrtSetDbgFlag ( _CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF );
 #elif defined (PSP)
     pspFpuSetEnable(0); //disable FPU Exceptions until we find where the FPU errors come from
+    pspDebugScreenPrintf("Wagic:Loading resources...");
 #endif
 #endif //QT_CONFIG
     //_CrtSetBreakAlloc(368);
     LOG("starting Game");
 
-	WResourceManager::Instance()->ResetCacheLimits();
+    string systemFolder = "Res/";
+    string foldersRoot = "";
+
     //Find the Res folder
-    wagic::ifstream mfile("Res.txt");
+    ifstream mfile("Res.txt");
     string resPath;
     if (mfile)
     {
@@ -105,22 +111,40 @@ void GameApp::Create()
         {
             if (resPath[resPath.size() - 1] == '\r')
                 resPath.erase(resPath.size() - 1); //Handle DOS files
-            string testfile = resPath;
+            string testfile = resPath + systemFolder;
             testfile.append("graphics/simon.dat");
-            if (fileExists(testfile.c_str()))
+            ifstream tempfile(testfile.c_str());
+            if (tempfile)
             {
-                JFileSystem::GetInstance()->SetResourceRoot(trim(resPath));
                 found = true;
+                tempfile.close();
+                foldersRoot = resPath;
             }
         }
         mfile.close();
     }
-    LOG("Res Root:");
-    LOG(JFileSystem::GetInstance()->GetResourceRoot().c_str());
+    LOG("init Res Folder at " + foldersRoot);
+    JFileSystem::init(foldersRoot + "User/", foldersRoot + systemFolder);
 
+    
+    // Create User Folders (for write access) if they don't exist
+    {
+        const char* folders[] = { "ai", "ai/baka", "ai/baka/stats", "campaigns", "graphics", "lang", "packs", "player", "player/stats", "profiles", "rules", "sets", "settings", "sound", "sound/sfx", "themes", "test"};
+
+        for (size_t i = 0; i < sizeof(folders)/sizeof(folders[0]); ++i)
+        {
+            JFileSystem::GetInstance()->MakeDir(string(folders[i]));
+        }
+    }
+
+    LOG("Loading Modrules");
     //Load Mod Rules before everything else
     gModRules.load("rules/modrules.xml");
 
+    LOG("Loading Unlockables");
+    //Load awards (needs to be loaded before any option are accessed)
+    Unlockable::load();
+
     //Link this to our settings manager.
     options.theGame = this;
 
@@ -128,42 +152,46 @@ void GameApp::Create()
     LOG("options.reloadProfile()");
     options.reloadProfile();
 
+
+    //Setup Cache before calling any gfx/sfx functions
+	WResourceManager::Instance()->ResetCacheLimits();
+
+
     LOG("Checking for music files");
     //Test for Music files presence
-    string filepath = JGE_GET_RES(WResourceManager::Instance()->musicFile("Track0.mp3"));
-    wagic::ifstream file(filepath.c_str());
-    if (file)
-        file.close();
-    else
-        HasMusic = 0;
+    JFileSystem * jfs = JFileSystem::GetInstance();
+    HasMusic = jfs->FileExists(WResourceManager::Instance()->musicFile("Track0.mp3")) && jfs->FileExists(WResourceManager::Instance()->musicFile("Track1.mp3"));
+    
+    LOG("Init Collection");
+    MTGAllCards::loadInstance();
+
+    LOG("Loading rules");
+    Rules::loadAllRules();
 
-    filepath = JGE_GET_RES(WResourceManager::Instance()->musicFile("Track1.mp3"));
-    wagic::ifstream file2(filepath.c_str());
-    if (file2)
-        file2.close();
-    else
-        HasMusic = 0;
 
     LOG("Loading Textures");
     LOG("--Loading menuicons.png");
     WResourceManager::Instance()->RetrieveTexture("menuicons.png", RETRIEVE_MANAGE);
     LOG("---Gettings menuicons.png quads");
-    //Creating thes quad in this specific order allows us to have them in the correct order to call them by integer id
-    manaIcons[Constants::MTG_COLOR_GREEN] = WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + 0 * 36, 38, 32, 32, "c_green",
-                    RETRIEVE_MANAGE);
-    manaIcons[Constants::MTG_COLOR_BLUE] = WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + 1 * 36, 38, 32, 32, "c_blue",
-                    RETRIEVE_MANAGE);
-    manaIcons[Constants::MTG_COLOR_RED] = WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + 3 * 36, 38, 32, 32, "c_red", RETRIEVE_MANAGE);
-    manaIcons[Constants::MTG_COLOR_BLACK] = WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + 2 * 36, 38, 32, 32, "c_black",
-                    RETRIEVE_MANAGE);
-    manaIcons[Constants::MTG_COLOR_WHITE] = WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + 4 * 36, 38, 32, 32, "c_white",
-                    RETRIEVE_MANAGE);
-    manaIcons[Constants::MTG_COLOR_LAND] = WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + 5 * 36, 38, 32, 32, "c_land",
-                    RETRIEVE_MANAGE);
-    manaIcons[Constants::MTG_COLOR_ARTIFACT] = WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + 6 * 36, 38, 32, 32, "c_artifact",
-                    RETRIEVE_MANAGE);
+   
+    //Load all icons from gModRules and save in manaIcons -> todo. Change the icons positions on menuicons.png to avoid use item->mColorId
+    vector<ModRulesBackGroundCardGuiItem *>items = gModRules.cardgui.background;
+    for(size_t i= 0; i < items.size(); i ++)
+    {
+        
+        ModRulesBackGroundCardGuiItem * item = items[i];
+        if (item->mMenuIcon == 1)
+        {
+            manaIcons.push_back(WResourceManager::Instance()->RetrieveQuad("menuicons.png", 2 + (float)item->mColorId * 36, 38, 32, 32, "c_" + item->MColorName, RETRIEVE_MANAGE));
+            Constants::MTGColorStrings.push_back(item->MColorName.c_str());
+        }
+    }
+    Constants::NB_Colors = (int)Constants::MTGColorStrings.size();
+    
+    items.erase(items.begin(),items.end());
+    
 
-    for (int i = sizeof(manaIcons) / sizeof(manaIcons[0]) - 1; i >= 0; --i)
+    for (int i = manaIcons.size()-1; i >= 0; --i)
         if (manaIcons[i].get())
             manaIcons[i]->SetHotSpot(16, 16);
 
@@ -193,37 +221,28 @@ void GameApp::Create()
     // The translator is ready now.
 
     LOG("--Loading various textures");
+    // Load in this function only textures that are used frequently throughout the game. These textures will constantly stay in Ram, so be frugal
     WResourceManager::Instance()->RetrieveTexture("phasebar.png", RETRIEVE_MANAGE);
     WResourceManager::Instance()->RetrieveTexture("wood.png", RETRIEVE_MANAGE);
     WResourceManager::Instance()->RetrieveTexture("gold.png", RETRIEVE_MANAGE);
     WResourceManager::Instance()->RetrieveTexture("goldglow.png", RETRIEVE_MANAGE);
     WResourceManager::Instance()->RetrieveTexture("backdrop.jpg", RETRIEVE_MANAGE);
     WResourceManager::Instance()->RetrieveTexture("handback.png", RETRIEVE_MANAGE);
-    WResourceManager::Instance()->RetrieveTexture("BattleIcon.png", RETRIEVE_MANAGE);
-    WResourceManager::Instance()->RetrieveTexture("DefenderIcon.png", RETRIEVE_MANAGE);
-    WResourceManager::Instance()->RetrieveTexture("shadow.png", RETRIEVE_MANAGE);
-    WResourceManager::Instance()->RetrieveTexture("extracostshadow.png", RETRIEVE_MANAGE);
-    WResourceManager::Instance()->RetrieveTexture("morph.jpg", RETRIEVE_MANAGE);
+    WResourceManager::Instance()->RetrieveTexture("shadows.png", RETRIEVE_MANAGE);
 
-    jq = WResourceManager::Instance()->RetrieveQuad("BattleIcon.png", 0, 0, 25, 25, "BattleIcon", RETRIEVE_MANAGE);
-    if (jq)
-        jq->SetHotSpot(12, 12);
-    jq = WResourceManager::Instance()->RetrieveQuad("DefenderIcon.png", 0, 0, 24, 23, "DefenderIcon", RETRIEVE_MANAGE);
-    if (jq)
-        jq->SetHotSpot(12, 12);
-    jq = WResourceManager::Instance()->RetrieveQuad("shadow.png", 0, 0, 16, 16, "shadow", RETRIEVE_MANAGE);
+    jq = WResourceManager::Instance()->RetrieveQuad("shadows.png", 2, 2, 16, 16, "white", RETRIEVE_MANAGE);
     if (jq)
         jq->SetHotSpot(8, 8);
-    jq = WResourceManager::Instance()->RetrieveQuad("extracostshadow.png", 0, 0, 16, 16, "extracostshadow", RETRIEVE_MANAGE);
+    jq = WResourceManager::Instance()->RetrieveQuad("shadows.png", 20, 2, 16, 16, "shadow", RETRIEVE_MANAGE);
     if (jq)
         jq->SetHotSpot(8, 8);
-    jq = WResourceManager::Instance()->RetrieveQuad("morph.jpg", 0, 0, MTG_MINIIMAGE_WIDTH, MTG_MINIIMAGE_HEIGHT, "morph", RETRIEVE_MANAGE);
+    jq = WResourceManager::Instance()->RetrieveQuad("shadows.png", 38, 2, 16, 16, "extracostshadow", RETRIEVE_MANAGE);
     if (jq)
-        jq->SetHotSpot(static_cast<float> (jq->mTex->mWidth / 2), static_cast<float> (jq->mTex->mHeight / 2));
+        jq->SetHotSpot(8, 8);
+
     jq = WResourceManager::Instance()->RetrieveQuad("phasebar.png", 0, 0, 0, 0, "phasebar", RETRIEVE_MANAGE);
 
-    LOG("Init Collection");
-    MTGAllCards::loadInstance();
+
 
     LOG("Creating Game States");
     mGameStates[GAME_STATE_DECK_VIEWER] = NEW GameStateDeckViewer(this);
@@ -252,9 +271,6 @@ void GameApp::Create()
     mCurrentState = NULL;
     mNextState = mGameStates[GAME_STATE_MENU];
 
-    LOG("--Load Game rules");
-    Rules::loadAllRules();
-
     //Set Audio volume
     JSoundSystem::GetInstance()->SetSfxVolume(options[Options::SFXVOLUME].number);
     JSoundSystem::GetInstance()->SetMusicVolume(options[Options::MUSICVOLUME].number);
@@ -295,9 +311,7 @@ void GameApp::Destroy()
     DeckManager::EndInstance();
     DeckStats::EndInstance();
 
-    SAFE_DELETE(Subtypes::subtypesList);
-
-    playMusic("none");
+    stopMusic();
 
     Translator::EndInstance();
     WCFilterFactory::Destroy();
@@ -306,6 +320,9 @@ void GameApp::Destroy()
     DeckEditorMenu::destroy();
 
     options.theGame = NULL;
+    Unlockable::Destroy();
+
+    AutoLineMacro::Destroy();
 
     Rules::unloadAllRules();
     LOG("==Destroying GameApp Successful==");
@@ -321,6 +338,7 @@ void GameApp::Update()
     if (systemError.size())
         return;
     JGE* mEngine = JGE::GetInstance();
+#if defined (PSP)
     if (mEngine->GetButtonState(JGE_BTN_MENU) && mEngine->GetButtonClick(JGE_BTN_CANCEL))
     {
         char s[80];
@@ -337,6 +355,7 @@ void GameApp::Update()
     //Restart Rendering engine when START and SQUARE ARE PRESSED SIMULTANEOUSLY
     if (mEngine->GetButtonState(JGE_BTN_MENU) && mEngine->GetButtonState(JGE_BTN_PRI))
         JRenderer::Destroy();
+#endif //PSP
 
     float dt = mEngine->GetDelta();
     if (dt > 35.0f) // min 30 FPS ;)
@@ -355,13 +374,13 @@ void GameApp::Update()
         mTrans->End();
         if (mTrans->to != NULL && !mTrans->bAnimationOnly)
         {
-            mCurrentState = mTrans->to;
+            SetCurrentState(mTrans->to);
             SAFE_DELETE(mGameStates[GAME_STATE_TRANSITION]);
             mCurrentState->Start();
         }
         else
         {
-            mCurrentState = mTrans->from;
+            SetCurrentState(mTrans->from);
             SAFE_DELETE(mGameStates[GAME_STATE_TRANSITION]);
         }
     }
@@ -370,7 +389,7 @@ void GameApp::Update()
         if (mCurrentState != NULL)
             mCurrentState->End();
 
-        mCurrentState = mNextState;
+        SetCurrentState(mNextState);
 
 #if defined (PSP)
         /*
@@ -407,7 +426,7 @@ void GameApp::Render()
     WResourceManager::Instance()->DebugRender();
 #endif
 
-#ifdef DEBUG
+#if defined(DEBUG) && !defined(IOS)
     JGE* mEngine = JGE::GetInstance();
     float fps = mEngine->GetFPS();
     totalFPS += fps;
@@ -418,7 +437,7 @@ void GameApp::Render()
     if (mFont)
     {
         mFont->SetColor(ARGB(255,255,255,255));
-        mFont->DrawString(buf, 10, SCREEN_HEIGHT-15);
+        mFont->DrawString(buf, 10, SCREEN_HEIGHT-25);
     }
 #endif
 
@@ -437,14 +456,28 @@ void GameApp::SetNextState(int state)
     mNextState = mGameStates[state];
 }
 
+void GameApp::SetCurrentState(GameState * state)
+{
+    if (mCurrentState == state)
+        return;
+
+    if (mCurrentState)
+        JGE::GetInstance()->SendCommand("leavegamestate:" + mCurrentState->getStringID());
+
+    mCurrentState = state;
+
+    if (mCurrentState)
+        JGE::GetInstance()->SendCommand("entergamestate:" + mCurrentState->getStringID());
+}
+
 void GameApp::Pause()
 {
-
+    pauseMusic();
 }
 
 void GameApp::Resume()
 {
-
+    resumeMusic();
 }
 
 void GameApp::DoTransition(int trans, int tostate, float dur, bool animonly)
@@ -485,7 +518,7 @@ void GameApp::DoTransition(int trans, int tostate, float dur, bool animonly)
         tb->bAnimationOnly = animonly;
         mGameStates[GAME_STATE_TRANSITION] = tb;
         mGameStates[GAME_STATE_TRANSITION]->Start();
-        mCurrentState = tb; //The old current state is ended inside our transition.
+        SetCurrentState(tb); //The old current state is ended inside our transition.
     }
     else if (toState)
     { //Somehow failed, just do standard SetNextState behavior
@@ -500,7 +533,9 @@ void GameApp::DoAnimation(int trans, float dur)
 
 void GameApp::playMusic(string filename, bool loop)
 {
-    if (filename.compare(currentMusicFile) == 0)
+    if(filename == "") filename = currentMusicFile;
+
+    if (filename.compare(currentMusicFile) == 0 && music)
         return;
 
     if (music)
@@ -517,3 +552,30 @@ void GameApp::playMusic(string filename, bool loop)
         currentMusicFile = filename;
     }
 }
+
+void GameApp::pauseMusic()
+{
+    if (music && currentMusicFile != "")
+    {
+        JSoundSystem::GetInstance()->PauseMusic(music);
+    }
+}
+
+
+
+void GameApp::resumeMusic()
+{
+    if (music && currentMusicFile != "")
+    {
+        JSoundSystem::GetInstance()->ResumeMusic(music);
+    }
+}
+
+void GameApp::stopMusic()
+{
+    if (music && currentMusicFile != "")
+    {
+        JSoundSystem::GetInstance()->StopMusic(music);
+        SAFE_DELETE(music);
+    }
+}
