@@ -7,31 +7,30 @@
 // Copyright (c) 2007 James Hui (a.k.a. Dr.Watson) <jhkhui@gmail.com>
 //
 //-------------------------------------------------------------------------------------
-#include "../include/config.h"
-#include <string.h>
+#include "PrecompiledHeader.h"
+
 #include <JGameLauncher.h>
 
-#include "../include/GameApp.h"
+#include "GameApp.h"
 
 static char GameName[] = "Wagic";
 
 //-------------------------------------------------------------------------------------
 JApp* JGameLauncher::GetGameApp()
 {
-  return NEW GameApp();
-};
-
+    return NEW GameApp();
+}
+;
 
 //-------------------------------------------------------------------------------------
 char *JGameLauncher::GetName()
 {
-  return GameName;
+    return GameName;
 }
 
-
 //-------------------------------------------------------------------------------------
 u32 JGameLauncher::GetInitFlags()
 {
-  return JINIT_FLAG_NORMAL;
+    return JINIT_FLAG_NORMAL;
 }
 
