@@ -5,6 +5,7 @@
 #include "Translate.h"
 #include "OptionItem.h"
 #include "StyleManager.h"
+#include "Credits.h"
 
 const string Options::optionNames[] = {
 //Global options
@@ -65,18 +66,17 @@ const string Options::optionNames[] = {
   "interruptAfterEnd",
 //Unlocked modes
   "prx_handler",
-  "prx_rimom",
-  "prx_rewehenots",
-  "prx_timreh",
   "prx_eviltwin",
   "prx_rnddeck",
   "aw_collector",
 
 };
+
+// MARK:  Options
 int Options::getID(string name)
 {
     if (0 == name.size())
-        INVALID_OPTION;
+        return INVALID_OPTION;
 
     std::transform(name.begin(), name.end(), name.begin(), ::tolower);
 
@@ -143,52 +143,56 @@ int Options::optionInterrupt(int gamePhase)
     //Huge, nearly illegible switch block spread out to improve readability.
     switch (gamePhase)
     {
-    case Constants::MTG_PHASE_BEFORE_BEGIN:
+    case MTG_PHASE_BEFORE_BEGIN:
         return INTERRUPT_BEFOREBEGIN;
 
-    case Constants::MTG_PHASE_UNTAP:
+    case MTG_PHASE_UNTAP:
         return INTERRUPT_UNTAP;
 
-    case Constants::MTG_PHASE_UPKEEP:
+    case MTG_PHASE_UPKEEP:
         return INTERRUPT_UPKEEP;
 
-    case Constants::MTG_PHASE_DRAW:
+    case MTG_PHASE_DRAW:
         return INTERRUPT_DRAW;
 
-    case Constants::MTG_PHASE_FIRSTMAIN:
+    case MTG_PHASE_FIRSTMAIN:
         return INTERRUPT_FIRSTMAIN;
 
-    case Constants::MTG_PHASE_COMBATBEGIN:
+    case MTG_PHASE_COMBATBEGIN:
         return INTERRUPT_BEGINCOMBAT;
 
-    case Constants::MTG_PHASE_COMBATATTACKERS:
+    case MTG_PHASE_COMBATATTACKERS:
         return INTERRUPT_ATTACKERS;
 
-    case Constants::MTG_PHASE_COMBATBLOCKERS:
+    case MTG_PHASE_COMBATBLOCKERS:
         return INTERRUPT_BLOCKERS;
 
-    case Constants::MTG_PHASE_COMBATDAMAGE:
+    case MTG_PHASE_COMBATDAMAGE:
         return INTERRUPT_DAMAGE;
 
-    case Constants::MTG_PHASE_COMBATEND:
+    case MTG_PHASE_COMBATEND:
         return INTERRUPT_ENDCOMBAT;
 
-    case Constants::MTG_PHASE_SECONDMAIN:
+    case MTG_PHASE_SECONDMAIN:
         return INTERRUPT_SECONDMAIN;
 
-    case Constants::MTG_PHASE_ENDOFTURN:
+    case MTG_PHASE_ENDOFTURN:
         return INTERRUPT_ENDTURN;
 
-    case Constants::MTG_PHASE_CLEANUP:
+    case MTG_PHASE_CLEANUP:
         return INTERRUPT_CLEANUP;
 
-    case Constants::MTG_PHASE_AFTER_EOT:
+    case MTG_PHASE_AFTER_EOT:
         return INTERRUPT_AFTEREND;
     }
 
     return INVALID_OPTION;
 }
 
+// MARK:  -
+
+// MARK:  GameOption
+
 GameOption::GameOption(int value) :
     number(value)
 {
@@ -247,7 +251,8 @@ PIXEL_TYPE GameOption::asColor(PIXEL_TYPE fallback)
         color[subpixel] = (unsigned char) atoi(temp.c_str());
     if (subpixel == 2)
         color[3] = 255;
-
+    
+    // TODO: WARNING - not all the values of color may be initialized at this point.  Do we need to initialize the array to some value?
     return ARGB(color[3],color[0],color[1],color[2]);
 }
 
@@ -309,6 +314,9 @@ bool GameOption::write(std::ofstream * file, string name)
     return true;
 }
 
+// MARK:  -
+
+// MARK:  GameOptions
 GameOptions::GameOptions(string filename)
 {
     mFilename = filename;
@@ -317,12 +325,13 @@ GameOptions::GameOptions(string filename)
 
 int GameOptions::load()
 {
-    wagic::ifstream file(mFilename.c_str());
-    std::string s;
-
-    if (file)
+    std::string contents;
+    if (JFileSystem::GetInstance()->readIntoString(mFilename, contents))
     {
-        while (std::getline(file, s))
+        std::stringstream stream(contents);
+        string s;
+
+        while (std::getline(stream, s))
         {
             if (!s.size())
                 continue;
@@ -334,13 +343,12 @@ int GameOptions::load()
             int id = Options::getID(name);
             if (id == INVALID_OPTION)
             {
-                if (!unknownMap[name]) unknownMap[name] = NEW GameOption(val);
+                if (!unknownMap[name]) unknownMap[name] = factorNewGameOption(name, val);
                 continue;
             }
 
             (*this)[id].read(val);
         }
-        file.close();
     }
     // (PSY) Make sure that cheatmode is switched off for ineligible profiles:
     if (options[Options::ACTIVE_PROFILE].str != SECRET_PROFILE)
@@ -369,8 +377,8 @@ int GameOptions::save()
 				(*this)[Options::CHEATMODEAIDECK].number = 0;
 		}
 
-    std::ofstream file(mFilename.c_str());
-    if (file)
+    std::ofstream file;
+    if (JFileSystem::GetInstance()->openForWrite(file, mFilename))
     {
         for (int x = 0; x < (int) values.size(); x++)
         {
@@ -399,6 +407,7 @@ int GameOptions::save()
     return 1;
 }
 
+
 GameOption& GameOptions::operator[](int optionID)
 {
     GameOption * go = get(optionID);
@@ -420,12 +429,25 @@ GameOption& GameOptions::operator[](string optionName)
 
 }
 
+GameOption * GameOptions::factorNewGameOption(string optionName, string value)
+{
+    GameOption * result =( Unlockable::unlockables.find(optionName) !=  Unlockable::unlockables.end())
+        ? NEW GameOptionAward()
+        : NEW GameOption();
+
+    if (value.size())
+        result->read(value);
+
+    return result;
+}
+
 GameOption * GameOptions::get(string optionName)
 {
    if (!unknownMap[optionName])
-        unknownMap[optionName] = NEW GameOption(0);
-
-    return unknownMap[optionName];
+   {
+       unknownMap[optionName] = factorNewGameOption(optionName);
+   }
+   return unknownMap[optionName];
 }
 
 GameOption * GameOptions::get(int optionID)
@@ -513,6 +535,10 @@ GameOptions::~GameOptions()
     unknownMap.clear();
 }
 
+// MARK:  - 
+
+// MARK:  GameSettings
+
 GameSettings options;
 
 GameSettings::GameSettings()
@@ -619,10 +645,6 @@ GameOption& GameSettings::operator[](string optionName)
 
 GameOption* GameSettings::get(int optionID)
 {
-#ifdef DEBUG
-    string option_name = Options::getName(optionID);
-#endif
-
     if (optionID < 0)
         return &invalid_option;
     else if (globalOptions && optionID <= Options::LAST_GLOBAL)
@@ -633,30 +655,33 @@ GameOption* GameSettings::get(int optionID)
     return &invalid_option;
 }
 
+void GameSettings::createProfileFolders()
+{
+     if (!profileOptions)
+         return;
+
+    string temp = profileFile("", "", false);
+    JFileSystem::GetInstance()->MakeDir(temp);
+    temp += "/stats";
+    JFileSystem::GetInstance()->MakeDir(temp);
+    temp = profileFile(PLAYER_SETTINGS, "", false);
+        
+    profileOptions->save();
+}
+
 int GameSettings::save()
 {
     if (globalOptions)
         globalOptions->save();
 
-    if (profileOptions)
-    {
-        //Force our directories to exist.
-        MAKEDIR(JGE_GET_RES("profiles").c_str());
-        string temp = profileFile("", "", false, false);
-        MAKEDIR(temp.c_str());
-        temp += "/stats";
-        MAKEDIR(temp.c_str());
-        temp = profileFile(PLAYER_SETTINGS, "", false);
-
-        profileOptions->save();
-    }
+    createProfileFolders();
 
     checkProfile();
 
     return 1;
 }
 
-string GameSettings::profileFile(string filename, string fallback, bool sanity, bool relative)
+string GameSettings::profileFile(string filename, string fallback, bool sanity)
 {
     char buf[512];
     string profile = (*this)[Options::ACTIVE_PROFILE].str;
@@ -666,30 +691,27 @@ string GameSettings::profileFile(string filename, string fallback, bool sanity,
         //No file, return root of profile directory
         if (filename == "")
         {
-            sprintf(buf, "%sprofiles/%s", (relative ? "" : JGE_GET_RES("").c_str()), profile.c_str());
+            sprintf(buf, "profiles/%s", profile.c_str());
             return buf;
         }
         //Return file
-        sprintf(buf, JGE_GET_RES("profiles/%s/%s").c_str(), profile.c_str(), filename.c_str());
+        sprintf(buf, "profiles/%s/%s", profile.c_str(), filename.c_str());
         if (fileExists(buf))
         {
-            if (relative)
-                sprintf(buf, "profiles/%s/%s", profile.c_str(), filename.c_str());
             return buf;
         }
     }
     else
     {
         //Use the default directory.
-        sprintf(buf, "%splayer%s%s", (relative ? "" : JGE_GET_RES("").c_str()), (filename == "" ? "" : "/"), filename.c_str());
+        sprintf(buf, "player%s%s", (filename == "" ? "" : "/"), filename.c_str());
         return buf;
     }
 
     //Don't fallback if sanity checking is disabled..
     if (!sanity)
     {
-        sprintf(buf, "%sprofiles/%s%s%s", (relative ? "" : JGE_GET_RES("").c_str()), profile.c_str(), (filename == "" ? "" : "/"),
-                        filename.c_str());
+        sprintf(buf, "profiles/%s%s%s", profile.c_str(), (filename == "" ? "" : "/"), filename.c_str());
         return buf;
     }
 
@@ -697,8 +719,7 @@ string GameSettings::profileFile(string filename, string fallback, bool sanity,
     if (fallback == "")
         return "";
 
-    sprintf(buf, "%s%s%s%s", (relative ? "" : JGE_GET_RES("").c_str()), fallback.c_str(), (filename == "" ? "" : "/"),
-                    filename.c_str());
+    sprintf(buf, "%s%s%s", fallback.c_str(), (filename == "" ? "" : "/"), filename.c_str());
     return buf;
 }
 
@@ -711,13 +732,13 @@ void GameSettings::reloadProfile()
 void GameSettings::checkProfile()
 {
     if (!globalOptions)
-        globalOptions = NEW GameOptions(JGE_GET_RES(GLOBAL_SETTINGS));
+        globalOptions = NEW GameOptions(GLOBAL_SETTINGS);
 
     //If it doesn't exist, load current profile.
     if (!profileOptions)
     {
         profileOptions = NEW GameOptions(profileFile(PLAYER_SETTINGS, "", false));
-        //Backwards compatability hack for unlocked modes.
+        //Backwards compatibility hack for unlocked modes.
         for (int x = Options::BEGIN_AWARDS; x < Options::LAST_NAMED; x++)
         {
             GameOptionAward * goa = dynamic_cast<GameOptionAward *> (globalOptions->get(x));
@@ -739,20 +760,8 @@ void GameSettings::checkProfile()
     {
         //If we had any default settings, we'd set them here.
 
-
-        //Make the proper directories
-        if (profileOptions)
-        {
-            //Force our directories to exist.
-            MAKEDIR(JGE_GET_RES("profiles").c_str());
-            string temp = profileFile("", "", false, false);
-            MAKEDIR(temp.c_str());
-            temp += "/stats";
-            MAKEDIR(temp.c_str());
-            temp = profileFile(PLAYER_SETTINGS, "", false);
-
-            profileOptions->save();
-        }
+        //Create proper directories
+        createProfileFolders();
     }
 
     //Find the set for which we have the most variety
@@ -834,6 +843,14 @@ SimplePad * GameSettings::keypadStart(string input, string * _dest, bool _cancel
 {
     if (keypad == NULL)
         keypad = NEW SimplePad();
+    // show keyboard
+#ifdef IOS
+    JGE *engine = JGE::GetInstance();
+    engine->SendCommand( "displayKeyboard", input);
+#elif ANDROID
+    JGE *engine = JGE::GetInstance();
+    engine->SendCommand( "displayKeyboard:" + input);    
+#endif
     keypad->bShowCancel = _cancel;
     keypad->bShowNumpad = _numpad;
     keypad->mX = _x;
@@ -854,6 +871,10 @@ void GameSettings::keypadShutdown()
     SAFE_DELETE(keypad);
 }
 
+// MARK:  - 
+
+// MARK:  EnumDefinition
+
 //EnumDefinition
 int EnumDefinition::findIndex(int value)
 {
@@ -914,6 +935,10 @@ bool GameOptionEnum::read(string input)
     return false;
 }
 
+// MARK:  - 
+
+// MARK:  OptionMaxGrade
+
 //Enum Definitions
 OptionMaxGrade OptionMaxGrade::mDef;
 OptionMaxGrade::OptionMaxGrade()
@@ -927,6 +952,10 @@ OptionMaxGrade::OptionMaxGrade()
 
 }
 ;
+// MARK:  - 
+
+// MARK:  OptionASkipPhase
+
 OptionASkipPhase OptionASkipPhase::mDef;
 OptionASkipPhase::OptionASkipPhase()
 {
@@ -935,6 +964,9 @@ OptionASkipPhase::OptionASkipPhase()
     mDef.values.push_back(EnumDefinition::assoc(Constants::ASKIP_FULL, "Full"));
 }
 ;
+// MARK:  - 
+
+// MARK:  OptionWhosFirst
 
 OptionWhosFirst OptionWhosFirst::mDef;
 OptionWhosFirst::OptionWhosFirst()
@@ -1002,6 +1034,10 @@ OptionKicker::OptionKicker()
     mDef.values.push_back(EnumDefinition::assoc(Constants::KICKER_CHOICE, "Offer Choice"));
 }
 ;
+// MARK:  - 
+
+// MARK:  GameOptionAward
+
 //GameOptionAward
 GameOptionAward::GameOptionAward()
 {
@@ -1137,6 +1173,10 @@ static JButton u32_to_button(u32 b)
         return JGE_BTN_NONE;
 }
 
+// MARK:  - 
+
+// MARK:  GameOptionKeyBindings
+
 bool GameOptionKeyBindings::read(string input)
 {
     istringstream iss(input);
@@ -1184,3 +1224,4 @@ bool GameOptionKeyBindings::write(std::ofstream* file, string name)
     *file << endl;
     return true;
 }
+// MARK:  -
