@@ -20,6 +20,7 @@
 #include "DeckEditorMenu.h"
 #include "SimpleMenu.h"
 #include "utils.h"
+#include "AIPlayer.h"
 
 
 
@@ -39,7 +40,7 @@ void StringExplode(string str, string separator, vector<string>* results)
 }
 
 GameStateDeckViewer::GameStateDeckViewer(GameApp* parent) :
-    GameState(parent)
+    GameState(parent, "deckeditor")
 {
     bgMusic = NULL;
     useFilter = 0;
@@ -57,11 +58,21 @@ GameStateDeckViewer::GameStateDeckViewer(GameApp* parent) :
     mAlpha = 255;
     menu = NULL;
     stw = NULL;
+    
+    statsPrevButton = NEW InteractiveButton(NULL, kPrevStatsButtonId, Fonts::MAIN_FONT, "Stats",  SCREEN_WIDTH_F - 50, SCREEN_HEIGHT_F - 20, JGE_BTN_PREV);
+    toggleDeckButton = NEW InteractiveButton(NULL, kToggleDeckActionId, Fonts::MAIN_FONT, "View Deck", 10, SCREEN_HEIGHT_F - 20, JGE_BTN_PRI);
+    sellCardButton = NEW InteractiveButton(NULL, kSellCardActionId, Fonts::MAIN_FONT, "Sell Card", (SCREEN_WIDTH_F/ 2) - 100, SCREEN_HEIGHT_F - 20, JGE_BTN_SEC);
+    filterButton = NEW InteractiveButton(NULL, kFilterButtonId, Fonts::MAIN_FONT, "filter", (SCREEN_WIDTH_F - 110), SCREEN_HEIGHT_F - 20, JGE_BTN_CTRL);
 }
 
 GameStateDeckViewer::~GameStateDeckViewer()
 {
     SAFE_DELETE(bgMusic);
+    SAFE_DELETE(toggleDeckButton);
+    SAFE_DELETE(sellCardButton);
+    SAFE_DELETE(statsPrevButton);
+    SAFE_DELETE(filterButton);
+    
     if (myDeck)
     {
         SAFE_DELETE(myDeck->parent);
@@ -152,9 +163,11 @@ void GameStateDeckViewer::switchDisplay()
     if (displayed_deck == myCollection)
     {
         displayed_deck = myDeck;
+        toggleDeckButton->setText("View Collection");
     }
     else
     {
+        toggleDeckButton->setText("View Deck");
         displayed_deck = myCollection;
     }
     source->swapSrc();
@@ -218,21 +231,15 @@ void GameStateDeckViewer::Start()
     lastPos = 0;
     lastTotal = 0;
 
-    pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), MTGCollection());
+    pricelist = NEW PriceList("settings/prices.dat", MTGCollection());
     playerdata = NEW PlayerData(MTGCollection());
     myCollection = NEW DeckDataWrapper(playerdata->collection);
     myCollection->Sort(WSrcCards::SORT_ALPHA);
     displayed_deck = myCollection;
 
     //Icons
-    mIcons[Constants::MTG_COLOR_ARTIFACT] = WResourceManager::Instance()->GetQuad("c_artifact");
-    mIcons[Constants::MTG_COLOR_LAND] = WResourceManager::Instance()->GetQuad("c_land");
-    mIcons[Constants::MTG_COLOR_WHITE] = WResourceManager::Instance()->GetQuad("c_white");
-    mIcons[Constants::MTG_COLOR_RED] = WResourceManager::Instance()->GetQuad("c_red");
-    mIcons[Constants::MTG_COLOR_BLACK] = WResourceManager::Instance()->GetQuad("c_black");
-    mIcons[Constants::MTG_COLOR_BLUE] = WResourceManager::Instance()->GetQuad("c_blue");
-    mIcons[Constants::MTG_COLOR_GREEN] = WResourceManager::Instance()->GetQuad("c_green");
-    for (int i = 0; i < 7; i++)
+    mIcons = manaIcons;
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         mIcons[i]->SetHotSpot(16, 16);
     }
@@ -247,11 +254,15 @@ void GameStateDeckViewer::Start()
         pspIcons[i] = WResourceManager::Instance()->RetrieveQuad("iconspsp.png", (float) i * 32, 0, 32, 32, buf);
         pspIcons[i]->SetHotSpot(16, 16);
     }
-
+#ifndef TOUCH_ENABLED
+    toggleDeckButton->setImage( pspIcons[6] );
+    sellCardButton->setImage( pspIcons[7] );
+#endif
+    
     //init welcome menu
     updateDecks();
 
-    GameApp::playMusic("track1.mp3");
+    GameApp::playMusic("Track1.mp3");
 
     loadIndexes();
     mEngine->ResetInput();
@@ -321,24 +332,70 @@ void GameStateDeckViewer::saveDeck()
 void GameStateDeckViewer::saveAsAIDeck(string deckName)
 {
 
-    vector<DeckMetaData *> aiDecks = GameState::BuildDeckList(JGE_GET_RES("ai/baka"), "ai_baka", NULL);
-    int nbAiDecks = aiDecks.size() + 1;
-    aiDecks.clear();
+    int deckId = AIPlayer::getTotalAIDecks() + 1;
 
-    string defaultAiDeckName = "deck";
     std::ostringstream oss;
-    oss << "deck" << nbAiDecks;
-    defaultAiDeckName = oss.str();
+    oss << "deck" <<deckId;
+    string aiDeckName = oss.str();
     oss.str("");
     if (myDeck->parent->meta_desc == "")
-        oss << endl << "Can you beat your own creations?" << endl << "User created AI Deck # " << nbAiDecks;
+        oss << endl << "Can you beat your own creations?" << endl << "User created AI Deck # " << deckId;
     else
         oss << myDeck->parent->meta_desc;
     string deckDesc = oss.str();
-    string filepath = JGE_GET_RES("ai/baka/");
-    filepath.append(defaultAiDeckName).append(".txt");
+    string filepath = "ai/baka/";
+    filepath.append(aiDeckName).append(".txt");
     DebugTrace("saving AI deck " << filepath);
     myDeck->save(filepath, true, deckName, deckDesc);
+    AIPlayer::invalidateTotalAIDecks(); //We added one AI deck, so we need to invalidate the count cache
+}
+
+void GameStateDeckViewer::sellCard()
+{
+    last_user_activity = 0;
+    SAFE_DELETE(subMenu);
+    char buffer[4096];
+    {
+        MTGCard * card = cardIndex[2];
+        if (card && displayed_deck->count(card))
+        {
+            price = pricelist->getSellPrice(card->getMTGId());
+            sprintf(buffer, "%s : %i %s", _(card->data->getName()).c_str(), price, _("credits").c_str());
+            const float menuXOffset = SCREEN_WIDTH_F - 300;
+            const float menuYOffset = SCREEN_HEIGHT_F / 2;
+            subMenu = NEW SimpleMenu(JGE::GetInstance(), MENU_CARD_PURCHASE, this, Fonts::MAIN_FONT, menuXOffset, menuYOffset, buffer);
+            subMenu->Add(MENU_ITEM_YES, "Yes");
+            subMenu->Add(MENU_ITEM_NO, "No", "", true);
+        }
+    }
+    stw->needUpdate = true;
+}
+
+bool GameStateDeckViewer::userPressedButton()
+{
+    return (
+            (toggleDeckButton->ButtonPressed()) 
+            || (sellCardButton->ButtonPressed())
+            || (statsPrevButton->ButtonPressed())
+            || (filterButton->ButtonPressed())
+            );
+  }
+
+void GameStateDeckViewer::setButtonState(bool state)
+{
+    toggleDeckButton->setIsSelectionValid(state);
+    sellCardButton->setIsSelectionValid(state);
+    statsPrevButton->setIsSelectionValid(state);
+    filterButton->setIsSelectionValid(state);
+    
+}
+
+void GameStateDeckViewer::RenderButtons()
+{
+    toggleDeckButton->Render();
+    sellCardButton->Render();
+    filterButton->Render();
+    statsPrevButton->Render();
 }
 
 void GameStateDeckViewer::Update(float dt)
@@ -348,7 +405,7 @@ void GameStateDeckViewer::Update(float dt)
     unsigned int distance2;
     unsigned int minDistance2 = -1;
     int n = 0;
-
+    
     if (options.keypadActive())
     {
         options.keypadUpdate(dt);
@@ -423,27 +480,43 @@ void GameStateDeckViewer::Update(float dt)
             }
             break;
         case JGE_BTN_OK:
-            last_user_activity = 0;
-            addRemove(cardIndex[2]);
+            if (mEngine->GetLeftClickCoordinates(x, y))
+            {
+                // verify that none of the buttons fired
+                if (userPressedButton())
+                {
+                    Update(dt);
+                    break;
+                }
+
+                for(int i=0; i < CARDS_DISPLAYED; i++)
+                {
+                    distance2 = static_cast<unsigned int>((cardsCoordinates[i].second - y) * (cardsCoordinates[i].second - y) + (cardsCoordinates[i].first - x) * (cardsCoordinates[i].first - x));
+                    if (distance2 < minDistance2)
+                    {
+                        minDistance2 = distance2;
+                        n = i;
+                    }
+                }
+
+                if(n != 2) 
+                {
+                    mSelected = n;
+                    last_user_activity = 0;
+                    mStage = STAGE_TRANSITION_SELECTED;
+                }
+                mEngine->LeftClickedProcessed();
+            }
+                
+            if(mStage != STAGE_TRANSITION_SELECTED && last_user_activity > .05)
+            {
+                last_user_activity = 0;
+                addRemove(cardIndex[2]);
+            }
+                
             break;
         case JGE_BTN_SEC:
-            last_user_activity = 0;
-            SAFE_DELETE(subMenu);
-            char buffer[4096];
-            {
-                MTGCard * card = cardIndex[2];
-                if (card && displayed_deck->count(card))
-                {
-                    price = pricelist->getSellPrice(card->getMTGId());
-                    sprintf(buffer, "%s : %i %s", _(card->data->getName()).c_str(), price, _("credits").c_str());
-                    const float menuXOffset = SCREEN_WIDTH_F - 300;
-                    const float menuYOffset = SCREEN_HEIGHT_F / 2;
-                    subMenu = NEW SimpleMenu(MENU_CARD_PURCHASE, this, Fonts::MAIN_FONT, menuXOffset, menuYOffset, buffer);
-                    subMenu->Add(MENU_ITEM_YES, "Yes");
-                    subMenu->Add(MENU_ITEM_NO, "No", "", true);
-                }
-            }
-            stw->needUpdate = true;
+            sellCard();
             break;
 
         case JGE_BTN_MENU:
@@ -474,32 +547,6 @@ void GameStateDeckViewer::Update(float dt)
             else if ((mStage == STAGE_ONSCREEN_MENU) && (++stw->currentPage > stw->pageCount)) stw->currentPage = 0;
             break;
         default: // no keypress
-            if (mEngine->GetLeftClickCoordinates(x, y))
-            {
-              for(int i=0; i < CARDS_DISPLAYED; i++)
-              {
-                distance2 = static_cast<unsigned int>((cardsCoordinates[i].second - y) * (cardsCoordinates[i].second - y) + (cardsCoordinates[i].first - x) * (cardsCoordinates[i].first - x));
-                if (distance2 < minDistance2)
-                {
-                    minDistance2 = distance2;
-                    n = i;
-                }
-              }
-
-              if (n < ((CARDS_DISPLAYED/2) - 1))
-              {
-                  last_user_activity = 0;
-                  mStage = STAGE_TRANSITION_RIGHT;
-              }
-              if (n > ((CARDS_DISPLAYED/2) + 1))
-              {
-                last_user_activity = 0;
-                mStage = STAGE_TRANSITION_LEFT;
-              }
-              mEngine->LeftClickedProcessed();
-              break;
-            }
-
             if (last_user_activity > NO_USER_ACTIVITY_HELP_DELAY)
             {
                 if (mStage != STAGE_ONSCREEN_MENU)
@@ -517,9 +564,42 @@ void GameStateDeckViewer::Update(float dt)
             }
             else
                 last_user_activity += dt;
+                
+            break;
         }
 
     }
+    if (mStage == STAGE_TRANSITION_SELECTED)
+    {
+        if (mSelected < 2)
+        {
+            mRotation -= dt * MED_SPEED;
+            if (mRotation < mSelected-2)
+            {
+                do
+                {
+                    rotateCards(STAGE_TRANSITION_RIGHT);
+                    mRotation += 1;
+                } while (mRotation < -1.0f);
+                mStage = STAGE_WAITING;
+                mRotation = 0;
+            }
+        }
+        else if (mSelected > 2)
+        {
+            mRotation += dt * MED_SPEED;
+            if (mRotation > mSelected - 2)
+            {
+                do
+                {
+                    rotateCards(STAGE_TRANSITION_LEFT);
+                    mRotation -= 1;
+                } while (mRotation > 1.0f);
+                mStage = STAGE_WAITING;
+                mRotation = 0;
+            }
+        }
+    }
     if (mStage == STAGE_TRANSITION_RIGHT || mStage == STAGE_TRANSITION_LEFT)
     {
         if (mStage == STAGE_TRANSITION_RIGHT)
@@ -637,9 +717,13 @@ void GameStateDeckViewer::renderOnScreenBasicInfo()
                 WSrcDeck::UNFILTERED_UNIQUE));
 
     float w = mFont->GetStringWidth(buffer);
-    renderer->FillRoundRect(SCREEN_WIDTH - (w + 27), y + 5, w + 10, 15, 5, ARGB(128,0,0,0));
+    PIXEL_TYPE backupColor = mFont->GetColor();
 
+    renderer->FillRoundRect(SCREEN_WIDTH - (w + 27), y + 5, w + 10, 15, 5, ARGB(hudAlpha/2,0,0,0));
+    mFont->SetColor(ARGB(hudAlpha,255,255,255));
     mFont->DrawString(buffer, SCREEN_WIDTH - 22, y + 15, JGETEXT_RIGHT);
+    mFont->SetColor(backupColor);
+    
     if (useFilter != 0) renderer->RenderQuad(mIcons[useFilter - 1].get(), SCREEN_WIDTH - 10, y + 15, 0.0f, 0.5, 0.5);
 }
 
@@ -696,7 +780,7 @@ void GameStateDeckViewer::renderDeckBackground()
     int max2 = 0;
     int maxC2 = 4;
 
-    for (int i = 0; i < Constants::MTG_NB_COLORS - 1; i++)
+    for (int i = 0; i < Constants::NB_Colors - 1; i++)
     {
         int value = myDeck->getCount(i);
         if (value > max1)
@@ -727,7 +811,6 @@ void GameStateDeckViewer::renderDeckBackground()
 
 void GameStateDeckViewer::renderOnScreenMenu()
 {
-
     WFont * font = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
     font->SetColor(ARGB(255,255,255,255));
     JRenderer * r = JRenderer::GetInstance();
@@ -741,55 +824,91 @@ void GameStateDeckViewer::renderOnScreenMenu()
     float rightPspX = SCREEN_WIDTH - 100 + rightTransition;
     float rightPspY = SCREEN_HEIGHT / 2 - 20;
 
+#ifdef TOUCH_ENABLED
+    bool renderPSPIcons = false;
+#else
+    bool renderPSPIcons = true;
+#endif
+    
     if (stw->currentPage == 0)
     {
         //FillRects
         r->FillRect(0 - (onScreenTransition * 84), 0, 84, SCREEN_HEIGHT, ARGB(128,0,0,0));
-        r->FillRect(SCREEN_WIDTH - 204 + (onScreenTransition * 204), 0, 200, SCREEN_HEIGHT, ARGB(128,0,0,0));
-
-        //LEFT PSP CIRCLE render
-        r->FillCircle(leftPspX, leftPspY, 40, ARGB(128,50,50,50));
-
-        r->RenderQuad(pspIcons[0].get(), leftPspX, leftPspY - 20, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[1].get(), leftPspX, leftPspY + 20, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[2].get(), leftPspX - 20, leftPspY, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[3].get(), leftPspX + 20, leftPspY, 0, pspIconsSize, pspIconsSize);
-
-        font->DrawString(_("Prev."), leftPspX - 35, leftPspY - 15);
-        font->DrawString(_("Next"), leftPspX + 15, leftPspY - 15);
-        font->DrawString(_("card"), leftPspX - 35, leftPspY);
-        font->DrawString(_("card"), leftPspX + 15, leftPspY);
-        font->DrawString(_("Next edition"), leftPspX - 33, leftPspY - 35);
-        font->DrawString(_("Prev. edition"), leftPspX - 33, leftPspY + 25);
-
-        //RIGHT PSP CIRCLE render
-        r->FillCircle(rightPspX + (onScreenTransition * 204), rightPspY, 40, ARGB(128,50,50,50));
-        r->RenderQuad(pspIcons[4].get(), rightPspX + 20, rightPspY, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[5].get(), rightPspX, rightPspY - 20, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[6].get(), rightPspX - 20, rightPspY, 0, pspIconsSize, pspIconsSize);
-        r->RenderQuad(pspIcons[7].get(), rightPspX, rightPspY + 20, 0, pspIconsSize, pspIconsSize);
-
-        font->DrawString(_("Toggle Images"), rightPspX - 35, rightPspY - 40);
-
-        if (displayed_deck == myCollection)
+        r->FillRect(SCREEN_WIDTH - 204 + (onScreenTransition * 204), 0, 204, SCREEN_HEIGHT, ARGB(128,0,0,0));
+        if (renderPSPIcons)
         {
-            font->DrawString(_("Add card"), rightPspX + 20, rightPspY - 15);
-            font->DrawString(_("View Deck"), rightPspX - 20, rightPspY - 15, JGETEXT_RIGHT);
+            //LEFT PSP CIRCLE render
+            r->FillCircle(leftPspX, leftPspY, 40, ARGB(128,50,50,50));
+            
+            r->RenderQuad(pspIcons[0].get(), leftPspX, leftPspY - 20, 0, pspIconsSize, pspIconsSize);
+            r->RenderQuad(pspIcons[1].get(), leftPspX, leftPspY + 20, 0, pspIconsSize, pspIconsSize);
+            r->RenderQuad(pspIcons[2].get(), leftPspX - 20, leftPspY, 0, pspIconsSize, pspIconsSize);
+            r->RenderQuad(pspIcons[3].get(), leftPspX + 20, leftPspY, 0, pspIconsSize, pspIconsSize);
+            
+            font->DrawString(_("Prev."), leftPspX - 35, leftPspY - 15);
+            font->DrawString(_("Next"), leftPspX + 15, leftPspY - 15);
+            font->DrawString(_("card"), leftPspX - 35, leftPspY);
+            font->DrawString(_("card"), leftPspX + 15, leftPspY);
+            font->DrawString(_("Next edition"), leftPspX - 33, leftPspY - 35);
+            font->DrawString(_("Prev. edition"), leftPspX - 33, leftPspY + 25);
+            
+            //RIGHT PSP CIRCLE render
+            r->FillCircle(rightPspX + (onScreenTransition * 204), rightPspY, 40, ARGB(128,50,50,50));
+            r->RenderQuad(pspIcons[4].get(), rightPspX + 20, rightPspY, 0, pspIconsSize, pspIconsSize);
+            r->RenderQuad(pspIcons[5].get(), rightPspX, rightPspY - 20, 0, pspIconsSize, pspIconsSize);
+            r->RenderQuad(pspIcons[6].get(), rightPspX - 20, rightPspY, 0, pspIconsSize, pspIconsSize);
+            r->RenderQuad(pspIcons[7].get(), rightPspX, rightPspY + 20, 0, pspIconsSize, pspIconsSize);
+            
+            font->DrawString(_("Toggle Images"), rightPspX - 35, rightPspY - 40);
+            
+            if (displayed_deck == myCollection)
+            {
+                font->DrawString(_("Add card"), rightPspX + 20, rightPspY - 15);
+                font->DrawString(_("View Deck"), rightPspX - 20, rightPspY - 15, JGETEXT_RIGHT);
+            }
+            else
+            {
+                font->DrawString(_("Remove card"), rightPspX + 20, rightPspY - 15);
+                font->DrawString(_("View Collection"), rightPspX - 20, rightPspY - 15, JGETEXT_RIGHT);
+            }
+            font->DrawString(_("Sell card"), rightPspX - 30, rightPspY + 20);
+            //Bottom menus
+            font->DrawString(_("menu"), SCREEN_WIDTH - 35 + rightTransition, SCREEN_HEIGHT - 15);
+            font->DrawString(_("filter"), SCREEN_WIDTH - 95 + rightTransition, SCREEN_HEIGHT - 15);
+
+            if (displayed_deck == myCollection)
+            {
+                font->DrawString(_("in: collection"), 5 - leftTransition, 5);
+                font->DrawString(_("Use SQUARE to view your deck,"), SCREEN_WIDTH - 200 + rightTransition, 5);
+            }
+            else
+            {
+                font->DrawString(_("in: deck"), 5 - leftTransition, 5);
+                font->DrawString(_("Use SQUARE to view collection,"), SCREEN_WIDTH - 200 + rightTransition, 5);
+            }
+            
+            font->DrawString(_("Press L/R to cycle through"), SCREEN_WIDTH - 200 + rightTransition, 5 + fH);
+            font->DrawString(_("deck statistics."), SCREEN_WIDTH - 200 + rightTransition, 5 + fH * 2);
         }
         else
         {
-            font->DrawString(_("Remove card"), rightPspX + 20, rightPspY - 15);
-            font->DrawString(_("View Collection"), rightPspX - 20, rightPspY - 15, JGETEXT_RIGHT);
+            // print stuff here about the editor commands
+            float textYOffset = SCREEN_HEIGHT_F/2;
+            font->DrawString(_("Click on the card image"), SCREEN_WIDTH - 200 + rightTransition, textYOffset - (2 * fH));
+            if (displayed_deck == myCollection)
+                font->DrawString(_("to add card to deck."), SCREEN_WIDTH - 200 + rightTransition, textYOffset - fH);
+            else
+                font->DrawString(_("to remove card from deck."), SCREEN_WIDTH - 200 + rightTransition, textYOffset - fH);
         }
-        font->DrawString(_("Sell card"), rightPspX - 30, rightPspY + 20);
-        //Bottom menus
-        font->DrawString(_("menu"), SCREEN_WIDTH - 35 + rightTransition, SCREEN_HEIGHT - 15);
-        font->DrawString(_("filter"), SCREEN_WIDTH - 95 + rightTransition, SCREEN_HEIGHT - 15);
-
         //Your Deck Information
         char buffer[300];
         int nb_letters = 0;
-        for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+        int value = myDeck->getCount(WSrcDeck::UNFILTERED_COPIES);
+        
+        sprintf(buffer, _("Your Deck: %i cards").c_str(), value);
+        font->DrawString(buffer, SCREEN_WIDTH - 200 + rightTransition, SCREEN_HEIGHT / 2 + 25);
+
+        for (int j = 0; j < Constants::NB_Colors; j++)
         {
             int value = myDeck->getCount(j);
             if (value > 0)
@@ -797,7 +916,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
                 sprintf(buffer, "%i", value);
                 font->DrawString(buffer, SCREEN_WIDTH - 190 + rightTransition + nb_letters * 13, SCREEN_HEIGHT / 2 + 40);
                 r->RenderQuad(mIcons[j].get(), SCREEN_WIDTH - 197 + rightTransition + nb_letters * 13, SCREEN_HEIGHT / 2 + 46, 0, 0.5,
-                                0.5);
+                              0.5);
                 if (value > 9)
                 {
                     nb_letters += 3;
@@ -808,22 +927,6 @@ void GameStateDeckViewer::renderOnScreenMenu()
                 }
             }
         }
-        int value = myDeck->getCount(WSrcDeck::UNFILTERED_COPIES);
-        sprintf(buffer, _("Your Deck: %i cards").c_str(), value);
-        font->DrawString(buffer, SCREEN_WIDTH - 200 + rightTransition, SCREEN_HEIGHT / 2 + 25);
-
-        if (displayed_deck == myCollection)
-        {
-            font->DrawString(_("in: collection"), 5 - leftTransition, 5);
-            font->DrawString(_("Use SQUARE to view your deck,"), SCREEN_WIDTH - 200 + rightTransition, 5);
-        }
-        else
-        {
-            font->DrawString(_("in: deck"), 5 - leftTransition, 5);
-            font->DrawString(_("Use SQUARE to view collection,"), SCREEN_WIDTH - 200 + rightTransition, 5);
-        }
-        font->DrawString(_("Press L/R to cycle through"), SCREEN_WIDTH - 200 + rightTransition, 5 + fH);
-        font->DrawString(_("deck statistics."), SCREEN_WIDTH - 200 + rightTransition, 5 + fH * 2);
 
     }
     else
@@ -839,9 +942,10 @@ void GameStateDeckViewer::renderOnScreenMenu()
         r->FillRect(SCREEN_WIDTH / 2 + rightTransition, 0, SCREEN_WIDTH / 2, SCREEN_HEIGHT, ARGB(128,0,0,0));
         r->FillRect(10 + leftTransition, 10, SCREEN_WIDTH / 2 - 10, SCREEN_HEIGHT - 20, ARGB(128,0,0,0));
         r->FillRect(SCREEN_WIDTH / 2 + rightTransition, 10, SCREEN_WIDTH / 2 - 10, SCREEN_HEIGHT - 20, ARGB(128,0,0,0));
+#ifndef TOUCH_ENABLED
         font->DrawString(_("menu"), SCREEN_WIDTH - 35 + rightTransition, SCREEN_HEIGHT - 15);
         font->DrawString(_("filter"), SCREEN_WIDTH - 95 + rightTransition, SCREEN_HEIGHT - 15);
-
+#endif
         int nb_letters = 0;
         float posX, posY;
         DWORD graphColor;
@@ -863,7 +967,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             posY += 10;
 
             // Counts by color
-            for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+            for (int j = 0; j < Constants::NB_Colors; j++)
             {
                 int value = myDeck->getCount(j);
                 if (value > 0)
@@ -981,7 +1085,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             posY = 70;
 
             // Column titles
-            for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+            for (int j = 0; j < Constants::NB_Colors - 1; j++)
             {
                 r->RenderQuad(mIcons[j].get(), 52 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
             }
@@ -990,20 +1094,20 @@ void GameStateDeckViewer::renderOnScreenMenu()
             //font->DrawString(_("Ty"), 27 + leftTransition, posY-16);
 
             // Horizontal table lines
-            r->DrawLine(27 + leftTransition, posY - 20, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 20,
+            r->DrawLine(27 + leftTransition, posY - 20, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 20,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, posY - 1, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 1,
+            r->DrawLine(27 + leftTransition, posY - 1, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 1,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, 2 * 10 + posY + 12, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, 2 * 10
+            r->DrawLine(27 + leftTransition, 2 * 10 + posY + 12, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, 2 * 10
                     + posY + 12, ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, 3 * 10 + posY + 14, 60 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, 3 * 10
+            r->DrawLine(27 + leftTransition, 3 * 10 + posY + 14, 60 + (Constants::NB_Colors - 2) * 15 + leftTransition, 3 * 10
                     + posY + 14, ARGB(128, 255, 255, 255));
 
             // Vertical table lines
             r->DrawLine(26 + leftTransition, posY - 20, 26 + leftTransition, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
             r->DrawLine(43 + leftTransition, posY - 20, 43 + leftTransition, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
-            r->DrawLine(60 + leftTransition + (Constants::MTG_NB_COLORS - 2) * 15, posY - 20, 60 + leftTransition
-                    + (Constants::MTG_NB_COLORS - 2) * 15, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
+            r->DrawLine(60 + leftTransition + (Constants::NB_Colors - 2) * 15, posY - 20, 60 + leftTransition
+                    + (Constants::NB_Colors - 2) * 15, 3 * 10 + posY + 14, ARGB(128, 255, 255, 255));
 
             font->DrawString(_("BL"), 27 + leftTransition, posY);
             font->DrawString(_("NB"), 27 + leftTransition, posY + 10);
@@ -1012,7 +1116,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
 
             int curCount;
 
-            for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+            for (int j = 0; j < Constants::NB_Colors - 1; j++)
             {
                 curCount = stw->countBasicLandsPerColor[j];
                 sprintf(buffer, (curCount == 0 ? "." : "%i"), curCount);
@@ -1049,13 +1153,13 @@ void GameStateDeckViewer::renderOnScreenMenu()
 
             int totalProducedSymbols;
             totalProducedSymbols = 0;
-            for (int i = 1; i < Constants::MTG_NB_COLORS - 1; i++)
+            for (int i = 1; i < Constants::NB_Colors - 1; i++)
             {
                 totalProducedSymbols += stw->countLandsPerColor[i] + stw->countBasicLandsPerColor[i]; //!! Move to updatestats!
             }
 
             posY = 50;
-            for (int i = 1; i < Constants::MTG_NB_COLORS - 1; i++)
+            for (int i = 1; i < Constants::NB_Colors - 1; i++)
             {
                 if (stw->countLandsPerColor[i] + stw->countBasicLandsPerColor[i] > 0)
                 {
@@ -1127,7 +1231,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             posY = 70;
 
             // Column titles
-            for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+            for (int j = 0; j < Constants::NB_Colors - 1; j++)
             {
                 r->RenderQuad(mIcons[j].get(), 67 + j * 15 + leftTransition, posY - 10, 0, 0.5, 0.5);
             }
@@ -1136,11 +1240,11 @@ void GameStateDeckViewer::renderOnScreenMenu()
             font->DrawString(_("#"), 45 + leftTransition, posY - 16);
 
             // Horizontal table lines
-            r->DrawLine(27 + leftTransition, posY - 20, 75 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 20,
+            r->DrawLine(27 + leftTransition, posY - 20, 75 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 20,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, posY - 1, 75 + (Constants::MTG_NB_COLORS - 2) * 15 + leftTransition, posY - 1,
+            r->DrawLine(27 + leftTransition, posY - 1, 75 + (Constants::NB_Colors - 2) * 15 + leftTransition, posY - 1,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(27 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12, 75 + (Constants::MTG_NB_COLORS - 2)
+            r->DrawLine(27 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12, 75 + (Constants::NB_Colors - 2)
                     * 15 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12, ARGB(128, 255, 255, 255));
 
             // Vertical table lines
@@ -1150,8 +1254,8 @@ void GameStateDeckViewer::renderOnScreenMenu()
                     ARGB(128, 255, 255, 255));
             r->DrawLine(58 + leftTransition, posY - 20, 58 + leftTransition, Constants::STATS_MAX_MANA_COST * 10 + posY + 12,
                     ARGB(128, 255, 255, 255));
-            r->DrawLine(75 + leftTransition + (Constants::MTG_NB_COLORS - 2) * 15, posY - 20, 75 + leftTransition
-                    + (Constants::MTG_NB_COLORS - 2) * 15, Constants::STATS_MAX_MANA_COST * 10 + posY + 12,
+            r->DrawLine(75 + leftTransition + (Constants::NB_Colors - 2) * 15, posY - 20, 75 + leftTransition
+                    + (Constants::NB_Colors - 2) * 15, Constants::STATS_MAX_MANA_COST * 10 + posY + 12,
                     ARGB(128, 255, 255, 255));
 
             for (int i = 0; i <= Constants::STATS_MAX_MANA_COST; i++)
@@ -1160,12 +1264,12 @@ void GameStateDeckViewer::renderOnScreenMenu()
                 font->DrawString(buffer, 30 + leftTransition, posY);
                 sprintf(buffer, ((*countPerCost)[i] > 0) ? _("%i").c_str() : ".", (*countPerCost)[i]);
                 font->DrawString(buffer, 45 + leftTransition, posY);
-                for (int j = 0; j < Constants::MTG_NB_COLORS - 1; j++)
+                for (int j = 0; j < Constants::NB_Colors - 1; j++)
                 {
                     sprintf(buffer, ((*countPerCostAndColor)[i][j] > 0) ? _("%i").c_str() : ".", (*countPerCostAndColor)[i][j]);
                     font->DrawString(buffer, 64 + leftTransition + j * 15, posY);
                 }
-                r->FillRect(77.f + leftTransition + (Constants::MTG_NB_COLORS - 2) * 15.0f, posY + 2.0f, (*countPerCost)[i] * 5.0f,
+                r->FillRect(77.f + leftTransition + (Constants::NB_Colors - 2) * 15.0f, posY + 2.0f, (*countPerCost)[i] * 5.0f,
                         8.0f, graphColor);
                 posY += 10;
             }
@@ -1230,7 +1334,7 @@ void GameStateDeckViewer::renderOnScreenMenu()
             font->DrawString(_("Total colored manasymbols in cards' casting costs:"), 20 + leftTransition, 30);
 
             posY = 50;
-            for (int i = 1; i < Constants::MTG_NB_COLORS - 1; i++)
+            for (int i = 1; i < Constants::NB_Colors - 1; i++)
             {
                 if (stw->totalCostPerColor[i] > 0)
                 {
@@ -1299,7 +1403,7 @@ void GameStateDeckViewer::renderCard(int id, float rotation)
 
     float max_scale = 0.96f;
     float x_center_0 = 180;
-    float right_border = SCREEN_WIDTH - 20;
+    float right_border = SCREEN_WIDTH + 180;
 
     float x_center = x_center_0 + cos((rotation + 8 - id) * M_PI / 12) * (right_border - x_center_0);
     float scale = max_scale / 1.12f * cos((x_center - x_center_0) * 1.5f / (right_border - x_center_0)) + 0.2f * max_scale * cos(
@@ -1396,9 +1500,8 @@ void GameStateDeckViewer::renderCard(int id)
 
 void GameStateDeckViewer::Render()
 {
-
-    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
-
+    setButtonState(false);
+    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);    
     JRenderer::GetInstance()->ClearScreen(ARGB(0,0,0,0));
     if (displayed_deck == myDeck && mStage != STAGE_MENU) 
 		renderDeckBackground();
@@ -1439,6 +1542,7 @@ void GameStateDeckViewer::Render()
 
     if (displayed_deck->Size() > 0)
     {
+        setButtonState(true);
         renderSlideBar();
     }
     else
@@ -1451,23 +1555,32 @@ void GameStateDeckViewer::Render()
     }
     else if (mStage == STAGE_WELCOME)
     {
+        setButtonState(false);
         welcome_menu->Render();
     }
     else
     {
+        setButtonState(true);
         renderOnScreenBasicInfo();
-
     }
+    
     if (mStage == STAGE_MENU)
     {
+        setButtonState(false);
         menu->Render();
     }
+    
     if (subMenu) subMenu->Render();
 
-    if (filterMenu && !filterMenu->isFinished()) filterMenu->Render();
-
+    if (filterMenu && !filterMenu->isFinished()) 
+    {
+        setButtonState(false);
+        filterMenu->Render();
+    }
+    
     if (options.keypadActive()) options.keypadRender();
 
+    RenderButtons();
 }
 
 int GameStateDeckViewer::loadDeck(int deckid)
@@ -1496,7 +1609,7 @@ int GameStateDeckViewer::loadDeck(int deckid)
         SAFE_DELETE(myDeck->parent);
         SAFE_DELETE(myDeck);
     }
-    myDeck = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(deckname, "", false, false).c_str(), MTGCollection()));
+    myDeck = NEW DeckDataWrapper(NEW MTGDeck(options.profileFile(deckname, "", false).c_str(), MTGCollection()));
 
     // Check whether the cards in the deck are actually available in the player's collection:
     int cheatmode = options[Options::CHEATMODE].number;
@@ -1515,7 +1628,7 @@ int GameStateDeckViewer::loadDeck(int deckid)
             }
             else
             {
-                myDeck->Remove(current); //Nope. Remove it from deck.
+                myDeck->Remove(current,howmanyinDeck-i); //Nope. Remove it from deck.
                 break;
             }
         }
@@ -1668,12 +1781,42 @@ void GameStateDeckViewer::ButtonPressed(int controllerId, int controlId)
     }
 }
 
+/*
+  to keep actions consistent across the different platforms, we need to branch the way swipes are interpreted.  iOS5 changes
+ the way a swipe moves a document on the page.  swipe down is to simulate dragging the page down instead of moving down 
+ on a page.
+ */
 void GameStateDeckViewer::OnScroll(int inXVelocity, int inYVelocity)
 {
-    if (abs(inYVelocity) > 300)
+    int magnitude = static_cast<int>( sqrtf( (float )( (inXVelocity * inXVelocity) + (inYVelocity * inYVelocity))));
+    
+    bool flickHorizontal = (abs(inXVelocity) > abs(inYVelocity));
+    bool flickUp = !flickHorizontal && (inYVelocity > 0) ? true : false;
+    bool flickRight = flickHorizontal && (inXVelocity < 0) ? true : false;
+    
+    if (flickHorizontal)
     {
-        bool flickUpwards = (inYVelocity < 0);
-        mEngine->HoldKey_NoRepeat(flickUpwards ? JGE_BTN_DOWN : JGE_BTN_UP);
+        if(abs(inXVelocity) > 300)
+        {
+            //determine how many cards to move, the faster the velocity the more cards to move.
+            // the display is setup so that there is a max of 2 cards to the left and 7 cards to the right
+            // of the current card.
+            int numCards = (magnitude / 500) % 8;
+            int offset = 0;
+            if ( (numCards == 0) && magnitude) numCards = 7;
+            if ( !flickRight)
+            {
+                if (numCards > 1)
+                    offset = 0;
+            }
+            else
+                offset = 2 + numCards;
 
+            mEngine->LeftClickedProcessed();
+            mEngine->LeftClicked(static_cast<int>(cardsCoordinates[offset].first), static_cast<int>(cardsCoordinates[offset].second));
+            mEngine->HoldKey_NoRepeat(JGE_BTN_OK);
+        }
     }
+    else
+        mEngine->HoldKey_NoRepeat(flickUp ? JGE_BTN_UP : JGE_BTN_DOWN);
 }
