@@ -10,7 +10,6 @@
 #include "DeckManager.h"
 
 #include "DeckMetaData.h"
-#include "MTGRules.h"
 #include "Credits.h"
 #include "Translate.h"
 #include "Rules.h"
@@ -24,7 +23,7 @@
 #include "NetworkPlayer.h"
 #endif
 
-#if defined (WIN32) || defined (LINUX)
+#if defined (WIN32) || defined (LINUX) || defined(IOS)
 #include <time.h>
 #endif
 
@@ -32,12 +31,14 @@ const float MENU_FONT_SCALE = 1.0f;
 
 enum ENUM_DUEL_STATE
 {
+    DUEL_STATE_UNSET = 0,
     DUEL_STATE_START,
     DUEL_STATE_END,
     DUEL_STATE_CHOOSE_DECK1,
     DUEL_STATE_DECK1_DETAILED_INFO,
     DUEL_STATE_DECK2_DETAILED_INFO,
     DUEL_STATE_CHOOSE_DECK1_TO_2,
+    DUEL_STATE_CHOOSE_DECK2_TO_1,
     DUEL_STATE_CHOOSE_DECK2,
     DUEL_STATE_CHOOSE_DECK2_TO_PLAY,
     DUEL_STATE_ERROR_NO_DECK,
@@ -45,12 +46,13 @@ enum ENUM_DUEL_STATE
     DUEL_STATE_PLAY,
     DUEL_STATE_BACK_TO_MAIN_MENU,
     DUEL_STATE_MENU,
-#ifdef NETWORK_SUPPORT
-    DUEL_STATE_OPPONENT_WAIT,
-#endif //NETWORK_SUPPORT
+    DUEL_STATE_OPPONENT_WAIT, // used For Network only
     DUEL_STATE_ERROR
 };
 
+const char * stateStrings[] = { "unset", "start", "end", "choose_deck1", "deck1_detailed_info", "deck2_detailed_info", "choose_deck1_to_2", "choose_deck2", "choose_deck2_to_play",
+    "error_no_deck", "cancel", "play", "back_to_main_menu", "menu", "oponent_wait", "error" };
+
 enum ENUM_DUEL_MENUS
 {
     DUEL_MENU_GAME_MENU,
@@ -64,23 +66,41 @@ int GameStateDuel::selectedPlayerDeckId = 0;
 int GameStateDuel::selectedAIDeckId = 0;
 
 GameStateDuel::GameStateDuel(GameApp* parent) :
-GameState(parent)
+GameState(parent, "duel")
 {
-    for (int i = 0; i < 2; i++)
-    {
-        mPlayers[i] = NULL;
-    }
     premadeDeck = false;
     game = NULL;
     deckmenu = NULL;
     opponentMenu = NULL;
     menu = NULL;
     popupScreen = NULL;
+    mGamePhase = DUEL_STATE_UNSET;
 
 #ifdef TESTSUITE
     testSuite = NULL;
 #endif
 
+#ifdef AI_CHANGE_TESTING
+    totalTestGames = 0;
+    testPlayer2Victories = 0;
+    totalAIDecks = 0;
+    {
+        int found = 1;
+        while (found)
+        {
+            found = 0;
+            char buffer[512];
+            sprintf(buffer, "ai/baka/deck%i.txt", totalAIDecks + 1);
+            if (FileExists(buffer))
+            {
+                found = 1;
+                totalAIDecks++;
+            }
+        } 
+        
+    }
+#endif
+
     credits = NULL;
 
 #ifdef NETWORK_SUPPORT
@@ -100,12 +120,14 @@ void GameStateDuel::Start()
     renderer->EnableVSync(true);
     OpponentsDeckid = 0;
 
+    game = NEW GameObserver(WResourceManager::Instance(), JGE::GetInstance());
+
 #ifdef TESTSUITE
     SAFE_DELETE(testSuite);
-    testSuite = NEW TestSuite(JGE_GET_RES("test/_tests.txt").c_str(),MTGCollection());
+    testSuite = NEW TestSuite("test/_tests.txt");
 #endif
 
-    mGamePhase = DUEL_STATE_CHOOSE_DECK1;
+    setGamePhase(DUEL_STATE_CHOOSE_DECK1);
     credits = NEW Credits();
 
     menu = NULL;
@@ -156,7 +178,7 @@ void GameStateDuel::Start()
                 deckmenu->Add(MENUITEM_NEW_DECK, "Create your Deck!", desc);
             }
             premadeDeck = true;
-            fillDeckMenu(deckmenu, JGE_GET_RES("player/premade"));
+            fillDeckMenu(deckmenu, "player/premade");
         }
         else if (gModRules.general.hasDeckEditor())
         {
@@ -164,57 +186,8 @@ void GameStateDuel::Start()
         }
         deckmenu->Add(MENUITEM_CANCEL, "Main Menu", "Return to Main Menu");
     }
-
-    for (int i = 0; i < 2; ++i)
-    {
-        mPlayers[i] = NULL;
-    }
-}
-
-void GameStateDuel::loadPlayer(int playerId, int decknb, bool isAI, bool isNetwork)
-{
-    if (decknb)
-    {
-        if (!isAI)
-        { //Human Player
-            if(playerId == 0)
-            {
-                char deckFile[255];
-                if (premadeDeck)
-                    sprintf(deckFile, JGE_GET_RES("player/premade/deck%i.txt").c_str(), decknb);
-                else
-                    sprintf(deckFile, "%s/deck%i.txt", options.profileFile().c_str(), decknb);
-                char deckFileSmall[255];
-                sprintf(deckFileSmall, "player_deck%i", decknb);
-                mPlayers[playerId] = NEW HumanPlayer(deckFile, deckFileSmall);
-#ifdef NETWORK_SUPPORT
-                if(isNetwork)
-                {
-                    ProxyPlayer* mProxy;
-                    mProxy = NEW ProxyPlayer(mPlayers[playerId], mParent->mpNetwork);
-                }
-            }
-            else
-            {   //Remote player
-                mPlayers[playerId] = NEW RemotePlayer(mParent->mpNetwork);
-#endif //NETWORK_SUPPORT
-            }
-        }
-        else
-        { //AI Player, chooses deck
-            AIPlayerFactory playerCreator;
-            Player * opponent = NULL;
-            if (playerId == 1) opponent = mPlayers[0];
-            mPlayers[playerId] = playerCreator.createAIPlayer(MTGCollection(), opponent, decknb);
-        }
-    }
-    else
-    { //Random deck
-        AIPlayerFactory playerCreator;
-        Player * opponent = NULL;
-        if (playerId == 1) opponent = mPlayers[0];
-        mPlayers[playerId] = playerCreator.createAIPlayer(MTGCollection(), opponent);
-    }
+    
+    mEngine->ResetInput();
 }
 
 void GameStateDuel::initRand(unsigned int seed)
@@ -230,19 +203,15 @@ void GameStateDuel::loadTestSuitePlayers()
     if (!testSuite) return;
     initRand(testSuite->seed);
     SAFE_DELETE(game);
+    game = new GameObserver(WResourceManager::Instance(), JGE::GetInstance());
+    testSuite->setObserver(game);
     for (int i = 0; i < 2; i++)
     {
-        mPlayers[i] = NEW TestSuiteAI(testSuite, i);
+        game->loadTestSuitePlayer(i, testSuite);
     }
-    mParent->gameType = testSuite->gameType;
+    mParent->gameType = testSuite->getGameType();
 
-    GameObserver::Init(mPlayers, 2);
-    game = GameObserver::GetInstance();
-    game->startGame(mParent->rules);
-    if (mParent->gameType == GAME_TYPE_MOMIR)
-    {
-        game->addObserver(NEW MTGMomirRule(-1, MTGCollection()));
-    }
+    game->startGame(mParent->gameType, mParent->rules);
 }
 #endif
 
@@ -255,22 +224,9 @@ void GameStateDuel::End()
 #endif
 
     JRenderer::GetInstance()->EnableVSync(false);
-    if (!premadeDeck && mPlayers[0] && mPlayers[1])
-    { // save the stats for the game
-        mPlayers[0]->End();
-    }
-    else if ( !mPlayers[1] && mPlayers[0] )
-        // clean up player object
-        SAFE_DELETE( mPlayers[0] );
-    GameObserver::EndInstance(); // this will delete both player objects if a game has been played
 
-    game = NULL;
+    SAFE_DELETE(game);
     premadeDeck = false;
-
-    for (int i = 0; i < 2; i++)
-    {
-        mPlayers[i] = NULL;
-    }
     SAFE_DELETE(credits);
 
     SAFE_DELETE(menu);
@@ -286,15 +242,10 @@ void GameStateDuel::End()
 //TODO Move This to utils or ResourceManager. Don't we have more generic functions that can do that?
 bool GameStateDuel::MusicExist(string FileName)
 {
-    string filepath = JGE_GET_RES(WResourceManager::Instance()->musicFile(FileName));
-    wagic::ifstream file(filepath.c_str());
-    if (file)
-    {
-        file.close();
-        return true;
-    }
-    else
-        return false;
+    string musicFilename = WResourceManager::Instance()->musicFile(FileName);
+    if (musicFilename.length() < 1) return false;
+    
+    return FileExists(musicFilename);
 }
 
 void GameStateDuel::ConstructOpponentMenu()
@@ -309,13 +260,64 @@ void GameStateDuel::ConstructOpponentMenu()
         DeckManager * deckManager = DeckManager::GetInstance();
         vector<DeckMetaData*> opponentDeckList;
         int nbUnlockedDecks = options[Options::CHEATMODEAIDECK].number ? 1000 : options[Options::AIDECKS_UNLOCKED].number;
-        opponentDeckList = fillDeckMenu(opponentMenu, JGE_GET_RES("ai/baka"), "ai_baka", mPlayers[0], nbUnlockedDecks);
+        opponentDeckList = fillDeckMenu(opponentMenu, "ai/baka", "ai_baka", game->getPlayer(0), nbUnlockedDecks);
         deckManager->updateMetaDataList(&opponentDeckList, true);
         opponentMenu->Add(MENUITEM_CANCEL, "Cancel", _("Choose a different player deck").c_str());
         opponentDeckList.clear();
     }
 }
 
+void GameStateDuel::setGamePhase(int newGamePhase) {
+    if (mGamePhase == newGamePhase)
+        return;
+
+    if (mGamePhase)
+        JGE::GetInstance()->SendCommand("leaveduelphase:" + string(stateStrings[mGamePhase]));
+
+    mGamePhase = newGamePhase;
+
+    if (mGamePhase )
+        JGE::GetInstance()->SendCommand("enterduelphase:" +  string(stateStrings[mGamePhase]));
+}
+
+#ifdef AI_CHANGE_TESTING
+boost::mutex GameStateDuel::mMutex;
+
+void GameStateDuel::ThreadProc(void* inParam)
+{
+    GameStateDuel* instance = reinterpret_cast<GameStateDuel*>(inParam);
+    float counter = 1.0f;
+    while(instance->mGamePhase != DUEL_STATE_BACK_TO_MAIN_MENU)
+    {
+        GameObserver observer;
+        int oldTurn = -1;
+        int oldPhase = -1;
+        int stagnationCounter = -1;
+
+        observer.loadPlayer(0, PLAYER_TYPE_CPU_TEST);
+        observer.loadPlayer(1, PLAYER_TYPE_CPU_TEST);
+        observer.startGame(instance->mParent->gameType, instance->mParent->rules);
+
+        while(!observer.didWin()) {
+            if(observer.turn == oldTurn && observer.currentGamePhase == oldPhase) {
+                stagnationCounter++;
+            } else {
+                stagnationCounter = 0;
+                oldTurn = observer.turn;
+                oldPhase = observer.currentGamePhase;
+            }
+            if(stagnationCounter >= 1000)
+            {
+                observer.dumpAssert(false);
+            }
+            observer.Update(counter++);
+        }
+
+        instance->handleResults(&observer);
+    }
+}
+#endif //AI_CHANGE_TESTING
+
 void GameStateDuel::Update(float dt)
 {
     switch (mGamePhase)
@@ -332,27 +334,28 @@ void GameStateDuel::Update(float dt)
     case DUEL_STATE_CHOOSE_DECK1:
         if (!mParent->rules->canChooseDeck())
         {
-            mGamePhase = DUEL_STATE_PLAY;
+            setGamePhase(DUEL_STATE_PLAY);
         }
 #ifdef TESTSUITE
         else if (mParent->players[1] == PLAYER_TYPE_TESTSUITE)
         {
+            testSuite->setRules(mParent->rules);
+
             if (testSuite && testSuite->loadNext())
             {
                 loadTestSuitePlayers();
-                mGamePhase = DUEL_STATE_PLAY;
-                testSuite->pregameTests();
-                testSuite->initGame();
+                setGamePhase(DUEL_STATE_PLAY);
+                testSuite->initGame(game);
             }
             else
             {
                 if (!game)
                 {
-                    mGamePhase = DUEL_STATE_ERROR;
+                    setGamePhase(DUEL_STATE_ERROR);
                 }
                 else
                 {
-                    mGamePhase = DUEL_STATE_END;
+                    setGamePhase(DUEL_STATE_END);
                 }
             }
         }
@@ -365,17 +368,24 @@ void GameStateDuel::Update(float dt)
             }
             else
             {
-                loadPlayer(0);
-                mGamePhase = DUEL_STATE_CHOOSE_DECK2;
+                game->loadPlayer(0, mParent->players[0]);
+                setGamePhase(DUEL_STATE_CHOOSE_DECK2);
             }
         }
         break;
     case DUEL_STATE_CHOOSE_DECK1_TO_2:
         if (deckmenu->isClosed())
-            mGamePhase = DUEL_STATE_CHOOSE_DECK2;
+            setGamePhase(DUEL_STATE_CHOOSE_DECK2);
         else
             deckmenu->Update(dt);
         break;
+    case DUEL_STATE_CHOOSE_DECK2_TO_1:
+        if (opponentMenu->isClosed()) {
+            setGamePhase(DUEL_STATE_CHOOSE_DECK1);
+            SAFE_DELETE(opponentMenu);
+        } else
+            opponentMenu->Update(dt);
+        break;
     case DUEL_STATE_CHOOSE_DECK2:
         if (mParent->players[1] == PLAYER_TYPE_HUMAN)
             deckmenu->Update(dt);
@@ -388,8 +398,8 @@ void GameStateDuel::Update(float dt)
             }
             else
             {
-                loadPlayer(1);
-                mGamePhase = DUEL_STATE_PLAY;
+                game->loadPlayer(1, mParent->players[1]);
+                setGamePhase(DUEL_STATE_PLAY);
             }
         }
         break;
@@ -397,7 +407,7 @@ void GameStateDuel::Update(float dt)
         if (mParent->players[1] == PLAYER_TYPE_HUMAN)
         {
             if (deckmenu->isClosed())
-                mGamePhase = DUEL_STATE_PLAY;
+                setGamePhase(DUEL_STATE_PLAY);
             else
                 deckmenu->Update(dt);
         }
@@ -405,29 +415,15 @@ void GameStateDuel::Update(float dt)
         {
             ConstructOpponentMenu();
             if (opponentMenu->isClosed())
-                mGamePhase = DUEL_STATE_PLAY;
+                setGamePhase(DUEL_STATE_PLAY);
             else
                 opponentMenu->Update(dt);
         }
         break;
     case DUEL_STATE_PLAY:
-        if (!game)
+        if (!game->isStarted())
         {
-            GameObserver::Init(mPlayers, 2);
-            game = GameObserver::GetInstance();
-            game->startGame(mParent->rules);
-            if (mParent->gameType == GAME_TYPE_MOMIR)
-            {
-                game->addObserver(NEW MTGMomirRule(-1, MTGCollection()));
-            }
-			if (mParent->gameType == GAME_TYPE_STONEHEWER)
-			{
-				game->addObserver(NEW MTGStoneHewerRule(-1,MTGCollection()));
-			}
-			if (mParent->gameType == GAME_TYPE_HERMIT)
-			{
-				game->addObserver(NEW MTGHermitRule(-1));
-			}
+            game->startGame(mParent->gameType, mParent->rules);
 
             //start of in game music code
             musictrack = "";
@@ -445,48 +441,72 @@ void GameStateDuel::Update(float dt)
             else if (mParent->gameType == GAME_TYPE_RANDOM1 || mParent->gameType == GAME_TYPE_RANDOM2) musictrack
                 = "ai_baka_music_random.mp3";
 
-            if (!MusicExist(musictrack)) musictrack = "ai_baka_music.mp3";
+            if (!MusicExist(musictrack)) 
+                musictrack = "ai_baka_music.mp3";
 
             GameApp::playMusic(musictrack);
         }
         game->Update(dt);
         //run a "post update" init call in the rules. This is for things such as Manapool, which gets emptied in the update
         // That's mostly because of a legacy bug, where we use the update sequence for some things when we should use events (such as phase changes)
-        mParent->rules->postUpdateInit();
-        if (game->gameOver)
+        mParent->rules->postUpdateInit(game);
+
+        if (game->didWin())
         {
-            if (game->players[1]->playMode != Player::MODE_TEST_SUITE) credits->compute(game->players[0], game->players[1], mParent);
-            mGamePhase = DUEL_STATE_END;
+            if (game->players[1]->playMode != Player::MODE_TEST_SUITE) credits->compute(game, mParent);
+            setGamePhase(DUEL_STATE_END);
 #ifdef TESTSUITE
             if (mParent->players[1] == PLAYER_TYPE_TESTSUITE)
             {
                 if (testSuite->loadNext())
                 {
                     loadTestSuitePlayers();
-                    mGamePhase = DUEL_STATE_PLAY;
-                    testSuite->initGame();
+                    setGamePhase(DUEL_STATE_PLAY);
+                    testSuite->initGame(game);
+                }
+                else {
+                    setGamePhase(DUEL_STATE_END);
                 }
-                else
-                    mGamePhase = DUEL_STATE_END;
             }
             else
 #endif
-                if (mParent->players[0] == PLAYER_TYPE_CPU && mParent->players[1] == PLAYER_TYPE_CPU)
+#ifdef AI_CHANGE_TESTING
+            {
+                if (mParent->players[0] == PLAYER_TYPE_CPU_TEST && mParent->players[1] == PLAYER_TYPE_CPU_TEST)
                 {
+                    handleResults(game);
                     End();
                     Start();
+
+                    if(mWorkerThread.empty())
+                    {   // "I don't like to wait" mode
+                        size_t thread_count = 1;
+                        startTime = JGEGetTime();
+
+                #ifdef QT_CONFIG
+                        thread_count = QThread::idealThreadCount();
+                #endif
+                        for(size_t i = 0; i < (thread_count); i++)
+                            mWorkerThread.push_back(boost::thread(ThreadProc, this));
+                    }
                 }
+            }
+#endif
+            if (mParent->players[0] == PLAYER_TYPE_CPU && mParent->players[1] == PLAYER_TYPE_CPU)
+            {
+                End();
+                Start();
+            }
         }
         if (mEngine->GetButtonClick(JGE_BTN_MENU))
         {
             if (!menu)
             {
-                menu = NEW SimpleMenu(DUEL_MENU_GAME_MENU, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 25,
-                    game->players[1]->deckName.c_str());
+                menu = NEW SimpleMenu(JGE::GetInstance(), DUEL_MENU_GAME_MENU, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 25);
                 int cardsinhand = game->currentPlayer->game->hand->nb_cards;
 
                 //almosthumane - mulligan
-                if ((game->turn < 1) && (cardsinhand != 0) && game->currentGamePhase == Constants::MTG_PHASE_FIRSTMAIN
+                if ((game->turn < 1) && (cardsinhand != 0) && game->currentGamePhase == MTG_PHASE_FIRSTMAIN
                     && game->currentPlayer->game->inPlay->nb_cards == 0 && game->currentPlayer->game->graveyard->nb_cards == 0
 					&& game->currentPlayer->game->exile->nb_cards == 0 && game->currentlyActing() == (Player*)game->currentPlayer) //1st Play Check
                     //IF there was no play at the moment automatically mulligan
@@ -495,9 +515,15 @@ void GameStateDuel::Update(float dt)
                 }
                 //END almosthumane - mulligan
                 menu->Add(MENUITEM_MAIN_MENU, "Back to main menu");
+#ifdef DEBUG
+                menu->Add(MENUITEM_UNDO, "Undo");
+#endif
+#ifdef TESTSUITE
+                menu->Add(MENUITEM_LOAD, "Load");
+#endif
                 menu->Add(MENUITEM_CANCEL, "Cancel");
             }
-            mGamePhase = DUEL_STATE_MENU;
+            setGamePhase(DUEL_STATE_MENU);
         }
         break;
 #ifdef NETWORK_SUPPORT
@@ -507,7 +533,7 @@ void GameStateDuel::Update(float dt)
         { // Player loaded
           menu->Close();
           SAFE_DELETE(menu);
-          mGamePhase = DUEL_STATE_PLAY;
+          setGamePhase(DUEL_STATE_PLAY);
         } else if(menu == NULL)
         {
           loadPlayer(1, 42/* 0 not good*/, false, true);
@@ -530,13 +556,21 @@ void GameStateDuel::Update(float dt)
         menu->Update(dt);
         if (menu->isClosed())
         {
-            mGamePhase = DUEL_STATE_PLAY;
+            setGamePhase(DUEL_STATE_PLAY);
             SAFE_DELETE(menu);
         }
         break;
     case DUEL_STATE_BACK_TO_MAIN_MENU:
         if (menu)
         {
+#ifdef AI_CHANGE_TESTING
+            while(mWorkerThread.size())
+            {
+              mWorkerThread.back().join();
+              mWorkerThread.pop_back();
+            }
+#endif //AI_CHANGE_TESTING
+
             menu->Update(dt);
             if (menu->isClosed())
             {
@@ -563,6 +597,31 @@ void GameStateDuel::Render()
 
     if (game) game->Render();
 
+#ifdef AI_CHANGE_TESTING
+      if (game && totalTestGames)
+      {
+          char buf[4096];
+          int currentTime = JGEGetTime();
+
+          if (totalTestGames < 2.5 * totalAIDecks)
+          {
+               mFont->SetColor(ARGB(255,255,255,0));
+               sprintf(buf, "Results are not significant, you should let at least %i more games run", (int)(totalAIDecks * 2.5) - totalTestGames);
+               mFont->DrawString(buf,0,SCREEN_HEIGHT/2 - 20);
+          }
+
+          mFont->SetColor(ARGB(255,255,255,255));
+          float ratio = float(testPlayer2Victories) / float(totalTestGames);
+          if (ratio < 0.48)
+              mFont->SetColor(ARGB(255,255,0,0));
+          if (ratio > 0.52)
+              mFont->SetColor(ARGB(255,0,255,0));
+          sprintf(buf, "Victories Player 2/total Games: %i/%i - Games/second: %f",
+                  testPlayer2Victories, totalTestGames, (float)(1000*totalTestGames)/(currentTime - startTime));
+          mFont->DrawString(buf,0,SCREEN_HEIGHT/2);
+      }
+#endif
+
     switch (mGamePhase)
     {
     case DUEL_STATE_END:
@@ -574,6 +633,12 @@ void GameStateDuel::Render()
             {
                 r->ClearScreen(ARGB(255,0,0,0));
                 char buf[4096];
+                mFont->SetColor(ARGB(255,255,255,255));
+
+                int elapsedTime = (testSuite->endTime - testSuite->startTime);
+                sprintf(buf, "Time to run the tests: %is", elapsedTime/1000);
+                mFont->DrawString(buf,0,SCREEN_HEIGHT/2 - 20);
+
                 int nbFailed = testSuite->nbFailed;
                 int nbTests = testSuite->nbTests;
                 if (!nbFailed)
@@ -584,7 +649,7 @@ void GameStateDuel::Render()
                 {
                     sprintf(buf, "%i tests out of %i FAILED!", nbFailed, nbTests);
                 }
-                mFont->SetColor(ARGB(255,255,255,255));
+  
                 mFont->DrawString(buf,0,SCREEN_HEIGHT/2);
                 nbFailed = testSuite->nbAIFailed;
                 nbTests = testSuite->nbAITests;
@@ -625,8 +690,13 @@ void GameStateDuel::Render()
             mFont->DrawString(_("LOADING DECKS").c_str(), 0, SCREEN_HEIGHT / 2);
         else
         {
-            if (opponentMenu)
+            if (opponentMenu && !opponentMenu->isClosed())
+            {
                 opponentMenu->Render();
+                // display the selected player deck name too
+                string selectedPlayerDeckName = "Player Deck: " + game->players[0]->deckName;
+                mFont->DrawString( selectedPlayerDeckName.c_str(), 30, 40);
+            }
             else if (deckmenu && !deckmenu->isClosed()) deckmenu->Render();
 
             if (menu) menu->Render();
@@ -654,7 +724,17 @@ void GameStateDuel::Render()
             mFont->SetColor(ARGB(255,255,255,255));
             mFont->DrawString(buffer, SCREEN_WIDTH / 2, 0, JGETEXT_CENTER);
         }
-        if (menu) menu->Render();
+        if (menu) 
+        {
+            menu->Render();
+            
+            // display the player deck names in their respective corners
+            string playerDeckName =  game->players[0]->deckName;
+            string opponentDeckName = game->players[1]->deckName;
+            float playerDeckNamePixelLength = mFont->GetStringWidth(playerDeckName.c_str());
+            mFont->DrawString( opponentDeckName, 0, 50);
+            mFont->DrawString( playerDeckName, SCREEN_WIDTH_F - playerDeckNamePixelLength, SCREEN_HEIGHT_F - 50);
+        }
     }
 }
 
@@ -673,7 +753,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             if (!popupScreen->isClosed())
             {
                 popupScreen->Close();
-                mGamePhase = DUEL_STATE_CHOOSE_DECK1;
+                setGamePhase(DUEL_STATE_CHOOSE_DECK1);
                 SAFE_DELETE( popupScreen );
             }
             else
@@ -690,7 +770,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             if (!popupScreen->isClosed())
             {
                 popupScreen->Close();
-                mGamePhase = DUEL_STATE_CHOOSE_DECK2_TO_PLAY;
+                setGamePhase(DUEL_STATE_CHOOSE_DECK2_TO_PLAY);
                 SAFE_DELETE( popupScreen );
             }
             else
@@ -706,9 +786,9 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
         switch (controlId)
         {
         case MENUITEM_RANDOM_AI:
-            loadPlayer(1);
+            game->loadPlayer(1, mParent->players[1]);
             opponentMenu->Close();
-            mGamePhase = DUEL_STATE_CHOOSE_DECK2_TO_PLAY;
+            setGamePhase(DUEL_STATE_CHOOSE_DECK2_TO_PLAY);
             break;
         default:
             // cancel option.  return to player deck selection
@@ -717,8 +797,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             {
                 opponentMenu->Close();
                 deckmenu->Close();
-                mParent->SetNextState(DUEL_STATE_CHOOSE_DECK1);
-                mGamePhase = DUEL_STATE_CHOOSE_DECK1;
+                setGamePhase(DUEL_STATE_CHOOSE_DECK2_TO_1);
                 break;
             }
 
@@ -736,7 +815,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
                 {
                     popupScreen->Update(selectedDeck);
                 }
-                mGamePhase = DUEL_STATE_DECK2_DETAILED_INFO;
+                setGamePhase(DUEL_STATE_DECK2_DETAILED_INFO);
                 break;
             }
             else if (controlId == MENUITEM_MORE_INFO && !opponentMenu->showDetailsScreen())
@@ -746,10 +825,10 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             }
             else if (controlId != MENUITEM_EVIL_TWIN && aiDeckSize > 0) // evil twin
                 deckNumber = deckManager->getAIDeckOrderList()->at(controlId - 1)->getDeckId();
-            loadPlayer(1, deckNumber, 1);
+            game->loadPlayer(1, mParent->players[1], deckNumber, premadeDeck);
             OpponentsDeckid = deckNumber;
             opponentMenu->Close();
-            mGamePhase = DUEL_STATE_CHOOSE_DECK2_TO_PLAY;
+            setGamePhase(DUEL_STATE_CHOOSE_DECK2_TO_PLAY);
             break;
         }
         break;
@@ -758,17 +837,17 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
 
         if (controlId == MENUITEM_RANDOM_PLAYER) // Random Player Deck Selection
         {
-            vector<DeckMetaData *> * playerDeckList = deckManager->getPlayerDeckOrderList();
-            deckNumber = playerDeckList->at(WRand() % (playerDeckList->size()))->getDeckId();
-            loadPlayer(0, deckNumber);
+            deckmenu->selectRandomDeck(false);
+            
+            game->loadPlayer(0, mParent->players[0], deckmenu->getSelectedDeckId(), premadeDeck);
             deckmenu->Close();
-            mGamePhase = DUEL_STATE_CHOOSE_DECK2_TO_PLAY;
+            setGamePhase(DUEL_STATE_CHOOSE_DECK2_TO_PLAY);
             break;
         }
         else if (controlId == MENUITEM_MAIN_MENU || controlId == MENUITEM_CANCEL) // user clicked on "Cancel"
         {
             if (deckmenu) deckmenu->Close();
-            mGamePhase = DUEL_STATE_BACK_TO_MAIN_MENU;
+            setGamePhase(DUEL_STATE_BACK_TO_MAIN_MENU);
             break;
         }
         else if (controlId == MENUITEM_MORE_INFO && deckmenu->showDetailsScreen())
@@ -785,7 +864,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             {
                 popupScreen->Update(selectedDeck);
             }
-            mGamePhase = DUEL_STATE_DECK1_DETAILED_INFO;
+            setGamePhase(DUEL_STATE_DECK1_DETAILED_INFO);
             break;
         }
         else if (controlId == MENUITEM_MORE_INFO)
@@ -803,61 +882,76 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             vector<DeckMetaData *> * playerDeck = deckManager->getPlayerDeckOrderList();
             if (!premadeDeck && controlId > 0) 
                 deckNumber = playerDeck->at(controlId - 1)->getDeckId();
-            loadPlayer(0, deckNumber, false
-#ifdef NETWORK_SUPPORT
-                       ,(mParent->players[1] == PLAYER_TYPE_REMOTE)
-#endif //NETWORK_SUPPORT
-                       );
+            game->loadPlayer(0, mParent->players[0], deckNumber, premadeDeck);
             deckmenu->Close();
 #ifdef NETWORK_SUPPORT
             if(mParent->players[1] == PLAYER_TYPE_REMOTE)
             {   // no need to choose an opponent deck in network mode
-                mGamePhase = DUEL_STATE_OPPONENT_WAIT;
+                setGamePhase(DUEL_STATE_OPPONENT_WAIT);
             }
             else
 #endif //NETWORK_SUPPORT
             {
-                mGamePhase = DUEL_STATE_CHOOSE_DECK1_TO_2;
+                setGamePhase(DUEL_STATE_CHOOSE_DECK1_TO_2);
             }
             playerDeck = NULL;
         }
         else
         {
-            loadPlayer(1, controlId);
+            game->loadPlayer(1, mParent->players[1], controlId, premadeDeck);
             deckmenu->Close();
-            mGamePhase = DUEL_STATE_CHOOSE_DECK2_TO_PLAY;
+            setGamePhase(DUEL_STATE_CHOOSE_DECK2_TO_PLAY);
         }
         break;
 
     default:
-
         switch (controlId)
         {
 
         case MENUITEM_MAIN_MENU:
             menu->Close();
-            mGamePhase = DUEL_STATE_BACK_TO_MAIN_MENU;
+            setGamePhase(DUEL_STATE_BACK_TO_MAIN_MENU);
             break;
         case MENUITEM_CANCEL:
             menu->Close();
-            mGamePhase = DUEL_STATE_CANCEL;
+            setGamePhase(DUEL_STATE_CANCEL);
             break;
         case MENUITEM_MULLIGAN:
             //almosthumane - mulligan
-            game->currentPlayer->takeMulligan();
+            game->Mulligan();
             
             menu->Close();
-            mGamePhase = DUEL_STATE_CANCEL;
+            setGamePhase(DUEL_STATE_CANCEL);
             break;
-
-            //END almosthumane - mulligan
+        case MENUITEM_UNDO:
+            {
+            game->undo();
+            menu->Close();
+            setGamePhase(DUEL_STATE_CANCEL);
+            break;
+            }
+#ifdef TESTSUITE
+        case MENUITEM_LOAD:
+            {
+            std::string theGame;
+            if (JFileSystem::GetInstance()->readIntoString("test/game/timetwister.txt", theGame))
+            {
+                game->load(theGame);
+            }
+            menu->Close();
+            setGamePhase(DUEL_STATE_CANCEL);
+            break;
+            }
+#endif
         }
+            mEngine->ReleaseKey( JGE_BTN_MENU );
 
     }
 }
 
 void GameStateDuel::OnScroll(int inXVelocity, int inYVelocity)
 {
+    // ignore magnitude for now, since no action requires scrolling
     if (abs(inYVelocity) > 300)
     {
         bool flickUpwards = (inYVelocity < 0);
