@@ -20,12 +20,14 @@
 #include <JLogger.h>
 #include "Rules.h"
 #include "ModRules.h"
+#include "Credits.h"
+#include "AIPlayer.h"
 
 #ifdef NETWORK_SUPPORT
 #include <JNetwork.h>
 #endif//NETWORK_SUPPORT
 
-static const char* GAME_VERSION = "WTH?! 0.16.0 - wololo.net";
+static const char* GAME_VERSION = "WTH?! " WAGIC_VERSION_STRING " - wololo.net";
 
 enum ENUM_MENU_STATE_MAJOR
 {
@@ -52,7 +54,7 @@ enum ENUM_MENU_STATE_MINOR
 };
 
 GameStateMenu::GameStateMenu(GameApp* parent) :
-    GameState(parent)
+    GameState(parent, "menu")
 {
     mGuiController = NULL;
     subMenuController = NULL;
@@ -71,10 +73,8 @@ GameStateMenu::~GameStateMenu()
 
 void GameStateMenu::Create()
 {
-    mDip = NULL;
     mGuiController = NULL;
     mReadConf = 0;
-    mCurrentSetName[0] = 0;
 
     //load all the icon images. Menu icons are managed, so we can do this here.
     int n = 0;
@@ -97,8 +97,10 @@ void GameStateMenu::Create()
     string lang = options[Options::LANG].str;
     if (lang.size())
     {
-        lang = JGE_GET_RES("lang/") + lang + ".txt";
-        if (fileExists(lang.c_str()))
+        string langpath = "lang/";
+        langpath.append(lang);
+        langpath.append(".txt");
+        if (JFileSystem::GetInstance()->FileExists(langpath))
             langChosen = true;
     }
     if (!langChosen)
@@ -107,9 +109,10 @@ void GameStateMenu::Create()
     }
     scroller = NEW TextScroller(Fonts::MAIN_FONT, SCREEN_WIDTH / 2 - 90, SCREEN_HEIGHT - 17, 180);
     scrollerSet = 0;
-
     splashTex = NULL;
 
+    JFileSystem::GetInstance()->scanfolder("sets/", setFolders);
+    mCurrentSetFolderIndex = 0;
 }
 
 void GameStateMenu::Destroy()
@@ -134,7 +137,6 @@ void GameStateMenu::Start()
     mParent->gameType = GAME_TYPE_CLASSIC;
 
     //Manual clean up of some cache Data. Ideally those should clean themselves up, so this is kind of a hack for now
-    JFileSystem::GetInstance()->clearZipCache();
     WResourceManager::Instance()->ClearUnlocked();
 
     bgTexture = WResourceManager::Instance()->RetrieveTexture("menutitle.png", RETRIEVE_LOCK);
@@ -147,6 +149,8 @@ void GameStateMenu::Start()
         currentState = currentState | MENU_STATE_MINOR_FADEIN;
 
     wallpaper = "";
+    scrollerSet = 0; // This will force-update the scroller text
+    mPercentComplete = 0;
 }
 
 void GameStateMenu::genNbCardsStr()
@@ -179,47 +183,20 @@ void GameStateMenu::genNbCardsStr()
 void GameStateMenu::fillScroller()
 {
     scroller->Reset();
-    char buffer[4096];
     char buff2[512];
 
-    DeckStats * stats = DeckStats::GetInstance();
-	vector<DeckMetaData *> playerDecks = BuildDeckList(options.profileFile(), "", NULL, 6);
-    int totalGames = 0;
-	for (size_t j = 0; j < playerDecks.size(); j++)
-    {
-		DeckMetaData* meta = playerDecks[j];
-        if (meta)
-            meta->LoadStats();
-		sprintf(buffer, "stats/player_deck%i.txt", meta->getDeckId());
-        string deckstats = options.profileFile(buffer);
-        if (fileExists(deckstats.c_str()))
-        {
-            stats->load(deckstats.c_str());
-            int percentVictories = stats->percentVictories();
-
-			sprintf(buff2, _("You have a %i%% victory ratio with \"%s\"").c_str(), percentVictories, meta->getName().c_str());
-            scroller->Add(buff2);
-			sprintf(buff2, _("You have played %i games with \"%s\"").c_str(), meta->getGamesPlayed(), meta->getName().c_str());
-            scroller->Add(buff2);
-			totalGames += meta->getGamesPlayed();
-        }
-    }
-    if (totalGames)
-    {
-        sprintf(buff2, _("You have played a total of %i games").c_str(), totalGames);
-        scroller->Add(buff2);
-    }
-
     if (!options[Options::DIFFICULTY_MODE_UNLOCKED].number)
         scroller->Add(_("Unlock the difficult mode for more challenging duels!"));
-    if (!options[Options::MOMIR_MODE_UNLOCKED].number)
-        scroller->Add(_("Interested in playing Momir Basic? You'll have to unlock it first :)"));
-	if (!options[Options::STONEHEWER_MODE_UNLOCKED].number)
-		scroller->Add(_("Love Equipment and want a real challenge? Unlock Stone Hewer Basic:)"));
-    if (!options[Options::RANDOMDECK_MODE_UNLOCKED].number)
-        scroller->Add(_("You haven't unlocked the random deck mode yet"));
-    if (!options[Options::EVILTWIN_MODE_UNLOCKED].number)
-        scroller->Add(_("You haven't unlocked the evil twin mode yet"));
+
+    for (map<string, Unlockable *>::iterator it = Unlockable::unlockables.begin(); it !=  Unlockable::unlockables.end(); ++it) {
+        Unlockable * award = it->second;
+        if (!award->isUnlocked())
+        {
+            if (award->getValue("teaser").size())
+                scroller->Add(_(award->getValue("teaser")));
+        }
+    }
+
     if (!options[Options::RANDOMDECK_MODE_UNLOCKED].number)
         scroller->Add(_("You haven't unlocked the random deck mode yet"));
     if (!options[Options::EVILTWIN_MODE_UNLOCKED].number)
@@ -235,65 +212,76 @@ void GameStateMenu::fillScroller()
     sprintf(buff2, _("You have unlocked %i expansions out of %i").c_str(), nbunlocked, setlist.size());
     scroller->Add(buff2);
 
-    PlayerData * playerdata = NEW PlayerData(MTGCollection());
-
-    if (gModRules.general.hasDeckEditor() && gModRules.general.hasShop())
-    {
-        int totalCards = playerdata->collection->totalCards();
-        if (totalCards)
-        {
-            sprintf(buff2, _("You have a total of %i cards in your collection").c_str(), totalCards);
-            scroller->Add(buff2);
-
-            int estimatedValue = playerdata->collection->totalPrice();
-            sprintf(buff2, _("The shopkeeper would buy your entire collection for around %i credits").c_str(), estimatedValue / 2);
-            scroller->Add(buff2);
-
-            sprintf(buff2, _("The cards in your collection have an average value of %i credits").c_str(), estimatedValue / totalCards);
-            scroller->Add(buff2);
-        }
-    }
-
-    sprintf(buff2, _("You currently have %i credits").c_str(), playerdata->credits);
-    SAFE_DELETE(playerdata);
-    scroller->Add(buff2);
-
     scroller->Add(_("More cards and mods at http://wololo.net/wagic"));
 
-    scroller->Add(_("These stats will be updated next time you run Wagic"));
-
     scrollerSet = 1;
     scroller->setRandom();
 }
-void GameStateMenu::resetDirectory()
-{
-    if (mDip != NULL)
-    {
-        closedir(mDip);
-        mDip = NULL;
-    }
-}
-int GameStateMenu::nextDirectory(const char * root, const char * file)
-{
-    int found = 0;
-    if (!mDip)
-    {
-        mDip = opendir(root);
+
+int GameStateMenu::gamePercentComplete() {
+    if (mPercentComplete)
+        return mPercentComplete;
+
+    int done = 0;
+    int total = 0;
+
+    total++;
+    if (options[Options::DIFFICULTY_MODE_UNLOCKED].number)
+        done++;
+
+    for (map<string, Unlockable *>::iterator it = Unlockable::unlockables.begin(); it !=  Unlockable::unlockables.end(); ++it) {
+        total++;
+        if (it->second->isUnlocked())
+            total++;
     }
 
-    while (!found && (mDit = readdir(mDip)))
+    total++;
+    if (options[Options::RANDOMDECK_MODE_UNLOCKED].number)
+        done++;
+
+    total++;
+    if (options[Options::EVILTWIN_MODE_UNLOCKED].number)
+        done++;
+
+    //Unlocked sets
+    total+= setlist.size();
+    for (int i = 0; i < setlist.size(); i++)
     {
-        sprintf(mCurrentSetFileName, "%s/%s/%s", root, mDit->d_name, file);
-        wagic::ifstream file(mCurrentSetFileName);
-        if (file)
-        {
-            sprintf(mCurrentSetName, "%s", mDit->d_name);
-            file.close();
-            found = 1;
-        }
+        if (1 == options[Options::optionSet(i)].number)
+            done++;
     }
-    if (!found)
-        resetDirectory();
+
+    //unlocked AI decks
+    int currentlyUnlocked = options[Options::AIDECKS_UNLOCKED].number;
+    int totalAIDecks = AIPlayer::getTotalAIDecks();
+    int reallyUnlocked = MIN(currentlyUnlocked, totalAIDecks);
+    total+= totalAIDecks / 10;
+    done+= reallyUnlocked / 10;
+
+    mPercentComplete = 100 * done / total;
+    return mPercentComplete;
+}
+
+int GameStateMenu::nextSetFolder(const string & root, const string & file)
+{
+    bool found = false;
+    while (!found && (mCurrentSetFolderIndex < setFolders.size()))
+    {
+        vector<string> folders;
+        folders.push_back(root);
+        folders.push_back(setFolders[mCurrentSetFolderIndex]);
+        mCurrentSetFileName = buildFilePath(folders, file);
+        
+        if (JFileSystem::GetInstance()->FileExists(mCurrentSetFileName))
+        {
+            mCurrentSetName = setFolders[mCurrentSetFolderIndex];
+            if (mCurrentSetName.length() && (mCurrentSetName[mCurrentSetName.length() - 1] == '/'))
+                mCurrentSetName.resize(mCurrentSetName.length() - 1);
+            found = true;
+        }
+        mCurrentSetFolderIndex++;
+    }
+
     return found;
 }
 
@@ -311,9 +299,8 @@ string GameStateMenu::loadRandomWallpaper()
         return wallpaper;
 
     vector<string> wallpapers;
-    wagic::ifstream file(JGE_GET_RES("graphics/wallpapers.txt").c_str());
-
-    if (!file)
+    izfstream file;
+    if (! JFileSystem::GetInstance()->openForRead(file, "graphics/wallpapers.txt"))
         return wallpaper;
 
     string s;
@@ -325,6 +312,7 @@ string GameStateMenu::loadRandomWallpaper()
             s.erase(s.size() - 1); //Handle DOS files
         wallpapers.push_back(s);
     }
+    file.close();
 
     int rnd = rand() % (wallpapers.size());
     wallpaper = wallpapers[rnd];
@@ -356,69 +344,61 @@ void GameStateMenu::setLang(int id)
 void GameStateMenu::loadLangMenu()
 {
     LOG("GameStateMenu::loadLangMenu");
-    subMenuController = NEW SimpleMenu(MENU_LANGUAGE_SELECTION, this, Fonts::MENU_FONT, 150, 60);
+    subMenuController = NEW SimpleMenu(JGE::GetInstance(), MENU_LANGUAGE_SELECTION, this, Fonts::MENU_FONT, 150, 60);
     if (!subMenuController)
         return;
-    resetDirectory();
-    if (!mDip)
-    {
-        mDip = opendir(JGE_GET_RES("lang").c_str());
-    }
 
-    while ((mDit = readdir(mDip)))
+    vector<string> langFiles = JFileSystem::GetInstance()->scanfolder("lang/");
+    for (size_t i = 0; i < langFiles.size(); ++i)
     {
-        string filename = JGE_GET_RES("lang/");
-        filename += mDit->d_name;
-        wagic::ifstream file(filename.c_str());
+        izfstream file;
+        string filePath = "lang/";
+        filePath.append(langFiles[i]);
+        if (! JFileSystem::GetInstance()->openForRead(file, filePath))
+            continue;
+
         string s;
         string lang;
-        if (file)
+
+        if (std::getline(file, s))
         {
-            if (std::getline(file, s))
-            {
-                lang = getLang(s);
-            }
-            file.close();
+            lang = getLang(s);
         }
+
+        file.close();
+
         if (lang.size())
         {
             langChoices = true;
-            string filen = mDit->d_name;
+            string filen = langFiles[i];
             langs.push_back(filen.substr(0, filen.size() - 4));
             subMenuController->Add(langs.size(), lang.c_str());
         }
     }
-    resetDirectory();
     LOG("GameStateMenu::loadLangMenu - Done");
 }
 
 void GameStateMenu::listPrimitives()
 {
     LOG("GameStateMenu::listPrimitives");
-    resetDirectory();
-    if (!mDip)
-    {
-        mDip = opendir(JGE_GET_RES("sets/primitives/").c_str());
-    }
+    vector<string> primitiveFiles = JFileSystem::GetInstance()->scanfolder("sets/primitives/");
 
-    if (!mDip)
+    if (!primitiveFiles.size())
     {
         DebugTrace("GameStateMenu.cpp:WARNING:Primitives folder is missing");
         primitivesLoadCounter = 0;
         return;
     }
 
-    while ((mDit = readdir(mDip)))
+    for (size_t i = 0; i < primitiveFiles.size(); ++i)
     {
-        string filename = JGE_GET_RES("sets/primitives/");
-        filename += mDit->d_name;
-        wagic::ifstream file(filename.c_str());
-        if (!file)
+        string filename = "sets/primitives/";
+        filename.append(primitiveFiles[i]);
+
+        if (! JFileSystem::GetInstance()->FileExists(filename))
             continue;
-        file.close();
         primitives.push_back(filename);
     }
-    resetDirectory();
     primitivesLoadCounter = 0;
     LOG("GameStateMenu::listPrimitives - Done");
 }
@@ -427,7 +407,7 @@ void GameStateMenu::ensureMGuiController()
 {
     if (!mGuiController)
     {
-        mGuiController = NEW JGuiController(100, this);
+        mGuiController = NEW JGuiController(JGE::GetInstance(), 100, this);
         if (mGuiController)
         {
             WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
@@ -449,11 +429,26 @@ void GameStateMenu::ensureMGuiController()
                 mGuiController->Add(NEW MenuItem(
                     item->mActionId, 
                     mFont, item->mDisplayName, 
-                    startX + (i * space), 50 + SCREEN_HEIGHT / 2, 
+                    startX + (i * space), 40 + SCREEN_HEIGHT / 2, 
                     mIcons[iconId].get(), mIcons[iconId + 1].get(),
                     item->mParticleFile.c_str(), WResourceManager::Instance()->GetQuad("particles").get(),
                     (i == 0)));
             }
+
+            JQuadPtr jq = WResourceManager::Instance()->RetrieveTempQuad("button_shoulder.png");
+            if (!jq.get()) return;
+            jq->SetHFlip(false);
+            jq->SetColor(ARGB(abs(255),255,255,255));
+            mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
+            vector<ModRulesOtherMenuItem *>otherItems = gModRules.menu.other;
+            if (otherItems.size()) {
+                mGuiController->Add(NEW OtherMenuItem(
+                                       otherItems[0]->mActionId,
+                                       mFont, otherItems[0]->mDisplayName,
+                                       SCREEN_WIDTH - 64, 2,
+                                       jq.get(), jq.get(), otherItems[0]->mKey, false
+                                       ));
+            }
         }
     }
 }
@@ -500,13 +495,13 @@ void GameStateMenu::Update(float dt)
         primitivesLoadCounter = primitives.size() + 1;
         if (mReadConf)
         {
-            MTGCollection()->load(mCurrentSetFileName, mCurrentSetName);
+            MTGCollection()->load(mCurrentSetFileName.c_str(), mCurrentSetName.c_str());
         }
         else
         {
             mReadConf = 1;
         }
-        if (!nextDirectory(JGE_GET_RES("sets/").c_str(), "_cards.dat"))
+        if (!nextSetFolder("sets/", "_cards.dat"))
         {
             //Remove temporary translations
             Translator::GetInstance()->tempValues.clear();
@@ -524,10 +519,8 @@ void GameStateMenu::Update(float dt)
             splashTex = NULL;
 
             //check for deleted collection / first-timer
-            wagic::ifstream file(options.profileFile(PLAYER_COLLECTION).c_str());
-            if (file)
+            if (JFileSystem::GetInstance()->FileExists(options.profileFile(PLAYER_COLLECTION)))
             {
-                file.close();
                 currentState = MENU_STATE_MAJOR_MAINMENU;
             }
             else
@@ -538,7 +531,6 @@ void GameStateMenu::Update(float dt)
             //Reload list of unlocked sets, now that we know about the sets.
             options.reloadProfile();
             genNbCardsStr();
-            resetDirectory();
             //All major things have been loaded, resize the cache to use it as efficiently as possible
             WResourceManager::Instance()->ResetCacheLimits();
         }
@@ -614,13 +606,16 @@ void GameStateMenu::Update(float dt)
             if (!hasChosenGameType)
             {
                 currentState = MENU_STATE_MAJOR_SUBMENU;
-                subMenuController = NEW SimpleMenu(MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
+                subMenuController = NEW SimpleMenu(JGE::GetInstance(), MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
                 if (subMenuController)
                 {
                     for (size_t i = 0; i < Rules::RulesList.size(); ++i)
                     {
                         Rules * rules = Rules::RulesList[i];
-                        if (!rules->hidden && (rules->unlockOption == INVALID_OPTION || options[rules->unlockOption].number))
+                        bool unlocked = rules->unlockOption == INVALID_OPTION 
+                            ? (rules->mUnlockOptionString.size() == 0 ||  options[rules->mUnlockOptionString].number !=0)
+                            :  options[rules->unlockOption].number != 0;
+                        if (!rules->hidden && (unlocked))
                         {
                             subMenuController->Add(SUBMENUITEM_END_OFFSET + i, rules->displayName.c_str());
                         }
@@ -672,6 +667,7 @@ void GameStateMenu::RenderTopMenu()
 {
     float leftTextPos = 10;
     float rightTextPos = SCREEN_WIDTH - 10;
+    JRenderer * renderer = JRenderer::GetInstance();
 
     vector<ModRulesOtherMenuItem *>items = gModRules.menu.other;
     for (size_t i = 0; i < items.size(); ++i)
@@ -695,53 +691,13 @@ void GameStateMenu::RenderTopMenu()
     mFont->SetColor(ARGB(128,255,255,255));
     mFont->DrawString(GAME_VERSION, rightTextPos, 5, JGETEXT_RIGHT);
     mFont->DrawString(nbcardsStr, leftTextPos, 5);
+    renderer->FillRect(leftTextPos, 26, 104, 8, ARGB(255, 100, 90, 60));
+    renderer->FillRect(leftTextPos + 2, 28, (float)(gamePercentComplete()), 4, ARGB(255,220,200, 125));
+    char buf[512];
+    sprintf(buf, _("achieved: %i%%").c_str(), gamePercentComplete());
+    mFont->DrawString(buf, (leftTextPos + 104) / 2, 35, JGETEXT_CENTER);
     mFont->SetScale(1.f);
     mFont->SetColor(ARGB(255,255,255,255));
-
-    if (!items.size())
-        return;
-
-    JQuadPtr jq = WResourceManager::Instance()->RetrieveTempQuad("button_shoulder.png");
-    if (!jq.get())
-        return;
-
-    mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
-    float olds = mFont->GetScale();
-
-    for (size_t i = 0; i < items.size(); ++i)
-    {
-        ModRulesOtherMenuItem * item = items[i];
-
-        int alpha = 255;
-        if (item->mActionId == MENUITEM_TROPHIES && options.newAward())
-            alpha = (int) (sin(timeIndex) * 255);
-        
-        float xPos = SCREEN_WIDTH - 64;
-        float xTextPos = xPos + 54;
-        int textAlign = JGETEXT_RIGHT;
-        jq->SetHFlip(false);
-
-        switch(item->mKey)
-        {
-        case JGE_BTN_PREV:
-            xPos = 5;
-            xTextPos = xPos + 10;
-            textAlign = JGETEXT_LEFT;
-            jq->SetHFlip(true);
-            break;
-        default:
-            break;
-        }
-
-        jq->SetColor(ARGB(abs(alpha),255,255,255));
-        mFont->SetColor(ARGB(abs(alpha),0,0,0));
-        string s = _(item->mDisplayName);
-        mFont->SetScale(1.0f);
-        mFont->SetScale(50.0f / mFont->GetStringWidth(s.c_str()));
-        JRenderer::GetInstance()->RenderQuad(jq.get(), xPos, 2);
-        mFont->DrawString(s, xTextPos, 9, textAlign);
-        mFont->SetScale(olds);
-    }
 }
 
 void GameStateMenu::Render()
@@ -768,9 +724,9 @@ void GameStateMenu::Render()
         }
 
         char text[512];
-        if (mCurrentSetName[0])
+        if (mCurrentSetName.size())
         {
-            sprintf(text, _("LOADING SET: %s").c_str(), mCurrentSetName);
+            sprintf(text, _("LOADING SET: %s").c_str(), mCurrentSetName.c_str());
         }
         else
         {
@@ -786,7 +742,6 @@ void GameStateMenu::Render()
     }
     else
     {
-        mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
         PIXEL_TYPE colors[] = {
 
         ARGB(255,3,3,0), ARGB(255,8,8,0), ARGB(255,21,21,10), ARGB(255,50,50,30), };
@@ -831,10 +786,14 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
         switch (controlId)
         {
         case MENUITEM_PLAY:
-            subMenuController = NEW SimpleMenu(MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
+            subMenuController = NEW SimpleMenu(JGE::GetInstance(), MENU_FIRST_DUEL_SUBMENU, this, Fonts::MENU_FONT, 150, 60);
             if (subMenuController)
             {
+#ifdef NETWORK_SUPPORT
                 subMenuController->Add(SUBMENUITEM_1PLAYER, "1 Player");
+#else
+                subMenuController->Add(SUBMENUITEM_1PLAYER, "Play Game");
+#endif
                 // TODO Put 2 players mode back
                 // This requires to fix the hand (to accept 2 players) OR to implement network game
 #ifdef NETWORK_SUPPORT
@@ -846,6 +805,10 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
                 if (Rules::getRulesByFilename("testsuite.txt"))
                     subMenuController->Add(SUBMENUITEM_TESTSUITE, "Test Suite");
 #endif
+
+#ifdef AI_CHANGE_TESTING
+                subMenuController->Add(SUBMENUITEM_TESTAI, "AI A/B Testing");
+#endif
                 currentState = MENU_STATE_MAJOR_SUBMENU | MENU_STATE_MINOR_NONE;
             }
             break;
@@ -922,7 +885,16 @@ void GameStateMenu::ButtonPressed(int controllerId, int controlId)
 #endif //NETWORK_SUPPORT
             currentState = MENU_STATE_MAJOR_MAINMENU | MENU_STATE_MINOR_SUBMENU_CLOSING;
             break;
-
+#ifdef AI_CHANGE_TESTING
+        case SUBMENUITEM_TESTAI:
+             options[Options::AIDECKS_UNLOCKED].number = 5000; //hack to force-test all decks
+            mParent->players[0] = PLAYER_TYPE_CPU_TEST;
+            mParent->players[1] = PLAYER_TYPE_CPU_TEST;
+            mParent->gameType = GAME_TYPE_DEMO;
+            subMenuController->Close();
+            currentState = MENU_STATE_MAJOR_DUEL | MENU_STATE_MINOR_SUBMENU_CLOSING;
+            break;
+#endif
 #ifdef TESTSUITE
             case SUBMENUITEM_TESTSUITE:
             mParent->rules = Rules::getRulesByFilename("testsuite.txt");
@@ -961,8 +933,6 @@ ostream& GameStateMenu::toString(ostream& out) const
                  << " ; currentState : " << currentState
                  << " ; mVolume : " << mVolume
                  << " ; nbcardsStr : " << nbcardsStr
-                 << " ; mDip : " << mDip
-                 << " ; mDit : " << mDit
                  << " ; mCurrentSetName : " << mCurrentSetName
                  << " ; mCurrentSetFileName : " << mCurrentSetFileName
                  << " ; mReadConf : " << mReadConf
