@@ -12,6 +12,7 @@
 #include "TestSuiteAI.h"
 
 #include <hge/hgedistort.h>
+#include "WFont.h"
 
 float GameStateShop::_x1[] = { 79, 19, 27, 103, 154, 187, 102, 144, 198, 133, 183 };
 float GameStateShop::_y1[] = { 150, 194, 222, 167, 164, 156, 195, 190, 175, 220, 220 };
@@ -27,7 +28,9 @@ float GameStateShop::_y4[] = { 169, 188, 250, 182, 182, 168, 220, 208, 198, 259,
 
 namespace
 {
-    float kGamepadIconSize = 0.5f;
+#ifndef TOUCH_ENABLED
+    float kPspIconScaleFactor = 0.5f;
+#endif // TOUCH_ENABLED
     std::string kOtherCardsString(": Other cards");
     std::string kCreditsString("Credits: ");
 }
@@ -41,7 +44,7 @@ BoosterDisplay::BoosterDisplay(int id, GameObserver* game, int x, int y, JGuiLis
 
 bool BoosterDisplay::CheckUserInput(JButton key)
 {
-    if (JGE_BTN_UP == key || JGE_BTN_DOWN == key)
+    if (JGE_BTN_UP == key || JGE_BTN_DOWN == key || JGE_BTN_PRI == key)
         return false;
 
     return CardDisplay::CheckUserInput(key);
@@ -49,11 +52,9 @@ bool BoosterDisplay::CheckUserInput(JButton key)
 }
 
 GameStateShop::GameStateShop(GameApp* parent) :
-    GameState(parent)
+    GameState(parent, "shop")
 {
     menu = NULL;
-    for (int i = 0; i < 8; i++)
-        altThumb[i] = NULL;
     boosterDisplay = NULL;
     taskList = NULL;
     srcCards = NULL;
@@ -76,10 +77,18 @@ GameStateShop::GameStateShop(GameApp* parent) :
 
     kOtherCardsString = _(kOtherCardsString);
     kCreditsString = _(kCreditsString);
+    
+    cycleCardsButton = NEW InteractiveButton(NULL, kCycleCardsButtonId, Fonts::MAIN_FONT, "New Cards", SCREEN_WIDTH_F - 80, SCREEN_HEIGHT_F - 20, JGE_BTN_PRI);
+    
+    showCardListButton = NEW InteractiveButton(NULL, kShowCardListButtonId, Fonts::MAIN_FONT, "Show List", SCREEN_WIDTH_F - 150, SCREEN_HEIGHT_F - 20, JGE_BTN_SEC);
+    disablePurchase = false;
+    clearInput = false;
 }
 
 GameStateShop::~GameStateShop()
 {
+    SAFE_DELETE( cycleCardsButton );
+    SAFE_DELETE( showCardListButton );
     End();
 }
 
@@ -106,7 +115,7 @@ void GameStateShop::Start()
     MTGAllCards * ac = MTGCollection();
     playerdata = NEW PlayerData(ac);
     myCollection = NEW DeckDataWrapper(playerdata->collection);
-    pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), ac);
+    pricelist = NEW PriceList("settings/prices.dat", ac);
     for (int i = 0; i < SHOP_SLOTS; i++)
     {
         WGuiCardDistort * dist;
@@ -131,16 +140,6 @@ void GameStateShop::Start()
         bigDisplay->setY(135);
     }
 
-    //alternateRender doesn't lock, so lock our thumbnails for hgeDistort.
-    altThumb[0] = WResourceManager::Instance()->RetrieveTexture("artifact_thumb.jpg", RETRIEVE_LOCK);
-    altThumb[1] = WResourceManager::Instance()->RetrieveTexture("green_thumb.jpg", RETRIEVE_LOCK);
-    altThumb[2] = WResourceManager::Instance()->RetrieveTexture("blue_thumb.jpg", RETRIEVE_LOCK);
-    altThumb[3] = WResourceManager::Instance()->RetrieveTexture("red_thumb.jpg", RETRIEVE_LOCK);
-    altThumb[4] = WResourceManager::Instance()->RetrieveTexture("black_thumb.jpg", RETRIEVE_LOCK);
-    altThumb[5] = WResourceManager::Instance()->RetrieveTexture("white_thumb.jpg", RETRIEVE_LOCK);
-    altThumb[6] = WResourceManager::Instance()->RetrieveTexture("land_thumb.jpg", RETRIEVE_LOCK);
-    altThumb[7] = WResourceManager::Instance()->RetrieveTexture("gold_thumb.jpg", RETRIEVE_LOCK);
-
     for (int i = 0; i < 8; ++i)
     {
         std::ostringstream stream;
@@ -210,16 +209,16 @@ string GameStateShop::descPurchase(int controlId, bool tiny)
 void GameStateShop::beginPurchase(int controlId)
 {
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MENU_FONT);
-    mFont->SetScale(DEFAULT_MENU_FONT_SCALE);
+    mFont->SetScale(SCALE_NORMAL);
     SAFE_DELETE(menu);
     if (mInventory[controlId] <= 0)
     {
-        menu = NEW SimpleMenu(-145, this, Fonts::MENU_FONT, SCREEN_WIDTH - 300, SCREEN_HEIGHT / 2, _("Sold Out").c_str());
+        menu = NEW SimpleMenu(JGE::GetInstance(), -145, this, Fonts::MENU_FONT, SCREEN_WIDTH - 300, SCREEN_HEIGHT / 2, _("Sold Out").c_str());
         menu->Add(-1, "Ok");
     }
     else if (playerdata->credits - mPrices[controlId] < 0)
     {
-        menu = NEW SimpleMenu(-145, this, Fonts::MENU_FONT, SCREEN_WIDTH - 300, SCREEN_HEIGHT / 2, _("Not enough credits").c_str());
+        menu = NEW SimpleMenu(JGE::GetInstance(), -145, this, Fonts::MENU_FONT, SCREEN_WIDTH - 300, SCREEN_HEIGHT / 2, _("Not enough credits").c_str());
         menu->Add(-1, "Ok");
         if (options[Options::CHEATMODE].number)
         {
@@ -233,7 +232,7 @@ void GameStateShop::beginPurchase(int controlId)
             sprintf(buf, _("Purchase Booster: %i credits").c_str(), mPrices[controlId]);
         else
             sprintf(buf, _("Purchase Card: %i credits").c_str(), mPrices[controlId]);
-        menu = NEW SimpleMenu(-145, this, Fonts::MENU_FONT, SCREEN_WIDTH - 300, SCREEN_HEIGHT / 2, buf);
+        menu = NEW SimpleMenu(JGE::GetInstance(), -145, this, Fonts::MENU_FONT, SCREEN_WIDTH - 300, SCREEN_HEIGHT / 2, buf);
 
         menu->Add(controlId, "Yes");
         menu->Add(-1, "No");
@@ -441,13 +440,6 @@ void GameStateShop::End()
     SAFE_DELETE(packlist);
     deleteDisplay();
 
-    //Release alternate thumbnails.
-    for (int i = 0; i < 8; i++)
-    {
-        WResourceManager::Instance()->Release(altThumb[i]);
-        altThumb[i] = NULL;
-    }
-
     SAFE_DELETE(menu);
     SAFE_DELETE(taskList);
 }
@@ -495,12 +487,12 @@ void GameStateShop::Update(float dt)
             menu->Update(dt);
         else
         {
-            menu = NEW SimpleMenu(11, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 20);
+            menu = NEW SimpleMenu(JGE::GetInstance(), 11, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 20);
             menu->Add(22, "Ask about...");
             menu->Add(14, "Check task board");
             if (options[Options::CHEATMODE].number)
                 menu->Add(-2, "Steal 1,000 credits");
-            menu->Add(12, "Save & Back to Main Menu");
+            menu->Add(12, "Save And Exit");
             menu->Add(kCancelMenuID, "Cancel");
         }
         break;
@@ -525,9 +517,9 @@ void GameStateShop::Update(float dt)
                 {
                     if (!menu)
                     {
-                        menu = NEW SimpleMenu(11, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 20);
+                        menu = NEW SimpleMenu(JGE::GetInstance(), 11, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 20);
                         menu->Add(15, "Return to shop");
-                        menu->Add(12, "Save & Back to Main Menu");
+                        menu->Add(12, "Save And Exit");
                         menu->Add(kCancelMenuID, "Cancel");
                     }
                 }
@@ -616,13 +608,6 @@ void GameStateShop::Update(float dt)
                 taskList = NEW TaskList();
             taskList->Start();
         }
-        else if (btn == JGE_BTN_PRI)
-        {
-            srcCards->Shuffle();
-            load();
-        }
-        else if (btn == JGE_BTN_CANCEL)
-            options[Options::DISABLECARDS].number = !options[Options::DISABLECARDS].number;
         else if (boosterDisplay)
         {
             if (btn == JGE_BTN_SEC)
@@ -634,12 +619,47 @@ void GameStateShop::Update(float dt)
             }
             return;
         }
+        else if (btn == JGE_BTN_PRI) // so we don't shuffle while we view our newly purchased booster display.
+        {
+            srcCards->Shuffle();
+            load();
+            disablePurchase = false;
+            clearInput = true;
+            return;
+
+        }
+        else if (btn == JGE_BTN_CANCEL)
+            options[Options::DISABLECARDS].number = !options[Options::DISABLECARDS].number;
         else if (btn == JGE_BTN_SEC)
+		{
             bListCards = !bListCards;
+            disablePurchase = false;
+            clearInput = true;
+            return;
+		}
         else if (shopMenu)
         {
-            if (shopMenu->CheckUserInput(btn))
-                srcCards->Touch();
+#if defined (IOS) || defined (ANDROID)
+            if ((cycleCardsButton->ButtonPressed() || showCardListButton->ButtonPressed()))
+#else 
+            if ( (btn == JGE_BTN_OK) && (cycleCardsButton->ButtonPressed() || showCardListButton->ButtonPressed()))
+#endif
+            {
+                disablePurchase = true;
+                return;
+            }
+            else 
+#if defined (IOS) || defined (ANDROID)
+                if (clearInput && (btn == JGE_BTN_OK))
+                {
+                    clearInput = false;
+                    disablePurchase = false;
+                    return;
+                }
+            else
+#endif
+                if (shopMenu->CheckUserInput(btn))
+                    srcCards->Touch();
         }
         if (shopMenu)
             shopMenu->Update(dt);
@@ -664,6 +684,19 @@ void GameStateShop::deleteDisplay()
     subBooster.clear();
     SAFE_DELETE(boosterDisplay);
 }
+
+void GameStateShop::enableButtons()
+{
+    cycleCardsButton->setIsSelectionValid(true);
+    showCardListButton->setIsSelectionValid(true);
+}
+
+void GameStateShop::renderButtons()
+{
+    cycleCardsButton->Render();
+    showCardListButton->Render();
+}
+
 void GameStateShop::Render()
 {
     //Erase
@@ -690,6 +723,7 @@ void GameStateShop::Render()
 
     if (shopMenu)
         shopMenu->Render();
+    
     if (filterMenu && !filterMenu->isFinished())
         filterMenu->Render();
     else
@@ -738,10 +772,14 @@ void GameStateShop::Render()
     mFont->SetColor(ARGB(255,255,255,255));
     mFont->DrawString(stream.str(), 5, SCREEN_HEIGHT - 14);
 
+#ifndef TOUCH_ENABLED
     float len = 4 + mFont->GetStringWidth(kOtherCardsString.c_str());
-	r->RenderQuad(pspIcons[6].get(), SCREEN_WIDTH - len - kGamepadIconSize - 10, SCREEN_HEIGHT - 8, 0, kGamepadIconSize, kGamepadIconSize);
+	r->RenderQuad(pspIcons[6].get(), SCREEN_WIDTH - len - 0.5 - 10, SCREEN_HEIGHT - 8, 0, kPspIconScaleFactor, kPspIconScaleFactor);
     mFont->DrawString(kOtherCardsString, SCREEN_WIDTH - len, SCREEN_HEIGHT - 14);
-
+#else
+    enableButtons();
+#endif
+    
     mFont->SetColor(ARGB(255,255,255,0));
     mFont->DrawString(descPurchase(bigSync.getPos()).c_str(), SCREEN_WIDTH / 2, SCREEN_HEIGHT - 14, JGETEXT_CENTER);
     mFont->SetColor(ARGB(255,255,255,255));
@@ -752,12 +790,14 @@ void GameStateShop::Render()
     }
     if (menu)
         menu->Render();
+    
+    if (!filterMenu || (filterMenu && filterMenu->isFinished()))
+        renderButtons();
 }
 
 void GameStateShop::ButtonPressed(int controllerId, int controlId)
 {
     int sel = bigSync.getOffset();
-
     switch (controllerId)
     {
     case -102: //Buying something...
@@ -827,6 +867,17 @@ void GameStateShop::ButtonPressed(int controllerId, int controlId)
     menu->Close();
 }
 
+void GameStateShop::OnScroll(int inXVelocity, int inYVelocity)
+{
+    // we ignore magnitude since there isn't any scrolling in the shop
+    if (abs(inXVelocity) > 200)
+    {
+        bool flickRight = (inXVelocity >= 0);
+        if (flickRight)
+            mEngine->HoldKey_NoRepeat(JGE_BTN_PRI);
+    }
+}
+
 //ShopBooster
 ShopBooster::ShopBooster()
 {
