@@ -16,6 +16,13 @@ const float kZoom_level1 = 1.4f;
 const float kZoom_level2 = 2.2f;
 const float kZoom_level3 = 2.7f;
 
+struct True: public Exp
+{
+    static inline bool test(DamagerDamaged* ref, DamagerDamaged* test)
+    {
+        return true;
+    }
+};
 struct Left: public Exp
 {
     static inline bool test(DamagerDamaged* ref, DamagerDamaged* test)
@@ -34,20 +41,21 @@ struct Right: public Exp
 JTexture* GuiCombat::ok_tex = NULL;
 
 GuiCombat::GuiCombat(GameObserver* go) :
-    GuiLayer(), go(go), active(false), activeAtk(NULL), ok(SCREEN_WIDTH - MARGIN, 210, 1, 0, 255), enemy_avatar(SCREEN_WIDTH
+    GuiLayer(go), active(false), activeAtk(NULL), ok(SCREEN_WIDTH - MARGIN, 210, 1, 0, 255), enemy_avatar(SCREEN_WIDTH
                     - MARGIN, TOP_LINE, 2, 0, 255), cursor_pos(NONE), step(DAMAGE)
 {
-    if (NULL == ok_tex)
+    if (NULL == ok_tex && go->getResourceManager())
     {
-        ok_tex = WResourceManager::Instance()->RetrieveTexture("Ok.png", RETRIEVE_LOCK);
+        ok_tex = go->getResourceManager()->RetrieveTexture("Ok.png", RETRIEVE_LOCK);
+        // send a message out to listeners that we created the GO button and it's location
     }
 }
 
 GuiCombat::~GuiCombat()
 {
-    if (ok_tex)
+    if (ok_tex && observer->getResourceManager())
     {
-        WResourceManager::Instance()->Release(ok_tex);
+        observer->getResourceManager()->Release(ok_tex);
         ok_tex = NULL;
     }
 
@@ -104,10 +112,10 @@ void GuiCombat::validateDamage()
     switch (step)
     {
     case FIRST_STRIKE:
-        go->nextCombatStep();
+        observer->nextCombatStep();
         break;
     case DAMAGE:
-        go->nextGamePhase();
+        observer->userRequestNextGamePhase(false, false);
         break;
     default:
         cout << "COMBAT : Cannot validate damage in this phase" << endl;
@@ -162,17 +170,18 @@ void GuiCombat::removeOne(DefenserDamaged* blocker, CombatStep step)
 
 bool GuiCombat::clickOK()
 {
+    observer->logAction(observer->currentlyActing(), "combatok");
     active = activeAtk = NULL;
     cursor_pos = NONE;
     switch (step)
     {
     case BLOCKERS:
     case TRIGGERS:
-			assert(false);//this is an assert for "do i show the screen that lets you select multiple blocker damage assignment.
+        assert(false);//this is an assert for "do i show the screen that lets you select multiple blocker damage assignment.
         return false; // that should not happen
 
     case ORDER:
-        go->nextGamePhase();
+        observer->userRequestNextGamePhase(true, false);
         return true;
     case FIRST_STRIKE:
         return false;
@@ -182,62 +191,17 @@ bool GuiCombat::clickOK()
     case END_FIRST_STRIKE:
         return false;
     case END_DAMAGE:
+        JGE::GetInstance()->SendCommand("combatGuiEndDamage");
         return false; // nothing;
     }
     return false;
 }
 
-bool GuiCombat::CheckUserInput(JButton key)
+
+void GuiCombat::shiftLeft()
 {
-    if (NONE == cursor_pos)
-        return false;
-    DamagerDamaged* oldActive = active;
-    switch (key)
+    switch (cursor_pos)
     {
-    case JGE_BTN_OK:
-        if (BLK == cursor_pos)
-        {
-            if (ORDER == step)
-                go->cardClick(active->card); //  { activeAtk->card->raiseBlockerRankOrder(active->card); }
-            else
-            {
-                signed damage = activeAtk->card->stepPower(step);
-                for (vector<DamagerDamaged*>::iterator it = activeAtk->blockers.begin(); *it != active; ++it)
-                    damage -= (*it)->sumDamages();
-                signed now = active->sumDamages();
-                damage -= now;
-                if (damage > 0)
-                    addOne(active, step);
-                else if (activeAtk->card->has(Constants::DEATHTOUCH))
-                    for (; now >= 1; --now)
-                        removeOne(active, step);
-                else
-                    for (now -= active->card->toughness; now >= 0; --now)
-                        removeOne(active, step);
-            }
-        }
-        else if (ATK == cursor_pos)
-        {
-            active = activeAtk->blockers.front();
-            active->zoom = kZoom_level3;
-            cursor_pos = BLK;
-        }
-        else if (OK == cursor_pos)
-        {
-            clickOK();
-        }
-        break;
-    case JGE_BTN_CANCEL:
-        if (BLK == cursor_pos)
-        {
-            oldActive->zoom = kZoom_level2;
-            active = activeAtk;
-            cursor_pos = ATK;
-        }
-        return true;
-    case JGE_BTN_LEFT:
-        switch (cursor_pos)
-        {
         case NONE:
             break;
         case OK:
@@ -277,11 +241,14 @@ bool GuiCombat::CheckUserInput(JButton key)
             }
         }
             break;
-        }
-        break;
-    case JGE_BTN_RIGHT:
-        switch (cursor_pos)
-        {
+    }
+}
+
+
+void GuiCombat::shiftRight( DamagerDamaged* oldActive )
+{
+    switch (cursor_pos)
+    {
         case NONE:
         case OK:
             break;
@@ -320,36 +287,145 @@ bool GuiCombat::CheckUserInput(JButton key)
             }
         }
             break;
+    }
+}
+
+bool GuiCombat::didClickOnButton( Pos buttonPosition, int& x, int& y)
+{
+    int x1 = static_cast<int>(buttonPosition.x - MARGIN);
+    int y1 = static_cast<int>(buttonPosition.y);
+    int x2 = static_cast<int>(buttonPosition.x + buttonPosition.width);
+    int y2 = static_cast<int>(buttonPosition.y + buttonPosition.height);
+    if ( (x >= x1 && x < x2) && (y >= y1 && y < y2))
+        return true;
+    return false;
+}
+
+bool GuiCombat::CheckUserInput(JButton key)
+{
+    if (NONE == cursor_pos)
+        return false;
+    DamagerDamaged* oldActive = active;
+    
+    int x,y;
+    if(observer->getInput()->GetLeftClickCoordinates(x, y))
+    {
+        // determine if OK button was clicked on
+        if (ok.width)
+        {
+            if (didClickOnButton(ok, x, y))
+            {
+                cursor_pos = OK;
+            }
         }
-        break;
-    case JGE_BTN_DOWN:
-        if (ORDER == step || BLK != cursor_pos || active->sumDamages() <= 0)
+        // position selected card
+        if (BLK == cursor_pos)
+        {
+            DamagerDamaged* selectedCard = closest<True> (activeAtk->blockers, NULL, static_cast<float> (x), static_cast<float> (y));
+            // find the index into the vector where the current selected card is.
+            int c1 = 0, c2 = 0;
+            int i = 0;
+            for ( vector<DamagerDamaged*>::iterator it = activeAtk->blockers.begin(); it != activeAtk->blockers.end(); ++it)
+            {
+                if ( *it == selectedCard )
+                    c2 = i;
+                else if ( *it == active)
+                    c1 = i;
+                i++;
+            }
+            // simulate pressing the "Left/Right D-Pad" control c1 - c2 times
+            if ( c1 > c2 ) // card selected is to the left of the current active card
+            {
+                for (int x = 0; x < c1 - c2; x++)
+                    shiftLeft();
+            }
+            else if ( c1 < c2 )
+            {
+                for (int x = 0; x < c2 - c1; x++)
+                    shiftRight( oldActive );
+            }
+        }
+    }
+    
+    switch (key)
+    {
+        case JGE_BTN_OK:
+            if (BLK == cursor_pos)
+            {
+                if (ORDER == step)
+                    observer->cardClick(active->card); //  { activeAtk->card->raiseBlockerRankOrder(active->card); }
+                else
+                {
+                    signed damage = activeAtk->card->stepPower(step);
+                    for (vector<DamagerDamaged*>::iterator it = activeAtk->blockers.begin(); *it != active; ++it)
+                        damage -= (*it)->sumDamages();
+                    signed now = active->sumDamages();
+                    damage -= now;
+                    if (damage > 0)
+                        addOne(active, step);
+                    else if (activeAtk->card->has(Constants::DEATHTOUCH))
+                        for (; now >= 1; --now)
+                            removeOne(active, step);
+                    else
+                        for (now -= active->card->toughness; now >= 0; --now)
+                            removeOne(active, step);
+                }
+            }
+            else if (ATK == cursor_pos)
+            {
+                active = activeAtk->blockers.front();
+                active->zoom = kZoom_level3;
+                cursor_pos = BLK;
+            }
+            else if (OK == cursor_pos)
+            {
+                clickOK();
+            }
             break;
-        removeOne(active, step);
-        break;
-    case JGE_BTN_UP:
-        if (ORDER == step || BLK != cursor_pos)
+        case JGE_BTN_CANCEL:
+            if (BLK == cursor_pos)
+            {
+                oldActive->zoom = kZoom_level2;
+                active = activeAtk;
+                cursor_pos = ATK;
+            }
+            return true;
+        case JGE_BTN_LEFT:
+            shiftLeft();
             break;
-        addOne(active, step);
-        break;
-    case JGE_BTN_PRI:
-        active = activeAtk = NULL;
-        cursor_pos = OK;
-        break;
-    case JGE_BTN_NEXT:
-        if (!options[Options::REVERSETRIGGERS].number)
-            return false;
-        active = activeAtk = NULL;
-        cursor_pos = OK;
-        break;
-    case JGE_BTN_PREV:
-        if (options[Options::REVERSETRIGGERS].number)
-            return false;
-        active = activeAtk = NULL;
-        cursor_pos = OK;
-        break;
-    default:
-        ;
+            
+        case JGE_BTN_RIGHT:
+            shiftRight( oldActive );
+            break;
+            
+        case JGE_BTN_DOWN:
+            if (ORDER == step || BLK != cursor_pos || active->sumDamages() <= 0)
+                break;
+            removeOne(active, step);
+            break;
+        case JGE_BTN_UP:
+            if (ORDER == step || BLK != cursor_pos)
+                break;
+            addOne(active, step);
+            break;
+        case JGE_BTN_PRI:
+            active = activeAtk = NULL;
+            cursor_pos = OK;
+            break;
+        case JGE_BTN_NEXT:
+            if (!options[Options::REVERSETRIGGERS].number)
+                return false;
+            active = activeAtk = NULL;
+            cursor_pos = OK;
+            break;
+        case JGE_BTN_PREV:
+            if (options[Options::REVERSETRIGGERS].number)
+                return false;
+            active = activeAtk = NULL;
+            cursor_pos = OK;
+            break;
+        default:
+            ;
     }
     if (oldActive != active)
     {
@@ -379,18 +455,37 @@ void GuiCombat::Render()
             (*it)->Render(step);
     if (activeAtk)
     {
+        float setH = 0;
+        float setW = 0;
         signed damage = activeAtk->card->stepPower(step);
         for (vector<DefenserDamaged*>::iterator q = activeAtk->blockers.begin(); q != activeAtk->blockers.end(); ++q)
         {
             (*q)->Render(step);
             damage -= (*q)->sumDamages();
+            setH = (*q)->Height;
+            setW = (*q)->Width;
         }
         if (damage < 0)
             damage = 0;
         if (activeAtk->card->has(Constants::TRAMPLE))
         {
-            go->opponent()->mAvatar->SetHotSpot(18, 25);
-            enemy_avatar.Render(go->opponent()->mAvatar.get());
+            if(activeAtk->card->isAttacking && activeAtk->card->isAttacking != observer->opponent())
+            {
+                JQuadPtr enemy = WResourceManager::Instance()->RetrieveCard((MTGCardInstance*)activeAtk->card->isAttacking, CACHE_THUMB);
+                float oldH = enemy->mHeight;
+                float oldW = enemy->mWidth;
+                enemy->mHeight = setH;
+                enemy->mWidth = setW;
+                enemy->SetHotSpot(18, 25);
+                enemy_avatar.Render(enemy.get());
+                enemy->mHeight = oldH;
+                enemy->mWidth = oldW;
+            }
+            else
+            {
+                observer->opponent()->getIcon()->SetHotSpot(18, 25);
+                enemy_avatar.Render(observer->opponent()->getIcon().get());
+            }
             WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
             mFont->SetColor(ARGB(255, 255, 64, 0));
             {
@@ -398,6 +493,7 @@ void GuiCombat::Render()
                 sprintf(buf, "%i", damage);
                 mFont->DrawString(buf, enemy_avatar.actX - 25, enemy_avatar.actY - 40);
             }
+
         }
     }
     if (ok_tex)
@@ -417,7 +513,7 @@ void GuiCombat::Render()
 
 int GuiCombat::resolve() // Returns the number of damage objects dealt this turn.
 {
-    DamageStack* stack = NEW DamageStack();
+    DamageStack* stack = NEW DamageStack(observer);
     for (inner_iterator it = attackers.begin(); it != attackers.end(); ++it)
     {
         MTGCardInstance * attacker = (*it)->card;
@@ -429,16 +525,17 @@ int GuiCombat::resolve() // Returns the number of damage objects dealt this turn
             dmg -= (*q)->sumDamages();
         }
 
-        if (dmg > 0 && ((!attacker->blocked) || attacker->has(Constants::TRAMPLE)))
-            stack->Add(NEW Damage((*it)->card, go->opponent(), dmg, DAMAGE_COMBAT));
+        if (dmg > 0 && ((!attacker->isBlocked()) || attacker->has(Constants::TRAMPLE)))
+            stack->Add(NEW Damage(observer, (*it)->card, (Damageable*)attacker->isAttacking?(Damageable*)attacker->isAttacking:observer->opponent(), dmg, DAMAGE_COMBAT));
+
         for (vector<Damage>::iterator d = (*it)->damages.begin(); d != (*it)->damages.end(); ++d)
             stack->Add(NEW Damage(*d));
     }
     int v = stack->mObjects.size();
     if (v > 0)
     {
-        go->mLayers->stackLayer()->Add(stack);
-        go->mLayers->stackLayer()->resolve(); // This will delete the damage stack which will in turn delete the Damage it contains
+        observer->mLayers->stackLayer()->Add(stack);
+        observer->mLayers->stackLayer()->resolve(); // This will delete the damage stack which will in turn delete the Damage it contains
     }
     else
         SAFE_DELETE(stack);
@@ -506,7 +603,7 @@ int GuiCombat::receiveEventPlus(WEvent* e)
 int GuiCombat::receiveEventMinus(WEvent* e)
 {
     if (WEventZoneChange* event = dynamic_cast<WEventZoneChange*>(e))
-        if (go->players[0]->game->inPlay == event->from || go->players[1]->game->inPlay == event->from)
+        if (observer->players[0]->game->inPlay == event->from || observer->players[1]->game->inPlay == event->from)
         {
             for (inner_iterator it = attackers.begin(); it != attackers.end(); ++it)
                 if ((*it)->card == event->card->previous || (*it)->card == event->card)
@@ -515,7 +612,7 @@ int GuiCombat::receiveEventMinus(WEvent* e)
                     if (activeAtk == *it)
                         activeAtk = NULL;
                     attackers.erase(it);
-                    trash(d);
+                    observer->mTrash->trash(d);
                     return 1;
                 }
                 else
@@ -524,7 +621,7 @@ int GuiCombat::receiveEventMinus(WEvent* e)
                         {
                             DefenserDamaged* d = *q;
                             (*it)->blockers.erase(q);
-                            trash(d);
+                            observer->mTrash->trash(d);
                             return 1;
                         }
             return 0;
@@ -538,7 +635,7 @@ int GuiCombat::receiveEventMinus(WEvent* e)
             {
                 AttackerDamaged* d = *it;
                 attackers.erase(it);
-                trash(d);
+                observer->mTrash->trash(d);
                 return 1;
             }
         return 0;
@@ -552,14 +649,14 @@ int GuiCombat::receiveEventMinus(WEvent* e)
                     {
                         DefenserDamaged* d = *q;
                         (*it)->blockers.erase(q);
-                        trash(d);
+                        observer->mTrash->trash(d);
                         return 1;
                     }
         return 0;
     }
     else if (WEventPhaseChange* event = dynamic_cast<WEventPhaseChange*>(e))
     {
-        if (Constants::MTG_PHASE_COMBATDAMAGE == event->to->id)
+        if (MTG_PHASE_COMBATDAMAGE == event->to->id)
             step = BLOCKERS;
         else
             cursor_pos = NONE;
@@ -578,9 +675,9 @@ int GuiCombat::receiveEventMinus(WEvent* e)
         {
             if (ORDER == step)
                 return 0; // Why do I take this twice ? >.>
-            if (!go->currentPlayer->displayStack())
+            if (!observer->currentPlayer->displayStack())
             {
-                go->nextCombatStep();
+                observer->nextCombatStep();
                 return 1;
             }
             for (inner_iterator it = attackers.begin(); it != attackers.end(); ++it)
@@ -608,7 +705,7 @@ int GuiCombat::receiveEventMinus(WEvent* e)
                 step = ORDER;
             }
             else
-                go->nextGamePhase();
+                observer->userRequestNextGamePhase(false, false);
             return 1;
         }
         case FIRST_STRIKE:
@@ -616,22 +713,22 @@ int GuiCombat::receiveEventMinus(WEvent* e)
             for (inner_iterator attacker = attackers.begin(); attacker != attackers.end(); ++attacker)
                 if ((*attacker)->card->has(Constants::FIRSTSTRIKE) || (*attacker)->card->has(Constants::DOUBLESTRIKE))
                     goto DAMAGE;
-            go->nextCombatStep();
+            observer->nextCombatStep();
             break;
         case END_FIRST_STRIKE:
             step = END_FIRST_STRIKE;
             for (inner_iterator attacker = attackers.begin(); attacker != attackers.end(); ++attacker)
                 autoaffectDamage(*attacker, FIRST_STRIKE);
             if (0 == resolve())
-                go->nextCombatStep();
+                observer->nextCombatStep();
             //else go->mLayers->stackLayer()->AddNextGamePhase(); //uncomment to add "interrupt" offer after first strike, rather than giving priority to current player
             return 1;
         case DAMAGE:
             DAMAGE: step = event->step;
-            if (!go->currentPlayer->displayStack())
+            if (!observer->currentPlayer->displayStack())
             {
-                ((AIPlayer *) go->currentPlayer)->affectCombatDamages(step);
-                go->nextGamePhase();
+                ((AIPlayer *) observer->currentPlayer)->affectCombatDamages(step);
+                observer->userRequestNextGamePhase(false, false);
                 return 1;
             }
             for (inner_iterator attacker = attackers.begin(); attacker != attackers.end(); ++attacker)
@@ -656,12 +753,12 @@ int GuiCombat::receiveEventMinus(WEvent* e)
                 cursor_pos = ATK;
             }
             else
-				go->nextCombatStep();
+                observer->nextCombatStep();
             return 1;
         case END_DAMAGE:
             step = END_DAMAGE;
             if (0 == resolve())
-                go->nextGamePhase();
+                observer->userRequestNextGamePhase(false, false);
             return 1;
         }
     return 0;
