@@ -3,28 +3,31 @@
 #include "GameApp.h"
 #include "GuiFrame.h"
 
-GuiFrame::GuiFrame()
+GuiFrame::GuiFrame(GameObserver* observer)
+    : GuiLayer(observer)
 {
-    if (WResourceManager::Instance()->GetTexture("wood.png"))
-        wood = WResourceManager::Instance()->RetrieveQuad("wood.png", 0, 0, SCREEN_WIDTH, 28);
-    else
+    if (observer->getResourceManager())
     {
-        GameApp::systemError += "Can't load wood texture : " __FILE__ "\n";
-    }
-
-    if (WResourceManager::Instance()->GetTexture("gold.png"))
-    {
-        gold1 = WResourceManager::Instance()->RetrieveQuad("gold.png", 0, 0, SCREEN_WIDTH, 6, "gold1");
-        gold2 = WResourceManager::Instance()->RetrieveQuad("gold.png", 0, 6, SCREEN_WIDTH, 6, "gold2");
-        if (WResourceManager::Instance()->GetTexture("goldglow.png"))
-            goldGlow = WResourceManager::Instance()->RetrieveQuad("goldglow.png", 1, 1, SCREEN_WIDTH - 2, 18);
-        if (gold2)
+        if (observer->getResourceManager()->GetTexture("wood.png"))
+            wood = observer->getResourceManager()->RetrieveQuad("wood.png", 0, 0, SCREEN_WIDTH, 28);
+        else
         {
-            gold2->SetColor(ARGB(127, 255, 255, 255));
-            gold2->SetHFlip(true);
+            GameApp::systemError += "Can't load wood texture : " __FILE__ "\n";
+        }
+
+        if (observer->getResourceManager()->GetTexture("gold.png"))
+        {
+            gold1 = observer->getResourceManager()->RetrieveQuad("gold.png", 0, 0, SCREEN_WIDTH, 6, "gold1");
+            gold2 = observer->getResourceManager()->RetrieveQuad("gold.png", 0, 6, SCREEN_WIDTH, 6, "gold2");
+            if (observer->getResourceManager()->GetTexture("goldglow.png"))
+                goldGlow = observer->getResourceManager()->RetrieveQuad("goldglow.png", 1, 1, SCREEN_WIDTH - 2, 18);
+            if (gold2)
+            {
+                gold2->SetColor(ARGB(127, 255, 255, 255));
+                gold2->SetHFlip(true);
+            }
         }
     }
-
     step = 0.0;
 
 }
