@@ -1,6 +1,6 @@
 #include "PrecompiledHeader.h"
 
-#include "CardSelectorSingleton.h"
+#include "CardSelector.h"
 #include "CardSelector.h"
 #include "GameApp.h"
 #include "Trash.h"
@@ -32,14 +32,17 @@ HandLimitor::HandLimitor(GuiHand* hand) :
 {
 }
 
-GuiHand::GuiHand(MTGHand* hand) :
-    GuiLayer(), hand(hand)
+GuiHand::GuiHand(GameObserver* observer, MTGHand* hand) :
+    GuiLayer(observer), hand(hand)
 {
-    back = WResourceManager::Instance()->RetrieveTempQuad("handback.png");
-    if (back.get())
-        back->SetTextureRect(1, 0, 100, 250);
-    else
-        GameApp::systemError = "Error loading hand texture : " __FILE__;
+    if(observer->getResourceManager())
+    {
+        back = observer->getResourceManager()->RetrieveTempQuad("handback.png");
+        if (back.get())
+            back->SetTextureRect(1, 0, 100, 250);
+        else
+            GameApp::systemError = "Error loading hand texture : " __FILE__;
+    }
 }
 
 GuiHand::~GuiHand()
@@ -61,9 +64,15 @@ bool GuiHand::isInHand(CardView* card)
     return (it != cards.end());
 }
 
-GuiHandOpponent::GuiHandOpponent(MTGHand* hand) :
-    GuiHand(hand)
+GuiHandOpponent::GuiHandOpponent(GameObserver* observer, MTGHand* hand) :
+    GuiHand(observer, hand)
 {
+    vector<MTGCardInstance *>::iterator ite;
+    for(ite = hand->cards.begin(); ite != hand->cards.end(); ite++)
+    {
+        WEventZoneChange event(*ite, NULL, hand);
+        receiveEventPlus(&event);
+    }
 }
 
 void GuiHandOpponent::Render()
@@ -81,8 +90,8 @@ void GuiHandOpponent::Render()
     }
 }
 
-GuiHandSelf::GuiHandSelf(MTGHand* hand) :
-    GuiHand(hand), state(Closed), backpos(ClosedX, SCREEN_HEIGHT - 250, 1.0, 0, 255)
+GuiHandSelf::GuiHandSelf(GameObserver* observer, MTGHand* hand) :
+    GuiHand(observer, hand), state(Closed), backpos(ClosedX, SCREEN_HEIGHT - 250, 1.0, 0, 255)
 {
     limitor = NEW HandLimitor(this);
     if (OptionHandDirection::HORIZONTAL == options[Options::HANDDIRECTION].number)
@@ -92,6 +101,13 @@ GuiHandSelf::GuiHandSelf(MTGHand* hand) :
         backpos.x = SCREEN_WIDTH - 30 * 7 - 14;
         backpos.UpdateNow();
     }
+
+    vector<MTGCardInstance *>::iterator ite;
+    for(ite = hand->cards.begin(); ite != hand->cards.end(); ite++)
+    {
+        WEventZoneChange event(*ite, NULL, hand);
+        receiveEventPlus(&event);
+    }
 }
 
 GuiHandSelf::~GuiHandSelf()
@@ -162,10 +178,10 @@ bool GuiHandSelf::CheckUserInput(JButton key)
     {
         state = (Open == state ? Closed : Open);
         if (Open == state)
-            CardSelectorSingleton::Instance()->Push();
-        CardSelectorSingleton::Instance()->Limit(Open == state ? limitor : NULL, CardView::handZone);
+            observer->getCardSelector()->Push();
+        observer->getCardSelector()->Limit(Open == state ? limitor : NULL, CardView::handZone);
         if (Closed == state)
-            CardSelectorSingleton::Instance()->Pop();
+            observer->getCardSelector()->Pop();
         if (OptionHandDirection::HORIZONTAL == options[Options::HANDDIRECTION].number)
             backpos.y = Open == state ? OpenY : ClosedY;
         else
@@ -251,7 +267,7 @@ int GuiHandSelf::receiveEventPlus(WEvent* e)
                 card = NEW CardView(CardView::handZone, ev->card, ClosedRowX, 0);
             card->t = 6 * M_PI;
             cards.push_back(card);
-            CardSelectorSingleton::Instance()->Add(card);
+            observer->getCardSelector()->Add(card);
             Repos();
             return 1;
         }
@@ -266,10 +282,10 @@ int GuiHandSelf::receiveEventMinus(WEvent* e)
                 if (event->card->previous == (*it)->card)
                 {
                     CardView* cv = *it;
-                    CardSelectorSingleton::Instance()->Remove(cv);
+                    observer->getCardSelector()->Remove(cv);
                     cards.erase(it);
                     Repos();
-                    trash(cv);
+                    observer->mTrash->trash(cv);
                     return 1;
                 }
         return 1;
@@ -304,7 +320,7 @@ int GuiHandOpponent::receiveEventMinus(WEvent* e)
                 {
                     CardView* cv = *it;
                     cards.erase(it);
-                    trash(cv);
+                    observer->mTrash->trash(cv);
                     return 1;
                 }
         return 0;
