@@ -4,6 +4,7 @@
 #include "GuiPhaseBar.h"
 #include "GameObserver.h"
 #include "Translate.h"
+#include "CardSelector.h"
 
 /*
  static int colors[] =
@@ -40,17 +41,22 @@ namespace
     }
 }
 
-GuiPhaseBar::GuiPhaseBar() :
-    phase(NULL), angle(0.0f)
+GuiPhaseBar::GuiPhaseBar(GameObserver* observer) :
+    GuiLayer(observer), PlayGuiObject(0, 0, 106, 0, false),
+  phase(NULL), angle(0.0f), zoomFactor(ICONSCALE)
 {
-    JQuad * quad = NULL;
-    if ((quad = WResourceManager::Instance()->GetQuad("phasebar")) != NULL)
+    JQuadPtr quad = WResourceManager::Instance()->GetQuad("phasebar");
+    if (quad.get() != NULL)
     {
         quad->mHeight = kHeight;
         quad->mWidth = kWidth;
     }
     else
         GameApp::systemError = "Error loading phasebar texture : " __FILE__;
+
+    zoom = ICONSCALE;
+    observer->getCardSelector()->Add(this);
+
 }
 
 GuiPhaseBar::~GuiPhaseBar()
@@ -63,59 +69,81 @@ void GuiPhaseBar::Update(float dt)
         angle -= 3 * dt;
     else
         angle = 0;
+
+    if (dt > 0.05f) dt = 0.05f;
+    if(zoomFactor + 0.05f < zoom)
+    {
+      zoomFactor += dt;
+    }
+    else if (zoomFactor - 0.05f > zoom)
+    {
+      zoomFactor -= dt;
+    }
+}
+
+void GuiPhaseBar::Entering()
+{
+    mHasFocus = true;
+    zoom = 1.4f*ICONSCALE;
+}
+
+bool GuiPhaseBar::Leaving(JButton key)
+{
+    mHasFocus = false;
+    zoom = ICONSCALE;
+    return true;
 }
 
 void GuiPhaseBar::Render()
 {
-    GameObserver * g = GameObserver::GetInstance();
-    JQuad * quad = WResourceManager::Instance()->GetQuad("phasebar");
-
-    JRenderer::GetInstance()->DrawLine(0, CENTER, SCREEN_WIDTH, CENTER, ARGB(255, 255, 255, 255));
+    JQuadPtr quad = WResourceManager::Instance()->GetQuad("phasebar");
+    //uncomment to draw a hideous line across hires screens.
+   // JRenderer::GetInstance()->DrawLine(0, CENTER, SCREEN_WIDTH, CENTER, ARGB(255, 255, 255, 255));
 
     unsigned int p = (phase->id + kPhases - 4) * (int) (kWidth + 1);
-    float centerYPosition = CENTER + (kWidth / 2) * angle * ICONSCALE / (M_PI / 6) - ICONSCALE * kWidth / 4;
+    float centerYPosition = CENTER + (kWidth / 2) * angle * zoomFactor / (M_PI / 6) - zoomFactor * kWidth / 4;
     float yPos = centerYPosition;
     float scale = 0;
     for (int glyph = 3; glyph < 6; ++glyph)
     {
-        scale = ICONSCALE * sinf(angle + glyph * M_PI / 6) / 2;
-        DrawGlyph(quad, glyph, yPos, angle, p, scale);
+        scale = zoomFactor * sinf(angle + glyph * M_PI / 6) / 2;
+        DrawGlyph(quad.get(), glyph, yPos, angle, p, scale);
         yPos += kWidth * scale;
     }
 
     yPos = centerYPosition;
     for (int glyph = 2; glyph > 0; --glyph)
     {
-        scale = ICONSCALE * sinf(angle + glyph * M_PI / 6) / 2;
+        scale = zoomFactor * sinf(angle + glyph * M_PI / 6) / 2;
         yPos -= kWidth * scale;
-        DrawGlyph(quad, glyph, yPos, angle, p, scale);
+        DrawGlyph(quad.get(), glyph, yPos, angle, p, scale);
     }
 
     if (angle > 0)
     {
-        scale = ICONSCALE * sinf(angle) / 2;
+        scale = zoomFactor * sinf(angle) / 2;
         yPos -= kWidth * scale;
         float xPos = static_cast<float> (p % (kPhases * (int) (kWidth + 1)));
         quad->SetTextureRect(xPos, kHeight, kWidth, kHeight);
-        JRenderer::GetInstance()->RenderQuad(quad, 0, yPos, 0.0, scale, scale);
+        JRenderer::GetInstance()->RenderQuad(quad.get(), 0, yPos, 0.0, scale, scale);
     }
 
     //print phase name
     WFont * font = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
     string currentP = _("your turn");
     string interrupt = "";
-    if (g->currentPlayer == g->players[1])
+    if (observer->currentPlayer == observer->players[1])
     {
         currentP = _("opponent's turn");
     }
     font->SetColor(ARGB(255, 255, 255, 255));
-    if (g->currentlyActing()->isAI())
+    if (observer->currentlyActing() && observer->currentlyActing()->isAI())
     {
         font->SetColor(ARGB(255, 128, 128, 128));
     }
-    if (g->currentlyActing() != g->currentPlayer)
+    if (observer->currentlyActing() != observer->currentPlayer)
     {
-        if (g->currentPlayer == g->players[0])
+        if (observer->currentPlayer == observer->players[0])
         {
             interrupt = _(" - ") + _("opponent plays");
         }
@@ -126,7 +154,7 @@ void GuiPhaseBar::Render()
     }
 
     char buf[64];
-    sprintf(buf, _("(%s%s) %s").c_str(), currentP.c_str(), interrupt.c_str(), _(PhaseRing::phaseName(phase->id)).c_str());
+    sprintf(buf, _("(%s%s) %s").c_str(), currentP.c_str(), interrupt.c_str(), observer->phaseRing->phaseName(phase->id));
     font->DrawString(buf, SCREEN_WIDTH - 5, 2, JGETEXT_RIGHT);
 }
 
@@ -140,3 +168,8 @@ int GuiPhaseBar::receiveEventMinus(WEvent *e)
     }
     return 1;
 }
+
+ostream& GuiPhaseBar::toString(ostream& out) const
+{
+    return out << "GuiPhaseBar";
+}
