@@ -28,7 +28,6 @@ GuiAvatar::GuiAvatar(float x, float y, bool hasFocus, Player * player, Corner co
 
 void GuiAvatar::Render()
 {
-    GameObserver * game = GameObserver::GetInstance();
     JRenderer * r = JRenderer::GetInstance();
     int life = player->life;
     int poisonCount = player->poisonCount;
@@ -57,27 +56,27 @@ void GuiAvatar::Render()
     float x0 = actX;
     float y0 = actY;
 
-    if (player->mAvatar.get())
+    if (player->getIcon().get())
     {
         if (corner == BOTTOM_RIGHT)
         {
-            x0 -= player->mAvatar->mWidth * actZ;
-            y0 -= player->mAvatar->mHeight * actZ;
+            x0 -= player->getIcon()->mWidth * actZ;
+            y0 -= player->getIcon()->mHeight * actZ;
         }
         switch (corner)
         {
         case TOP_LEFT:
-            player->mAvatar->SetHotSpot(0, 0);
+            player->getIcon()->SetHotSpot(0, 0);
             break;
         case BOTTOM_RIGHT:
-            player->mAvatar->SetHotSpot(35, 50);
+            player->getIcon()->SetHotSpot(35, 50);
             break;
         }
-        player->mAvatar->SetColor(ARGB((int)actA, 255, avatarRed, avatarRed));
-        r->RenderQuad(player->mAvatar.get(), actX, actY, actT, actZ, actZ);
+        player->getIcon()->SetColor(ARGB((int)actA, 255, avatarRed, avatarRed));
+        r->RenderQuad(player->getIcon().get(), actX, actY, actT, actZ, actZ);
         if (mHasFocus)
         {
-            r->FillRect(x0, x0, player->mAvatar->mWidth * actZ, player->mAvatar->mHeight * actZ, ARGB(abs(128 - wave),255,255,255));
+            r->FillRect(x0, x0, player->getIcon()->mWidth * actZ, player->getIcon()->mHeight * actZ, ARGB(abs(128 - wave),255,255,255));
         }
     }
 
@@ -88,11 +87,11 @@ void GuiAvatar::Render()
             avatarRed = 255;
     }
 
-    if (game->currentPlayer == player)
+    if (player->getObserver()->currentPlayer == player)
         r->DrawRect(x0 - 1, y0 - 1, 36 * actZ, 51 * actZ, ARGB((int)actA, 0, 255, 0));
-    else if (game->currentActionPlayer == player)
+    else if (player->getObserver()->currentActionPlayer == player)
         r->DrawRect(x0, y0, 34 * actZ, 49 * actZ, ARGB((int)actA, 0, 0, 255));
-    if (game->isInterrupting == player)
+    if (player->getObserver()->isInterrupting == player)
         r->DrawRect(x0, y0, 34 * actZ, 49 * actZ, ARGB((int)actA, 255, 0, 0));
 
     //Life
@@ -189,7 +188,7 @@ void GuiGameZone::Render()
 
 void GuiGameZone::ButtonPressed(int controllerId, int controlId)
 {
-    GameObserver::GetInstance()->ButtonPressed(this);
+    zone->owner->getObserver()->ButtonPressed(this);
 }
 
 bool GuiGameZone::CheckUserInput(JButton key)
@@ -214,7 +213,7 @@ void GuiGameZone::Update(float dt)
         if (fabs(c->actX - c->x) < 0.01 && fabs(c->actY - c->y) < 0.01)
         {
             cards.erase(it);
-            trash(c);
+            zone->owner->getObserver()->mTrash->trash(c);
             return;
         }
     }
@@ -223,7 +222,7 @@ void GuiGameZone::Update(float dt)
 GuiGameZone::GuiGameZone(float x, float y, bool hasFocus, MTGGameZone* zone, GuiAvatars* parent) :
     GuiStatic(static_cast<float> (GuiGameZone::Height), x, y, hasFocus, parent), zone(zone)
 {
-    cd = NEW CardDisplay(0, GameObserver::GetInstance(), static_cast<int> (x), static_cast<int> (y), this);
+    cd = NEW CardDisplay(0, zone->owner->getObserver(), static_cast<int> (x), static_cast<int> (y), this);
     cd->zone = zone;
     showCards = 0;
 }
@@ -276,7 +275,7 @@ int GuiGraveyard::receiveEventMinus(WEvent* e)
                 {
                     CardView* cv = *it;
                     cards.erase(it);
-                    trash(cv);
+                    zone->owner->getObserver()->mTrash->trash(cv);
                     return 1;
                 }
     return 0;
@@ -323,7 +322,7 @@ int GuiOpponentHand::receiveEventMinus(WEvent* e)
                 {
                     CardView* cv = *it;
                     cards.erase(it);
-                    trash(cv);
+                    zone->owner->getObserver()->mTrash->trash(cv);
                     return 1;
                 }
     return 0;
