@@ -19,16 +19,18 @@ SUPPORT_OBJECT_ANALYTICS(MTGCardInstance)
 MTGCardInstance MTGCardInstance::AnyCard = MTGCardInstance();
 MTGCardInstance MTGCardInstance::NoCard = MTGCardInstance();
 
-MTGCardInstance MTGCardInstance::ExtraRules[] = { MTGCardInstance(), MTGCardInstance() };
-
 MTGCardInstance::MTGCardInstance() :
-    CardPrimitive(), MTGCard(), Damageable(0), view(NULL)
+    CardPrimitive(), MTGCard(), Damageable(0, 0), view(NULL)
 {
     initMTGCI();
 }
 MTGCardInstance::MTGCardInstance(MTGCard * card, MTGPlayerCards * arg_belongs_to) :
-    CardPrimitive(card->data), MTGCard(card), Damageable(card->data->getToughness()), view(NULL)
+    CardPrimitive(card->data), MTGCard(card), Damageable(0, card->data->getToughness()), view(NULL)
 {
+    if(arg_belongs_to)
+      if(arg_belongs_to->owner)
+        observer = arg_belongs_to->owner->getObserver();
+
     initMTGCI();
     model = card;
     attacker = 0;
@@ -49,6 +51,15 @@ MTGCardInstance::MTGCardInstance(MTGCard * card, MTGPlayerCards * arg_belongs_to
     castMethod = Constants::NOT_CAST;
 }
 
+  MTGCardInstance * MTGCardInstance::createSnapShot()
+    {
+        MTGCardInstance * snapShot = NEW MTGCardInstance(*this);
+        snapShot->previous = NULL;
+        snapShot->counters = NEW Counters(snapShot);
+        controller()->game->garbage->addCard(snapShot);
+        return snapShot;
+    }
+
 void MTGCardInstance::copy(MTGCardInstance * card)
 {
     MTGCard * source = card->model;
@@ -64,7 +75,7 @@ void MTGCardInstance::copy(MTGCardInstance * card)
 
     manaCost.copy(data->getManaCost());
 
-    text = data->text;
+    setText(""); //The text is retrieved from the data anyways
     setName(data->name);
 
     power = data->power;
@@ -77,23 +88,28 @@ void MTGCardInstance::copy(MTGCardInstance * card)
 
     //Now this is dirty...
     int backupid = mtgid;
+    int castMethodBackUP = this->castMethod;
     mtgid = source->getId();
-    Spell * spell = NEW Spell(this);
-    AbilityFactory af;
-    GameObserver * g = GameObserver::GetInstance();
-    af.addAbilities(g->mLayers->actionLayer()->getMaxId(), spell);
+    MTGCardInstance * oldStored = this->storedSourceCard;
+    Spell * spell = NEW Spell(observer, this);
+    observer = card->observer;
+    AbilityFactory af(observer);
+    af.addAbilities(observer->mLayers->actionLayer()->getMaxId(), spell);
     delete spell;
     mtgid = backupid;
+    castMethod = castMethodBackUP;
+    backupTargets = this->backupTargets;
+    storedCard = oldStored;
 }
 
 MTGCardInstance::~MTGCardInstance()
 {
     SAFE_DELETE(counters);
-	if (previous != NULL)
-	{
-		//DebugTrace("MTGCardInstance::~MTGCardInstance():  deleting " << ToHex(previous));
-	    SAFE_DELETE(previous);
-	}
+    if (previous != NULL)
+    {
+        //DebugTrace("MTGCardInstance::~MTGCardInstance():  deleting " << ToHex(previous));
+        SAFE_DELETE(previous);
+    }
 }
 
 int MTGCardInstance::init()
@@ -102,6 +118,7 @@ int MTGCardInstance::init()
     CardPrimitive::init();
     data = this;
     X = 0;
+    castX = 0;
     return 1;
 }
 
@@ -112,17 +129,13 @@ void MTGCardInstance::initMTGCI()
     isToken = false;
     lifeOrig = 0;
     doDamageTest = 1;
+    skipDamageTestOnce = false;
     belongs_to = NULL;
     tapped = 0;
     untapping = 0;
     frozen = 0;
     fresh = 0;
     isMultiColored = 0;
-    isBlackAndWhite = 0;
-    isRedAndBlue = 0;
-    isBlackAndGreen = 0;
-    isBlueAndGreen = 0;
-    isRedAndWhite = 0;
     isLeveler = 0;
     enchanted = false;
     CDenchanted = 0;
@@ -132,6 +145,7 @@ void MTGCardInstance::initMTGCI()
     morphed = false;
     turningOver = false;
     isMorphed = false;
+    isFlipped = false;
     isPhased = false;
     phasedTurn = -1;
     didattacked = 0;
@@ -143,10 +157,18 @@ void MTGCardInstance::initMTGCI()
     damageToOpponent = false;
     damageToController = false;
     wasDealtDamage = false;
+    isDualWielding = false;
     suspended = false;
     castMethod = Constants::NOT_CAST;
     mPropertiesChangedSinceLastUpdate = false;
-
+    stillNeeded = true;
+    kicked = 0;
+    chooseacolor = -1;
+    chooseasubtype = "";
+    coinSide = -1;
+    isAttacking = NULL;
+    storedCard = NULL;
+    storedSourceCard = NULL;
 
     for (int i = 0; i < ManaCost::MANA_PAID_WITH_RETRACE +1; i++)
         alternateCostPaid[i] = 0;
@@ -158,6 +180,7 @@ void MTGCardInstance::initMTGCI()
     thatmuch = 0;
     flanked = 0;
     target = NULL;
+    playerTarget = NULL;
     type_as_damageable = DAMAGEABLE_MTGCARDINSTANCE;
     banding = NULL;
     owner = NULL;
@@ -169,18 +192,16 @@ void MTGCardInstance::initMTGCI()
     regenerateTokens = 0;
     blocked = false;
     currentZone = NULL;
+    cardsAbilities = vector<MTGAbility *>();
     data = this; //an MTGCardInstance point to itself for data, allows to update it without killing the underlying database item
 
-    if (basicAbilities[(int)Constants::CHANGELING])
+    if (observer && basicAbilities[(int)Constants::CHANGELING])
     {//if the card is a changeling, it gains all creature subtypes
-        const vector<string> values = Subtypes::subtypesList->getValuesById();
+        vector<string> values = MTGAllCards::getCreatureValuesById();
         for (size_t i = 0; i < values.size(); ++i)
         {
-            if (!Subtypes::subtypesList->isSubtypeOfType(i,Subtypes::TYPE_CREATURE))
-                continue;
-
-            //Don' want to send any event to the gameObserver inside of initMCGI, so calling the parent addType method instead of mine
-            CardPrimitive::addType(i);
+            //Don' want to send any event to the gameObserver inside of initMCGI, so calling the parent setSubtype method instead of mine
+            CardPrimitive::setSubtype(values[i].c_str());
         }
     }
 
@@ -194,17 +215,6 @@ void MTGCardInstance::initMTGCI()
     if (colored > 1)
     {
         isMultiColored = 1;
-
-        if(this->hasColor(Constants::MTG_COLOR_WHITE) && this->hasColor(Constants::MTG_COLOR_BLACK))
-            isBlackAndWhite = 1;
-        if(this->hasColor(Constants::MTG_COLOR_RED) && this->hasColor(Constants::MTG_COLOR_BLUE))
-            isRedAndBlue = 1;
-        if(this->hasColor(Constants::MTG_COLOR_GREEN) && this->hasColor(Constants::MTG_COLOR_BLACK))
-            isBlackAndGreen = 1;
-        if(this->hasColor(Constants::MTG_COLOR_BLUE) && this->hasColor(Constants::MTG_COLOR_GREEN))
-            isBlueAndGreen = 1;
-        if(this->hasColor(Constants::MTG_COLOR_RED) && this->hasColor(Constants::MTG_COLOR_WHITE))
-            isRedAndWhite = 1;
     }
 
     if(previous && previous->morphed && !turningOver)
@@ -231,12 +241,11 @@ void MTGCardInstance::addType(int type)
     //This is a hack for "transform", used in combination with "losesubtypes" on Basic Lands
     //See removeType below
     if (!name.length())
-        setName(Subtypes::subtypesList->find(type));
+        setName(MTGAllCards::findType(type));
 
     WEvent * e = NEW WEventCardChangeType(this, type, before, true);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
 }
@@ -253,12 +262,12 @@ void MTGCardInstance::setType(const char * type_text)
 
 void MTGCardInstance::setSubtype(string value)
 {
-    int id = Subtypes::subtypesList->find(value);
+    int id = MTGAllCards::findType(value);
     addType(id);
 }
 int MTGCardInstance::removeType(string value, int removeAll)
 {
-    int id = Subtypes::subtypesList->find(value);
+    int id = MTGAllCards::findType(value);
     return removeType(id, removeAll);
 }
 
@@ -274,21 +283,19 @@ int MTGCardInstance::removeType(int id, int removeAll)
         // so if we remove a subtype "Forest", we also need to remove its name.
         //This means the card might lose its name, but usually when we force remove a type, we add another one just after that.
         //see "AddType" above which force sets a name if necessary
-        if (name.compare(Subtypes::subtypesList->find(id)) == 0)
+        if (name.compare(MTGAllCards::findType(id)) == 0)
             setName("");
     }
     WEvent * e = NEW WEventCardChangeType(this, id, before, after);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
     return result;
 }
 
-int MTGCardInstance::isInPlay()
+int MTGCardInstance::isInPlay(GameObserver* game)
 {
-    GameObserver * game = GameObserver::GetInstance();
     for (int i = 0; i < 2; i++)
     {
         MTGGameZone * zone = game->players[i]->game->inPlay;
@@ -300,12 +307,17 @@ int MTGCardInstance::isInPlay()
 
 int MTGCardInstance::afterDamage()
 {
+    if(skipDamageTestOnce)
+    {
+        skipDamageTestOnce = false;
+        return 0;
+    }
     if (!doDamageTest)
         return 0;
     doDamageTest = 0;
     if (!isCreature())
         return 0;
-    if (life <= 0 && isInPlay())
+    if (life <= 0 && isInPlay(observer))
     {
         return destroy();
     }
@@ -358,16 +370,14 @@ void MTGCardInstance::eventattacked()
 {
     didattacked = 1;
     WEvent * e = NEW WEventCardAttacked(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //sets card as attacked alone and sends events
 void MTGCardInstance::eventattackedAlone()
 {
     WEvent * e = NEW WEventCardAttackedAlone(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //sets card as attacked and sends events
@@ -375,8 +385,7 @@ void MTGCardInstance::eventattackednotblocked()
 {
     didattacked = 1;
     WEvent * e = NEW WEventCardAttackedNotBlocked(this);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //sets card as attacked and sends events
@@ -384,8 +393,7 @@ void MTGCardInstance::eventattackedblocked(MTGCardInstance * opponent)
 {
     didattacked = 1;
     WEvent * e = NEW WEventCardAttackedBlocked(this,opponent);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //sets card as blocking and sends events
@@ -393,8 +401,7 @@ void MTGCardInstance::eventblocked(MTGCardInstance * opponent)
 {
     didblocked = 1;
     WEvent * e = NEW WEventCardBlocked(this,opponent);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 //Taps the card
@@ -404,8 +411,7 @@ void MTGCardInstance::tap()
         return;
     tapped = 1;
     WEvent * e = NEW WEventCardTap(this, 0, 1);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 void MTGCardInstance::untap()
@@ -414,8 +420,7 @@ void MTGCardInstance::untap()
         return;
     tapped = 0;
     WEvent * e = NEW WEventCardTap(this, 1, 0);
-    GameObserver * game = GameObserver::GetInstance();
-    game->receiveEvent(e);
+    observer->receiveEvent(e);
 }
 
 void MTGCardInstance::setUntapping()
@@ -485,8 +490,7 @@ int MTGCardInstance::initAttackersDefensers()
 int MTGCardInstance::cleanup()
 {
     initAttackersDefensers();
-    GameObserver * game = GameObserver::GetInstance();
-    if (!game || game->currentPlayer == controller())
+    if (!observer || observer->currentPlayer == controller())
     {
         summoningSickness = 0;
     }
@@ -503,11 +507,15 @@ int MTGCardInstance::cleanup()
 
 int MTGCardInstance::stillInUse()
 {
-    GameObserver * game = GameObserver::GetInstance();
-    if (game->mLayers->actionLayer()->stillInUse(this))
+    if (observer->mLayers->actionLayer()->stillInUse(this))
         return 1;
     if (!previous)
         return 0;
+    if (previous->stillNeeded)
+    {
+        previous->stillNeeded = false;
+        return 1;
+    }
     return previous->stillInUse();
 }
 
@@ -532,9 +540,7 @@ int MTGCardInstance::hasSummoningSickness()
 MTGCardInstance * MTGCardInstance::changeController(Player * newController)
 {
     Player * originalOwner = controller();
-    if (originalOwner == newController)
-        return this;
-    MTGCardInstance * copy = originalOwner->game->putInZone(this, originalOwner->game->inPlay, newController->game->inPlay);
+    MTGCardInstance * copy = originalOwner->game->putInZone(this, this->currentZone, newController->game->inPlay);
     copy->summoningSickness = 1;
     return copy;
 }
@@ -554,7 +560,7 @@ int MTGCardInstance::canAttack()
         return 0;
     if (!isCreature())
         return 0;
-    if (!isInPlay())
+    if (!isInPlay(observer))
         return 0;
     return 1;
 }
@@ -583,7 +589,7 @@ int MTGCardInstance::canBlock()
         return 0;
     if (!isCreature())
         return 0;
-    if (!isInPlay())
+    if (!isInPlay(observer))
         return 0;
     return 1;
 }
@@ -731,9 +737,8 @@ int MTGCardInstance::setAttacker(int value)
         previousTarget = p;
     attacker = value;
     WEvent * e = NEW WEventCreatureAttacker(this, previousTarget, target);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
     return 1;
@@ -751,6 +756,7 @@ int MTGCardInstance::toggleAttacker()
     {
         //untap();
         setAttacker(0);
+        isAttacking = NULL;
         return 1;
     }
     return 0;
@@ -789,9 +795,8 @@ int MTGCardInstance::raiseBlockerRankOrder(MTGCardInstance * blocker)
 
     std::iter_swap(it1, it2);
     WEvent* e = NEW WEventCreatureBlockerRank(*it1, *it2, this);
-    GameObserver * go = GameObserver::GetInstance();
-    if (go)
-        go->receiveEvent(e);
+    if (observer)
+        observer->receiveEvent(e);
     else
         SAFE_DELETE(e);
     //delete(e);
@@ -814,7 +819,10 @@ int MTGCardInstance::getDefenserRank(MTGCardInstance * blocker)
 int MTGCardInstance::removeBlocker(MTGCardInstance * blocker)
 {
     blockers.remove(blocker);
-    if (!blockers.size())
+    // Blockers can be removed "manually" (by the blocking player) at the Blockers step,
+    // Or "automatically" in the damage phase, when they die and regenerate (see http://code.google.com/p/wagic/issues/detail?id=563 )
+    // In the second case, we still want the card to be marked as "blocked" this turn
+    if (!blockers.size() && observer->currentGamePhase == MTG_PHASE_COMBATBLOCKERS)
     {
         blocked = false;
     }
@@ -831,13 +839,12 @@ int MTGCardInstance::addBlocker(MTGCardInstance * blocker)
 //Returns opponents to this card for this turn. This * should * take into account banding
 MTGCardInstance * MTGCardInstance::getNextOpponent(MTGCardInstance * previous)
 {
-    GameObserver * game = GameObserver::GetInstance();
     int foundprevious = 0;
     if (!previous)
         foundprevious = 1;
     if (attacker)
     {
-        MTGInPlay * inPlay = game->opponent()->game->inPlay;
+        MTGInPlay * inPlay = observer->opponent()->game->inPlay;
         for (int i = 0; i < inPlay->nb_cards; i++)
         {
             MTGCardInstance * current = inPlay->cards[i];
@@ -857,7 +864,7 @@ MTGCardInstance * MTGCardInstance::getNextOpponent(MTGCardInstance * previous)
     }
     else if (defenser)
     {
-        MTGInPlay * inPlay = game->currentPlayer->game->inPlay;
+        MTGInPlay * inPlay = observer->currentPlayer->game->inPlay;
         for (int i = 0; i < inPlay->nb_cards; i++)
         {
             MTGCardInstance * current = inPlay->cards[i];
@@ -879,10 +886,9 @@ MTGCardInstance * MTGCardInstance::getNextOpponent(MTGCardInstance * previous)
 
 int MTGCardInstance::setDefenser(MTGCardInstance * opponent)
 {
-    GameObserver * g = GameObserver::GetInstance();
     if (defenser)
     {
-        if (g->players[0]->game->battlefield->hasCard(defenser) || g->players[1]->game->battlefield->hasCard(defenser))
+        if (observer->players[0]->game->battlefield->hasCard(defenser) || observer->players[1]->game->battlefield->hasCard(defenser))
         {
             defenser->removeBlocker(this);
         }
@@ -894,7 +900,7 @@ int MTGCardInstance::setDefenser(MTGCardInstance * opponent)
     if (defenser)
         defenser->addBlocker(this);
     if (e)
-        g->receiveEvent(e);
+        observer->receiveEvent(e);
     return 1;
 }
 
@@ -1037,22 +1043,23 @@ int MTGCardInstance::cantBeBlockedBy(MTGCardInstance * card)
 }
 
 /* Choose a sound sample to associate to that card */
-JSample * MTGCardInstance::getSample()
+const string& MTGCardInstance::getSample()
 {
-    JSample * js;
-
     if (sample.size())
-        return WResourceManager::Instance()->RetrieveSample(sample);
+        return sample;
 
     for (int i = types.size() - 1; i > 0; i--)
     {
-        string type = Subtypes::subtypesList->find(types[i]);
+        string type = MTGAllCards::findType(types[i]);
+        std::transform(type.begin(), type.end(), type.begin(), ::tolower);
         type = type + ".wav";
-        js = WResourceManager::Instance()->RetrieveSample(type);
-        if (js)
+        if(getObserver() && getObserver()->getResourceManager())
         {
-            sample = string(type);
-            return js;
+            if (getObserver()->getResourceManager()->RetrieveSample(type))
+            {
+                sample = string(type);
+                return sample;
+            }
         }
     }
 
@@ -1064,28 +1071,33 @@ JSample * MTGCardInstance::getSample()
                 continue;
             string type = Constants::MTGBasicAbilities[x];
             type = type + ".wav";
-            js = WResourceManager::Instance()->RetrieveSample(type);
-            if (js)
+            if(getObserver() && getObserver()->getResourceManager())
             {
-                sample = string(type);
-                return js;
+                if (getObserver()->getResourceManager()->RetrieveSample(type))
+                {
+                    sample = string(type);
+                    return sample;
+                }
             }
         }
     }
 
     string type = "";
     if(!types.size())
-        return NULL;   
-    type = Subtypes::subtypesList->find(types[0]);
+        return sample;   
+    type = MTGAllCards::findType(types[0]);
+    std::transform(type.begin(), type.end(), type.begin(), ::tolower);
     type.append(".wav");
-    js = WResourceManager::Instance()->RetrieveSample(type);
-    if (js)
+    if(getObserver() && getObserver()->getResourceManager())
     {
-        sample = string(type);
-        return js;
+        if (getObserver()->getResourceManager()->RetrieveSample(type))
+        {
+            sample = string(type);
+            return sample;
+        }
     }
 
-    return NULL;
+    return sample;
 }
 
 int MTGCardInstance::stepPower(CombatStep step)
@@ -1117,3 +1129,8 @@ std::ostream& operator<<(std::ostream& out, const MTGCardInstance& c)
 {
     return c.toString(out);
 }
+
+MTGCardInstance* MTGCardInstance::clone()
+{
+    return NEW MTGCardInstance(model, owner->game);
+}
