@@ -11,6 +11,7 @@
 #include "utils.h"
 #include "DeckManager.h"
 #include <iomanip>
+#include "AbilityParser.h"
 
 #if defined (WIN32) || defined (LINUX)
 #include <time.h>
@@ -94,6 +95,16 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         }
         break;
 
+    case 'b': //buyback
+        if (!primitive) primitive = NEW CardPrimitive();
+        if (ManaCost * cost = primitive->getManaCost())
+        {
+            string value = val;
+            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+            cost->BuyBack = ManaCost::parseManaCost(value);
+        }
+        break;
+
     case 'c': //color
         if (!primitive) primitive = NEW CardPrimitive();
         {
@@ -109,57 +120,6 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         }
         break;
 
-    case 'g': //grade
-        if (s.size() - i - 1 > 2) currentGrade = getGrade(val[2]);
-        break;
-
-    case 'k': //kicker
-        if (!primitive) primitive = NEW CardPrimitive();
-        if (ManaCost * cost = primitive->getManaCost())
-        {
-            string value = val;
-            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            cost->kicker = ManaCost::parseManaCost(value);
-        }
-        break;
-
-    case 'o': //othercost
-        if (!primitive) primitive = NEW CardPrimitive();
-        if(key[5] == 'r')//otherrestrictions
-        {
-            string value = val;
-            primitive->setOtherRestrictions(value);
-        }
-        else
-        {
-            if (ManaCost * cost = primitive->getManaCost())
-            {
-                string value = val;
-                std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-                size_t name = value.find("name(");
-                string theName = "";
-                    if(name != string::npos)
-                    {
-                        size_t endName = value.find(")",name);
-                        theName = value.substr(name + 5,endName - name - 5);
-                        s.erase(name, endName - name + 1);
-                    }
-                    cost->alternative = ManaCost::parseManaCost(value);
-                    if(theName.size())
-                        cost->alternative->alternativeName.append(theName);
-            }
-        }
-        break;
-
-    case 'b': //buyback
-        if (!primitive) primitive = NEW CardPrimitive();
-        if (ManaCost * cost = primitive->getManaCost())
-        {
-            string value = val;
-            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-            cost->BuyBack = ManaCost::parseManaCost(value);
-        }
-        break;
     case 'f': //flashback//morph
         {
             if (!primitive) primitive = NEW CardPrimitive();
@@ -180,11 +140,35 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             }
             break;
         }
+
+    case 'g': //grade
+        if (s.size() - i - 1 > 2) currentGrade = getGrade(val[2]);
+        break;
+
     case 'i': //id
         if (!card) card = NEW MTGCard();
         card->setMTGId(atoi(val));
         break;
 
+    case 'k': //kicker
+        if (!primitive) primitive = NEW CardPrimitive();
+        if (ManaCost * cost = primitive->getManaCost())
+        {
+            string value = val;
+            std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+            size_t multikick = value.find("multi");
+            bool isMultikicker = false;
+            if(multikick != string::npos)
+            {
+                size_t endK = value.find("{",multikick);
+                value.erase(multikick, endK - multikick);
+                isMultikicker = true;
+        }
+            cost->kicker = ManaCost::parseManaCost(value);  
+            cost->kicker->isMulti = isMultikicker;
+        }
+        break;
+
     case 'm': //mana
         if (!primitive) primitive = NEW CardPrimitive();
         {
@@ -199,6 +183,34 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         primitive->setName(val);
         break;
 
+    case 'o': //othercost/otherrestriction
+        if (!primitive) primitive = NEW CardPrimitive();
+        if(key[5] == 'r')//otherrestrictions
+        {
+            string value = val;
+            primitive->setOtherRestrictions(value);
+        }
+        else
+        {
+            if (ManaCost * cost = primitive->getManaCost())
+            {
+                string value = val;
+                std::transform(value.begin(), value.end(), value.begin(), ::tolower);
+                size_t name = value.find("name(");
+                string theName = "";
+                    if(name != string::npos)
+                    {
+                        size_t endName = value.find(")",name);
+                        theName = value.substr(name + 5,endName - name - 5);
+                        value.erase(name, endName - name + 1);
+                    }
+                    cost->alternative = ManaCost::parseManaCost(value);
+                    if(theName.size())
+                        cost->alternative->alternativeName.append(theName);
+            }
+        }
+        break;
+
     case 'p':
         if ('r' == key[1])
         { // primitive
@@ -213,13 +225,12 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         }
         break;
 
-    case 'r': //retrace/rarity
+    case 'r': //retrace/rarity//restrictions
         if('s' == key[2] && 't' == key[3])//restrictions
         {
             if (!primitive) primitive = NEW CardPrimitive();
             string value = val;
             primitive->setRestrictions(value);
-            primitive->hasRestriction = true;
         }
         else if ('e' == key[1] && 't' == key[2])
         { //retrace
@@ -237,6 +248,7 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             card->setRarity(val[0]);
         }
         break;
+
     case 's': //subtype, suspend
         {
             if (s.find("suspend") != string::npos)
@@ -263,6 +275,7 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
             }
             break;
         }
+
     case 't':
         if (!primitive) primitive = NEW CardPrimitive();
         if (0 == strcmp("target", key))
@@ -283,7 +296,11 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
         break;
 
     default:
-        DebugTrace( endl << "MTGDECK Parsing Error: " << " [" << primitive->getName() << "]" << s << std::endl);
+        if(primitive) {
+            DebugTrace( endl << "MTGDECK Parsing Error: " << " [" << primitive->getName() << "]" << s << std::endl);
+        } else {
+            DebugTrace( endl << "MTGDECK Parsing Generic Error: " << s << std::endl);
+        }
         break;
     }
 
@@ -296,8 +313,9 @@ int MTGAllCards::processConfLine(string &s, MTGCard *card, CardPrimitive * primi
 
 void MTGAllCards::initCounters()
 {
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
-        colorsCount[i] = 0;
+    colorsCount.erase(colorsCount.begin(),colorsCount.end());
+    for (int i = 0; i < Constants::NB_Colors; i++)
+        colorsCount.push_back(0);
 }
 
 void MTGAllCards::init()
@@ -308,41 +326,6 @@ void MTGAllCards::init()
     initCounters();
 }
 
-void MTGAllCards::loadPrimitives(const char *root_directory)
-{
-    DIR *dir = opendir(root_directory);
-    struct dirent *dit;
-
-    while ((dit = readdir(dir)))
-    {
-        char fullname[1000];
-        sprintf(fullname, "%s/%s", root_directory, dit->d_name);
-        wagic::ifstream file(fullname);
-        if (!file)
-            continue;
-        file.close();
-        MTGCollection()->load(fullname);
-    }
-    closedir(dir);
-}
-
-void MTGAllCards::loadSets(const char *root_directory, const char* filename)
-{
-    DIR *dir = opendir(root_directory);
-    struct dirent *dit;
-
-    while ((dit = readdir(dir)))
-    {
-        char fullname[1000];
-        sprintf(fullname, "%s/%s/%s", root_directory, dit->d_name, filename);
-        wagic::ifstream file(fullname);
-        if (!file)
-            continue;
-        file.close();
-        MTGCollection()->load(fullname, dit->d_name);
-    }
-    closedir(dir);}
-
 int MTGAllCards::load(const char * config_file, const char * set_name, int autoload)
 {
     conf_read_mode = 0;
@@ -350,33 +333,42 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
     MTGSetInfo *si = setlist.getInfo(set_id);
 
     int lineNumber = 0;
-    wagic::ifstream setFile(config_file, ios::in | ios::ate);
-    if (!setFile) return total_cards;
-
-    streampos fileSize = setFile.tellg();
-    setFile.seekg(0, ios::beg);
-
     std::string contents;
-    contents.resize((std::string::size_type) fileSize);
-    setFile.read(&contents[0], fileSize);
-    std::stringstream stream(contents);
+    izfstream file;
+    if (!JFileSystem::GetInstance()->openForRead(file, config_file))
+        return total_cards;
 
     string s;
 
-    while (true)
+    while (getline(file,s))
     {
-        if (!std::getline(stream, s)) return total_cards;
         lineNumber++;
         if (!s.size()) continue;
+        if (s[s.size() - 1] == '\r')
+            s.erase(s.size() - 1); //Handle DOS files
+        if (!s.size()) continue;
+
+        if (s.find("#AUTO_DEFINE ") == 0)
+        {
+            string toAdd = s.substr(13);
+            AutoLineMacro::AddMacro(toAdd);
+            continue;
+        }
 
-        if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); // Handle DOS files
         switch (conf_read_mode)
         {
         case MTGAllCards::READ_ANYTHING:
             if (s[0] == '[')
             {
                 currentGrade = Constants::GRADE_SUPPORTED; // Default value
-                conf_read_mode = ('m' == s[1]) ? MTGAllCards::READ_METADATA : MTGAllCards::READ_CARD; // M for metadata.
+                if (s.size() < 2)
+                {
+                    DebugTrace("FATAL: Card file incorrect");
+                }
+                else
+                {
+                    conf_read_mode = ('m' == s[1]) ? MTGAllCards::READ_METADATA : MTGAllCards::READ_CARD; // M for metadata.
+                }
             }
             else
             {
@@ -388,6 +380,7 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
                     if (!maxGrade) maxGrade = Constants::GRADE_BORDERLINE; //Default setting for grade is borderline?
                     if (fileGrade > maxGrade)
                     {
+                        file.close();
                         return total_cards;
                     }
                 }
@@ -419,6 +412,7 @@ int MTGAllCards::load(const char * config_file, const char * set_name, int autol
             continue;
         }
     }
+    file.close();
     return total_cards;
 }
 
@@ -429,12 +423,6 @@ MTGAllCards::MTGAllCards()
     init();
 }
 
-MTGAllCards::MTGAllCards(const char * config_file, const char * set_name)
-{
-    init();
-    load(config_file, set_name, 0);
-}
-
 MTGAllCards::~MTGAllCards()
 {
     for (map<int, MTGCard *>::iterator it = collection.begin(); it != collection.end(); it++)
@@ -503,7 +491,7 @@ int MTGAllCards::countByColor(int color)
 {
     if (colorsCount[color] == 0)
     {
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        for (int i = 0; i < Constants::NB_Colors; i++)
         {
             colorsCount[i] = 0;
         }
@@ -613,39 +601,76 @@ MTGCard * MTGAllCards::_(int index)
     return getCardById(ids[index]);
 }
 
-MTGCard * MTGAllCards::getCardByName(string name)
+#ifdef TESTSUITE
+void MTGAllCards::prefetchCardNameCache()
 {
-    if (!name.size()) return NULL;
-    if (name[0] == '#') return NULL;
+    map<int, MTGCard *>::iterator it;
+    for (it = collection.begin(); it != collection.end(); it++)
+    {
+        MTGCard * c = it->second;
 
-    map<string, MTGCard * >::iterator cached = mtgCardByNameCache.end();
-    if (mtgCardByNameCache.size() > 0)
-        cached = mtgCardByNameCache.find(name);
+        //Name only
+        string cardName = c->data->name;
+        std::transform(cardName.begin(), cardName.end(), cardName.begin(), ::tolower);
+        mtgCardByNameCache[cardName] = c;
+
+        //Name + set
+        int setId = c->setId;
+        MTGSetInfo* setInfo = setlist.getInfo(setId);
+        if (setInfo)
+        {
+            string setName = setInfo->getName();
+            std::transform(setName.begin(), setName.end(), setName.begin(), ::tolower);
+            cardName = cardName + " (" + setName +  ")";
+            mtgCardByNameCache[cardName] = c;
+        }
+
+        // id
+        std::stringstream out;
+        out << c->getMTGId();
+        mtgCardByNameCache[out.str()] = c;
+    }    
+}
+#endif
+
+MTGCard * MTGAllCards::getCardByName(string nameDescriptor)
+{
+    boost::mutex::scoped_lock lock(instance->mMutex);
+    if (!nameDescriptor.size()) return NULL;
+    if (nameDescriptor[0] == '#') return NULL;
+
+    std::transform(nameDescriptor.begin(), nameDescriptor.end(), nameDescriptor.begin(), ::tolower);
+
+    map<string, MTGCard * >::iterator cached = mtgCardByNameCache.find(nameDescriptor);
     
     if (cached!= mtgCardByNameCache.end())
     {
         return cached->second;
     }
 
-    int cardnb = atoi(name.c_str());
+    int cardnb = atoi(nameDescriptor.c_str());
     if (cardnb)
     {
         MTGCard * result = getCardById(cardnb);
-        mtgCardByNameCache[name] = result;
+        mtgCardByNameCache[nameDescriptor] = result;
         return result;
     }
 
-    std::transform(name.begin(), name.end(), name.begin(), ::tolower);
     int setId = -1;
-    size_t found = name.find(" (");
+    size_t found = nameDescriptor.find(" (");
+    string name = nameDescriptor;
     if (found != string::npos)
     {
-        size_t end = name.find(")");
-        string setName = name.substr(found + 2, end - found - 2);
+        size_t end = nameDescriptor.find(")");
+        string setName = nameDescriptor.substr(found + 2, end - found - 2);
         trim(setName);
-        name = name.substr(0, found);
+        name = nameDescriptor.substr(0, found);
         trim(name);
         setId = setlist[setName];
+
+        //Reconstruct a clean string "name (set)" for cache consistency
+        nameDescriptor = name + " (" + setName + ")";
+        
     }
     map<int, MTGCard *>::iterator it;
     for (it = collection.begin(); it != collection.end(); it++)
@@ -655,12 +680,12 @@ MTGCard * MTGAllCards::getCardByName(string name)
         string cardName = c->data->name;
         std::transform(cardName.begin(), cardName.end(), cardName.begin(), ::tolower);
         if (cardName.compare(name) == 0) {
-            mtgCardByNameCache[name] = c;
+            mtgCardByNameCache[nameDescriptor] = c;
             return c;
         }
 
     }
-    mtgCardByNameCache[name] = NULL;
+    mtgCardByNameCache[nameDescriptor] = NULL;
     return NULL;
 }
 
@@ -676,7 +701,7 @@ MTGDeck::MTGDeck(MTGAllCards * _allcards)
 int MTGDeck::totalPrice()
 {
     int total = 0;
-    PriceList * pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), MTGCollection());
+    PriceList * pricelist = NEW PriceList("settings/prices.dat", MTGCollection());
     map<int, int>::iterator it;
     for (it = cards.begin(); it != cards.end(); it++)
     {
@@ -696,14 +721,16 @@ MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_onl
     size_t dot = filename.find(".");
     meta_name = filename.substr(slash + 1, dot - slash - 1);
     meta_id = atoi(meta_name.substr(4).c_str());
-    wagic::ifstream file(config_file);
-    std::string s;
-    if (file)
+    std::string contents;
+    if (JFileSystem::GetInstance()->readIntoString(config_file, contents))
     {
-        while (std::getline(file, s))
+        std::stringstream stream(contents);
+        std::string s;
+        while (std::getline(stream, s))
         {
             if (!s.size()) continue;
             if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
+            if (!s.size()) continue;
             if (s[0] == '#')
             {
                 size_t found = s.find("NAME:");
@@ -770,11 +797,10 @@ MTGDeck::MTGDeck(const char * config_file, MTGAllCards * _allcards, int meta_onl
                 DebugTrace("could not find Card matching name: " << s);
             }
         }
-        file.close();
     }
     else
     {
-        //TODO Error management
+        DebugTrace("FATAL:MTGDeck.cpp:MTGDeck - can't load deck file");
     }
 }
 
@@ -797,8 +823,9 @@ int MTGDeck::addRandomCards(int howmany, int * setIds, int nbSets, int rarity, c
 {
     if (howmany <= 0) return 1;
 
-    int unallowedColors[Constants::MTG_NB_COLORS + 1];
-    for (int i = 0; i < Constants::MTG_NB_COLORS; ++i)
+    vector<int> unallowedColors;
+    unallowedColors.resize(Constants::NB_Colors + 1);
+    for (int i = 0; i < Constants::NB_Colors; ++i)
     {
         if (nbcolors)
             unallowedColors[i] = 1;
@@ -840,7 +867,7 @@ int MTGDeck::addRandomCards(int howmany, int * setIds, int nbSets, int rarity, c
 
             if (ok)
             {
-                for (int j = 0; j < Constants::MTG_NB_COLORS; ++j)
+                for (int j = 0; j < Constants::NB_Colors; ++j)
                 {
                     if (unallowedColors[j] && card->data->hasColor(j))
                     {
@@ -976,10 +1003,10 @@ int MTGDeck::save(const string& destFileName, bool useExpandedDescriptions, cons
 {
     string tmp = destFileName;
     tmp.append(".tmp"); //not thread safe
-    std::ofstream file(tmp.c_str());
-    char writer[512];
-    if (file)
+    std::ofstream file;
+    if (JFileSystem::GetInstance()->openForWrite(file, tmp))
     {
+        char writer[512];
         DebugTrace("Saving Deck: " << deckTitle << " to " << destFileName );
         if (meta_name.size())
         {
@@ -1022,8 +1049,7 @@ int MTGDeck::save(const string& destFileName, bool useExpandedDescriptions, cons
             }
         }
         file.close();
-        std::remove(destFileName.c_str());
-        rename(tmp.c_str(), destFileName.c_str());
+        JFileSystem::GetInstance()->Rename(tmp, destFileName);
     }
     return 1;
 }
@@ -1049,7 +1075,6 @@ void MTGDeck::printDetailedDeckText(std::ofstream& file )
         MTGSetInfo *setInfo = setlist.getInfo(card->setId);
         string setName = setInfo->id;
         string cardName = card->data->getName();
-        string description = card->data->getText();
 
         currentCard << "#" << nbCards << " x " << cardName << " (" << setName << "), ";
 
@@ -1059,7 +1084,7 @@ void MTGDeck::printDetailedDeckText(std::ofstream& file )
         // Add the card's types 
         vector<int>::iterator typeIter;
         for ( typeIter = card->data->types.begin(); typeIter != card->data->types.end(); ++typeIter )
-            types << Subtypes::subtypesList->find( *typeIter ) << " ";
+            types << MTGAllCards::findType( *typeIter ) << " ";
 
         currentCard << trim(types.str()) << ", ";
         types.str(""); // reset the buffer.
@@ -1069,8 +1094,8 @@ void MTGDeck::printDetailedDeckText(std::ofstream& file )
             currentCard << card->data->getPower() << "/" << card->data->getToughness() << ", ";
 
 
-        if ( card->data->hasRestriction )
-            currentCard << ", " << card->data->otherrestriction;
+        if ( card->data->getOtherRestrictions().size() )
+            currentCard << ", " << card->data->getOtherRestrictions();
 
         for (size_t x = 0; x < card->data->basicAbilities.size(); ++x)
         {
@@ -1270,7 +1295,7 @@ MTGSetInfo::MTGSetInfo(string _id)
         counts[i] = 0;
 
     char myFilename[4096];
-    sprintf(myFilename, JGE_GET_RES("sets/%s/booster.txt").c_str(), id.c_str());
+    sprintf(myFilename, "sets/%s/booster.txt", id.c_str());
     mPack = NEW MTGPack(myFilename);
     if (!mPack->isValid())
     {
