@@ -1,9 +1,10 @@
 #include "PrecompiledHeader.h"
 
 #include "MTGGamePhase.h"
+#include "GuiPhaseBar.h"
 
-MTGGamePhase::MTGGamePhase(int id) :
-    ActionElement(id)
+MTGGamePhase::MTGGamePhase(GameObserver* g, int id) :
+    ActionElement(id), observer(g)
 {
     animation = 0;
     currentState = -1;
@@ -13,8 +14,7 @@ MTGGamePhase::MTGGamePhase(int id) :
 
 void MTGGamePhase::Update(float dt)
 {
-
-    int newState = GameObserver::GetInstance()->getCurrentGamePhase();
+    int newState = observer->getCurrentGamePhase();
     if (newState != currentState)
     {
         activeState = ACTIVE;
@@ -30,32 +30,37 @@ void MTGGamePhase::Update(float dt)
     {
         activeState = INACTIVE;
         animation = 0;
-
     }
-
 }
 
-bool MTGGamePhase::CheckUserInput(JButton key)
+bool MTGGamePhase::NextGamePhase()
 {
-    GameObserver * game = GameObserver::GetInstance();
     if (activeState == INACTIVE)
     {
-        JButton trigger = (options[Options::REVERSETRIGGERS].number ? JGE_BTN_NEXT : JGE_BTN_PREV);
-        if ((trigger == key) && game->currentActionPlayer == game->currentlyActing())
+        if (observer->currentActionPlayer == observer->currentlyActing())
         {
             activeState = ACTIVE;
-            game->userRequestNextGamePhase();
+            observer->userRequestNextGamePhase();
             return true;
         }
     }
     return false;
 }
 
+
+bool MTGGamePhase::CheckUserInput(JButton key)
+{
+    JButton trigger = (options[Options::REVERSETRIGGERS].number ? JGE_BTN_NEXT : JGE_BTN_PREV);
+    if (trigger == key)
+    {
+        return NextGamePhase();
+    }
+    return false;
+}
+
 MTGGamePhase * MTGGamePhase::clone() const
 {
-    MTGGamePhase * a = NEW MTGGamePhase(*this);
-    a->isClone = 1;
-    return a;
+    return NEW MTGGamePhase(*this);
 }
 
 ostream& MTGGamePhase::toString(ostream& out) const
