@@ -80,8 +80,8 @@ MTGPlayerCards::~MTGPlayerCards()
 
 void MTGPlayerCards::beforeBeginPhase()
 {
-    delete garbage;
-    garbage = NEW MTGGameZone();
+    SAFE_DELETE(garbageLastTurn);
+    garbageLastTurn = garbage = NEW MTGGameZone();
     garbage->setOwner(this->owner);
 
     library->beforeBeginPhase();
@@ -104,6 +104,7 @@ void MTGPlayerCards::setOwner(Player * player)
     removedFromGame->setOwner(player);
     stack->setOwner(player);
     garbage->setOwner(player);
+    garbageLastTurn->setOwner(player);
     temp->setOwner(player);
 }
 
@@ -220,7 +221,7 @@ void MTGPlayerCards::drawFromLibrary()
             return;
         }
 
-        library->owner->getObserver()->gameOver = library->owner;
+        library->owner->getObserver()->setLoser(library->owner);
         return;
     }
     MTGCardInstance * toMove = library->cards[library->nb_cards - 1];
@@ -243,7 +244,8 @@ void MTGPlayerCards::drawFromLibrary()
         }
     }
 
-    putInZone(toMove, library, hand);
+    if(putInZone(toMove, library, hand))
+        toMove->currentZone = hand;
 }
 
 void MTGPlayerCards::resetLibrary()
@@ -264,6 +266,7 @@ void MTGPlayerCards::init()
     removedFromGame = NEW MTGRemovedFromGame();
     exile = removedFromGame;
     garbage = NEW MTGGameZone();
+    garbageLastTurn = garbage;
     temp = NEW MTGGameZone();
 
     playRestrictions = NEW PlayRestrictions();
@@ -298,7 +301,6 @@ MTGCardInstance * MTGPlayerCards::putInHand(MTGCardInstance * card)
     return putInZone(card, card->currentZone, card->owner->game->hand);
 }
 
-
 // Moves a card from one zone to another
 // If the card is not actually in the expected "from" zone, does nothing and returns null 
 MTGCardInstance * MTGPlayerCards::putInZone(MTGCardInstance * card, MTGGameZone * from, MTGGameZone * to)
@@ -325,17 +327,22 @@ MTGCardInstance * MTGPlayerCards::putInZone(MTGCardInstance * card, MTGGameZone
         {
             if (card->isCreature() && g->getResourceManager())
             {
-                JSample * sample = g->getResourceManager()->RetrieveSample("graveyard.wav");
-                if (sample)
-                    JSoundSystem::GetInstance()->PlaySample(sample);
+                g->getResourceManager()->PlaySample("graveyard.wav");
             }
         }
     }
 
     MTGCardInstance * ret = copy;
-
+    for(int i = 0; i < 2; ++i)
+    {
+        if(to == g->players[i]->game->library && from == g->players[i]->game->library)//if its going to the library from the library we intend to put it on top.
+        {
+            g->players[i]->game->temp->addCard(copy);
+            g->players[i]->game->library->placeOnTop.push_back(copy);
+            return ret;//don't send event
+        }
+    }
     to->addCard(copy);
-
     //The "Temp" zone are purely for code purposes, and we don't want the abilities engine to
     //Trigger when cards move in this zone
     // Additionally, when they move "from" this zone,
@@ -410,6 +417,8 @@ MTGGameZone::~MTGGameZone()
 {
     for (size_t i = 0; i < cards.size(); i++)
     {
+        cards[i]->stillNeeded = false;
+        SAFE_DELETE(cards[i]->previous);
         SAFE_DELETE( cards[i] );
     }
     cards.clear();
@@ -419,6 +428,11 @@ MTGGameZone::~MTGGameZone()
 
 void MTGGameZone::beforeBeginPhase()
 {
+    cardsSeenLastTurn.clear();
+    for(size_t k = 0; k < cardsSeenThisTurn.size(); k++)
+    {
+        cardsSeenLastTurn.push_back(cardsSeenThisTurn[k]);
+    }
     cardsSeenThisTurn.clear();
 };
 
@@ -449,6 +463,8 @@ MTGCardInstance * MTGGameZone::removeCard(MTGCardInstance * card, int createCopy
             //if (card->isToken) //TODO better than this ?
             //  return card;
             //card->lastController = card->controller();
+            if(!card)
+                return NULL;
             if (createCopy)
             {
                 copy = card->clone();
@@ -456,7 +472,10 @@ MTGCardInstance * MTGGameZone::removeCard(MTGCardInstance * card, int createCopy
                 copy->view = card->view;
                 copy->isToken = card->isToken;
                 copy->X = card->X;
+                copy->castX = card->castX;
                 copy->kicked = card->kicked;
+                copy->storedCard = card->storedCard;
+                copy->storedSourceCard = card->storedSourceCard;
 
                 //stupid bug with tokens...
                 if (card->model == card)
@@ -497,7 +516,7 @@ size_t MTGGameZone::getIndex(MTGCardInstance * card)
 unsigned int MTGGameZone::countByType(const char * value)
 {
     int result = 0;
-    int subTypeId = Subtypes::subtypesList->find(value);
+    int subTypeId = MTGAllCards::findType(value);
     for (int i = 0; i < (nb_cards); i++)
     {
         if (cards[i]->hasType(subTypeId))
@@ -547,6 +566,18 @@ bool MTGGameZone::hasType(const char * value)
     return false;
 }
 
+bool MTGGameZone::hasTypeSpecificInt(int value1,int value)
+{
+    for (int i = 0; i < (nb_cards); i++)
+    {
+        if (cards[i]->hasType(value1) && cards[i]->hasType(value))
+        {
+            return true;
+        }
+    }
+    return false;
+}
+
 bool MTGGameZone::hasPrimaryType(const char * value,const char * secondvalue)
 {
     for (int i = 0; i < (nb_cards); i++)
@@ -631,18 +662,30 @@ bool MTGGameZone::hasAbility(int ability)
     return false;
 }
 
-int MTGGameZone::seenThisTurn(TargetChooser * tc, int castMethod)
+int MTGGameZone::seenThisTurn(TargetChooser * tc, int castMethod, bool lastTurn)
 {
     //The following 2 lines modify the passed TargetChooser. Call this function with care :/
     tc->setAllZones(); // This is to allow targetting cards without caring about the actual zone
     tc->targetter = NULL;
 
     int count = 0;
-    for (vector<MTGCardInstance *>::iterator iter = cardsSeenThisTurn.begin(); iter != cardsSeenThisTurn.end(); ++iter)
+    if (lastTurn)
     {
-        MTGCardInstance * c = (*iter);
-        if (c->matchesCastFilter(castMethod) &&  tc->canTarget(c))
-            count++;
+        for (vector<MTGCardInstance *>::iterator iter = cardsSeenLastTurn.begin(); iter != cardsSeenLastTurn.end(); ++iter)
+        {
+            MTGCardInstance * c = (*iter);
+            if (c && c->matchesCastFilter(castMethod) &&  tc->canTarget(c))
+                count++;
+        }
+    }
+    else
+    {
+        for (vector<MTGCardInstance *>::iterator iter = cardsSeenThisTurn.begin(); iter != cardsSeenThisTurn.end(); ++iter)
+        {
+            MTGCardInstance * c = (*iter);
+            if (c->matchesCastFilter(castMethod) &&  tc->canTarget(c))
+                count++;
+        }
     }
     return count;
 }
@@ -651,11 +694,19 @@ int MTGGameZone::seenThisTurn(string targetChooserDefinition, int castMethod)
 {
     TargetChooserFactory tcf(owner->getObserver());
     TargetChooser *tc = tcf.createTargetChooser(targetChooserDefinition, NULL);
-    int result = seenThisTurn(tc, castMethod);
-    delete(tc);
+    int result = seenThisTurn(tc, castMethod,false);
+    SAFE_DELETE(tc);
     return result;
 }
 
+int MTGGameZone::seenLastTurn(string targetChooserDefinition, int castMethod)
+{
+    TargetChooserFactory tcf(owner->getObserver());
+    TargetChooser *tc = tcf.createTargetChooser(targetChooserDefinition, NULL);
+    int result = seenThisTurn(tc, castMethod,true);
+    SAFE_DELETE(tc);
+    return result;
+}
 
 void MTGGameZone::cleanupPhase()
 {
@@ -775,7 +826,6 @@ MTGGameZone * MTGGameZone::intToZone(int zoneId, Player * p, Player * p2)
     case STACK:
         return p->game->stack;
     }
-
     if (!p2) return NULL;
     switch (zoneId)
     {    
@@ -833,7 +883,6 @@ MTGGameZone * MTGGameZone::intToZone(GameObserver *g, int zoneId, MTGCardInstanc
 
     MTGGameZone * result = intToZone(zoneId, p, p2);
     if (result) return result;
-
     switch (zoneId)
     {
     case TARGET_OWNER_GRAVEYARD:
@@ -842,11 +891,19 @@ MTGGameZone * MTGGameZone::intToZone(GameObserver *g, int zoneId, MTGCardInstanc
         return target->owner->game->graveyard;
     case OWNER_GRAVEYARD:
         return target->owner->game->graveyard;
+    case TARGETED_PLAYER_GRAVEYARD:
+        if(source->playerTarget)
+        return source->playerTarget->game->graveyard;
+        else return source->controller()->game->graveyard;
 
     case TARGET_OWNER_BATTLEFIELD:
         return target->owner->game->inPlay;
     case OWNER_BATTLEFIELD:
         return target->owner->game->inPlay;
+    case TARGETED_PLAYER_BATTLEFIELD:
+        if(source->playerTarget)
+            return source->playerTarget->game->inPlay;
+        else return source->controller()->game->inPlay;
 
     case TARGET_OWNER_HAND:
         return target->owner->game->hand;
@@ -854,6 +911,10 @@ MTGGameZone * MTGGameZone::intToZone(GameObserver *g, int zoneId, MTGCardInstanc
         return target->owner->game->hand;
     case OWNER_HAND:
         return target->owner->game->hand;
+    case TARGETED_PLAYER_HAND:
+        if(source->playerTarget)
+            return source->playerTarget->game->hand;
+        else return source->controller()->game->hand;
 
     case TARGET_OWNER_EXILE:
         return target->owner->game->removedFromGame;
@@ -861,16 +922,28 @@ MTGGameZone * MTGGameZone::intToZone(GameObserver *g, int zoneId, MTGCardInstanc
         return target->owner->game->removedFromGame;
     case OWNER_EXILE:
         return target->owner->game->removedFromGame;
+    case TARGETED_PLAYER_EXILE:
+        if(source->playerTarget)
+            return source->playerTarget->game->removedFromGame;
+        else return source->controller()->game->removedFromGame;
 
     case TARGET_OWNER_LIBRARY:
         return target->owner->game->library;
     case OWNER_LIBRARY:
         return target->owner->game->library;
+    case TARGETED_PLAYER_LIBRARY:
+        if(source->playerTarget)
+            return source->playerTarget->game->library;
+        else return source->controller()->game->library;
 
     case TARGET_OWNER_STACK:
         return target->owner->game->stack;
     case OWNER_STACK:
         return target->owner->game->stack;
+    case TARGETED_PLAYER_STACK:
+        if(source->playerTarget)
+            return source->playerTarget->game->stack;
+        else return source->controller()->game->stack;
     default:
         return NULL;
     }
@@ -880,44 +953,44 @@ MTGGameZone * MTGGameZone::intToZone(GameObserver *g, int zoneId, MTGCardInstanc
 int MTGGameZone::zoneStringToId(string zoneName)
 {
     const char * strings[] = { "mygraveyard", "opponentgraveyard", "targetownergraveyard", "targetcontrollergraveyard",
-                    "ownergraveyard", "graveyard",
+                    "ownergraveyard", "graveyard","targetedpersonsgraveyard",
 
-                    "myinplay", "opponentinplay", "targetownerinplay", "targetcontrollerinplay", "ownerinplay", "inplay",
+                    "myinplay", "opponentinplay", "targetownerinplay", "targetcontrollerinplay", "ownerinplay", "inplay","targetedpersonsinplay",
 
                     "mybattlefield", "opponentbattlefield", "targetownerbattlefield", "targetcontrollerbattlefield",
-                    "ownerbattlefield", "battlefield",
+                    "ownerbattlefield", "battlefield","targetedpersonsbattlefield",
 
-                    "myhand", "opponenthand", "targetownerhand", "targetcontrollerhand", "ownerhand", "hand",
+                    "myhand", "opponenthand", "targetownerhand", "targetcontrollerhand", "ownerhand", "hand","targetedpersonshand",
 
-                    "mylibrary", "opponentlibrary", "targetownerlibrary", "targetcontrollerlibrary", "ownerlibrary", "library",
+                    "mylibrary", "opponentlibrary", "targetownerlibrary", "targetcontrollerlibrary", "ownerlibrary", "library","targetedpersonslibrary",
 
                     "myremovedfromgame", "opponentremovedfromgame", "targetownerremovedfromgame",
-                    "targetcontrollerremovedfromgame", "ownerremovedfromgame", "removedfromgame",
+                    "targetcontrollerremovedfromgame", "ownerremovedfromgame", "removedfromgame","targetedpersonsremovefromgame",
 
-                    "myexile", "opponentexile", "targetownerexile", "targetcontrollerexile", "ownerexile", "exile",
+                    "myexile", "opponentexile", "targetownerexile", "targetcontrollerexile", "ownerexile", "exile","targetedpersonsexile",
 
-                    "mystack", "opponentstack", "targetownerstack", "targetcontrollerstack", "ownerstack", "stack",
+                    "mystack", "opponentstack", "targetownerstack", "targetcontrollerstack", "ownerstack", "stack","targetedpersonsstack",
 
     };
 
     int values[] = { MY_GRAVEYARD, OPPONENT_GRAVEYARD, TARGET_OWNER_GRAVEYARD, TARGET_CONTROLLER_GRAVEYARD, OWNER_GRAVEYARD,
-                    GRAVEYARD,
+                    GRAVEYARD,TARGETED_PLAYER_GRAVEYARD,
 
                     MY_BATTLEFIELD, OPPONENT_BATTLEFIELD, TARGET_OWNER_BATTLEFIELD, TARGET_CONTROLLER_BATTLEFIELD,
-                    OWNER_BATTLEFIELD, BATTLEFIELD,
+                    OWNER_BATTLEFIELD, BATTLEFIELD,TARGETED_PLAYER_BATTLEFIELD,
 
                     MY_BATTLEFIELD, OPPONENT_BATTLEFIELD, TARGET_OWNER_BATTLEFIELD, TARGET_CONTROLLER_BATTLEFIELD,
-                    OWNER_BATTLEFIELD, BATTLEFIELD,
+                    OWNER_BATTLEFIELD, BATTLEFIELD,TARGETED_PLAYER_BATTLEFIELD,
 
-                    MY_HAND, OPPONENT_HAND, TARGET_OWNER_HAND, TARGET_CONTROLLER_HAND, OWNER_HAND, HAND,
+                    MY_HAND, OPPONENT_HAND, TARGET_OWNER_HAND, TARGET_CONTROLLER_HAND, OWNER_HAND, HAND,TARGETED_PLAYER_HAND,
 
-                    MY_LIBRARY, OPPONENT_LIBRARY, TARGET_OWNER_LIBRARY, TARGET_CONTROLLER_LIBRARY, OWNER_LIBRARY, LIBRARY,
+                    MY_LIBRARY, OPPONENT_LIBRARY, TARGET_OWNER_LIBRARY, TARGET_CONTROLLER_LIBRARY, OWNER_LIBRARY, LIBRARY,TARGETED_PLAYER_LIBRARY,
 
-                    MY_EXILE, OPPONENT_EXILE, TARGET_OWNER_EXILE, TARGET_CONTROLLER_EXILE, OWNER_EXILE, EXILE,
+                    MY_EXILE, OPPONENT_EXILE, TARGET_OWNER_EXILE, TARGET_CONTROLLER_EXILE, OWNER_EXILE, EXILE,TARGETED_PLAYER_EXILE,
 
-                    MY_EXILE, OPPONENT_EXILE, TARGET_OWNER_EXILE, TARGET_CONTROLLER_EXILE, OWNER_EXILE, EXILE,
+                    MY_EXILE, OPPONENT_EXILE, TARGET_OWNER_EXILE, TARGET_CONTROLLER_EXILE, OWNER_EXILE, EXILE,TARGETED_PLAYER_EXILE,
 
-                    MY_STACK, OPPONENT_STACK, TARGET_OWNER_STACK, TARGET_CONTROLLER_STACK, OWNER_STACK, STACK, };
+                    MY_STACK, OPPONENT_STACK, TARGET_OWNER_STACK, TARGET_CONTROLLER_STACK, OWNER_STACK, STACK,TARGETED_PLAYER_STACK };
 
     int max = sizeof(values) / sizeof *(values);
 
