@@ -274,8 +274,6 @@ MTGEventBonus * MTGEventBonus::clone() const
 {
     return NEW MTGEventBonus(*this);
 }
-
-//
 MTGPutInPlayRule::MTGPutInPlayRule(GameObserver* observer, int _id) :
 PermanentAbility(observer, _id)
 {
@@ -452,6 +450,7 @@ int MTGPutInPlayRule::reactToClick(MTGCardInstance * card)
         if (!card->has(Constants::STORM))
         {
             copy->X = spell->computeX(copy);
+            copy->castX = copy->X;
         }
     }
 
@@ -584,6 +583,7 @@ int MTGKickerRule::reactToClick(MTGCardInstance * card)
         if (!card->has(Constants::STORM))
         {
             copy->X = spell->computeX(copy);
+            copy->castX = copy->X;
         }
     }
 
@@ -738,6 +738,7 @@ int MTGAlternativeCostRule::reactToClick(MTGCardInstance * card, ManaCost *alter
         {
             ManaCost * c = spellCost->Diff(alternateCost);
             copy->X = c->getCost(Constants::NB_Colors);
+            copy->castX = copy->X;
             delete c;
         }
     }
@@ -908,8 +909,15 @@ int MTGSuspendRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
     Player * player = game->currentlyActing();
     ManaCost * alternateManaCost = card->getManaCost()->suspend;
 
-    if (!player->game->hand->hasCard(card))
+    if (!player->game->hand->hasCard(card) || !alternateManaCost)
         return 0;
+    suspendmenu = "suspend";
+    char buffer[20];
+    if(alternateManaCost->hasX())
+        sprintf(buffer,"- X");
+    else
+        sprintf(buffer,"-%i",card->suspendedTime);
+    suspendmenu.append(buffer);
     return MTGAlternativeCostRule::isReactingToClick( card, mana, alternateManaCost  );
 }
 
@@ -990,6 +998,11 @@ int MTGSuspendRule::reactToClick(MTGCardInstance * card)
     return 1;
 }
 
+const char * MTGSuspendRule::getMenuText()
+{
+    return suspendmenu.c_str();
+}
+
 ostream& MTGSuspendRule::toString(ostream& out) const
 {
     out << "MTGSuspendRule ::: (";
@@ -1097,7 +1110,7 @@ int MTGMorphCostRule::reactToClick(MTGCardInstance * card)
     spell = game->mLayers->stackLayer()->addSpell(copy, NULL, spellCost, payResult, 0);
     spell->source->morphed = true;
     spell->source->isMorphed = true;
-    spell->source->name = "";
+    spell->source->name = "Morph";
     spell->source->power = 2;
     spell->source->toughness = 2;
     copy->morphed = true;
@@ -1107,6 +1120,7 @@ int MTGMorphCostRule::reactToClick(MTGCardInstance * card)
     if (!card->has(Constants::STORM))
     {
         copy->X = spell->computeX(copy);
+        copy->castX = copy->X;
     }
     return 1;
 }
@@ -1176,7 +1190,7 @@ int MTGAttackRule::receiveEvent(WEvent *e)
             for (int i = 0; i < z->nb_cards; i++)
             {
                 MTGCardInstance * card = z->cards[i];
-                if (!card->isAttacker() && card->has(Constants::MUSTATTACK))
+                if (!card->isAttacker() && !event->from->isExtra && card->has(Constants::MUSTATTACK))//cards are only required to attack in the real attack phase of a turn.
                     reactToClick(card);
                 if (!card->isAttacker() && card->has(Constants::TREASON) && p->isAI())
                     reactToClick(card);
@@ -1218,6 +1232,121 @@ MTGAttackRule * MTGAttackRule::clone() const
 {
     return NEW MTGAttackRule(*this);
 }
+//handling for planeswalker attacking choice
+MTGPlaneswalkerAttackRule::MTGPlaneswalkerAttackRule(GameObserver* observer, int _id) :
+PermanentAbility(observer, _id)
+{
+    aType = MTGAbility::MTG_ATTACK_RULE;
+}
+
+int MTGPlaneswalkerAttackRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
+{
+    if (currentPhase == MTG_PHASE_COMBATATTACKERS && card->controller() == game->currentPlayer && card->controller() == game->currentlyActing())//on my turn and when I am the acting player.
+    {
+        if(!card->controller()->opponent()->game->inPlay->hasType("planeswalker"))
+            return 0;
+        if(card->isPhased)
+            return 0;
+        if (card->isAttacker())
+            return 1;
+        if (card->canAttack())
+            return 1;
+    }
+    return 0;
+}
+
+int MTGPlaneswalkerAttackRule::reactToClick(MTGCardInstance * card)
+{
+    if (!isReactingToClick(card))
+        return 0;
+    //Graphically select the next card that can attack
+    if (!card->isAttacker())
+    {
+        game->getCardSelector()->PushLimitor();
+        game->getCardSelector()->Limit(this, CardView::playZone);
+        game->getCardSelector()->CheckUserInput(JGE_BTN_RIGHT);
+        game->getCardSelector()->Limit(NULL, CardView::playZone);
+        game->getCardSelector()->PopLimitor();
+    }
+
+    vector<MTGAbility*>selection;
+    MTGCardInstance * check = NULL;
+    int checkWalkers = card->controller()->opponent()->game->battlefield->cards.size();
+    for(int i = 0; i < checkWalkers;++i)
+    {
+        check = card->controller()->opponent()->game->battlefield->cards[i];
+        if(check->hasType(Subtypes::TYPE_PLANESWALKER))
+        {
+            MTGAbility * setPw = NEW AAPlaneswalkerAttacked(game, game->mLayers->actionLayer()->getMaxId(), card,check);
+            MTGAbility * setWalker = setPw->clone();
+            setWalker->oneShot = true;
+            selection.push_back(setWalker);
+            SAFE_DELETE(setPw);
+        }
+    }
+
+
+    if(selection.size())
+    {
+        MTGAbility * a1 = NEW MenuAbility(game, this->GetId(), card, card,false,selection);
+        game->mLayers->actionLayer()->currentActionCard = card;
+        a1->resolve();
+    }
+
+    return 1;
+}
+
+MTGPlaneswalkerAttackRule * MTGPlaneswalkerAttackRule::clone() const
+{
+    return NEW MTGPlaneswalkerAttackRule(*this);
+}
+
+bool MTGPlaneswalkerAttackRule::select(Target* t)
+{
+    if (CardView* c = dynamic_cast<CardView*>(t))
+    {
+        MTGCardInstance * card = c->getCard();
+        if (card->canAttack() && !card->isPhased)
+            return true;
+    }
+    return false;
+}
+bool MTGPlaneswalkerAttackRule::greyout(Target* t)
+{
+    return true;
+}
+
+//setting combat against planeswalker menu handling
+ AAPlaneswalkerAttacked::AAPlaneswalkerAttacked(GameObserver* observer, int id, MTGCardInstance * card, MTGCardInstance * _target):
+    InstantAbility(observer, id, source)
+{
+    this->target = _target;
+    menuText = _target->name.c_str();
+    attacker = card;
+}
+
+int AAPlaneswalkerAttacked::resolve()
+{
+    if(!attacker)
+        return 0;
+    attacker->isAttacking = this->target;
+    attacker->toggleAttacker();
+    return 1;
+}
+
+const char* AAPlaneswalkerAttacked::getMenuText()
+{
+    return menuText.c_str();
+}
+
+AAPlaneswalkerAttacked * AAPlaneswalkerAttacked::clone() const
+{
+    return NEW AAPlaneswalkerAttacked(*this);
+}
+
+AAPlaneswalkerAttacked::~AAPlaneswalkerAttacked()
+{
+}
 
 //this rules handles returning cards to combat triggers for activations.
 MTGCombatTriggersRule::MTGCombatTriggersRule(GameObserver* observer, int _id) :
@@ -1365,6 +1494,9 @@ MTGBlockRule::MTGBlockRule(GameObserver* observer, int _id) :
 PermanentAbility(observer, _id)
 {
     aType = MTGAbility::MTG_BLOCK_RULE;
+    tcb = NULL;
+    blocker = NULL;
+    blockAbility = NULL;
 }
 
 int MTGBlockRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
@@ -1374,7 +1506,13 @@ int MTGBlockRule::isReactingToClick(MTGCardInstance * card, ManaCost * mana)
         )
     {
         if (card->canBlock() && !card->isPhased)
+        {
+            if(card->isDefenser())
+                blockmenu = "Remove Blocker";
+            else
+                blockmenu = "Assign To Block";
             return 1;
+        }
     }
     return 0;
 }
@@ -1386,17 +1524,45 @@ int MTGBlockRule::reactToClick(MTGCardInstance * card)
     MTGCardInstance * currentOpponent = card->isDefenser();
     bool result = false;
     int canDefend = 0;
-    while (!result)
+    if(card->controller()->isHuman())
     {
-        currentOpponent = game->currentPlayer->game->inPlay->getNextAttacker(currentOpponent);
-        canDefend = card->toggleDefenser(currentOpponent);
+        if(!currentOpponent)
+        {
 
-        DebugTrace("Defenser Toggle: " << card->getName() << endl
-            << "- canDefend: " << (canDefend == 0) << endl
-            << "- currentOpponent: " << currentOpponent);
-        result = (canDefend || currentOpponent == NULL);
+            SAFE_DELETE(blockAbility);
+            TargetChooserFactory tf(card->getObserver());
+            tcb = tf.createTargetChooser("blockable",card);
+            tcb->targetter = NULL;
+            blocker = NEW AABlock(card->getObserver(),-1,card,NULL);
+            blocker->oneShot = true;
+            blocker->forceDestroy = 1;
+            blocker->canBeInterrupted = false;
+            blockAbility = NEW GenericTargetAbility(game, "choose attacker","blockers",-1, card,tcb, blocker);
+            blockAbility->oneShot = true;
+            blockAbility->forceDestroy = 1;
+            blockAbility->canBeInterrupted = false;
+            return blockAbility->reactToTargetClick(card);
+        }
+        else
+            card->toggleDefenser(NULL);
     }
-    return 1;
+    else
+        while (!result)
+        {
+            currentOpponent = game->currentPlayer->game->inPlay->getNextAttacker(currentOpponent);
+            canDefend = card->toggleDefenser(currentOpponent);
+
+            DebugTrace("Defenser Toggle: " << card->getName() << endl
+                << "- canDefend: " << (canDefend == 0) << endl
+                << "- currentOpponent: " << currentOpponent);
+            result = (canDefend || currentOpponent == NULL);
+        }
+        return 1;
+}
+
+const char * MTGBlockRule::getMenuText()
+{
+    return blockmenu.c_str();
 }
 
 ostream& MTGBlockRule::toString(ostream& out) const
@@ -1409,6 +1575,12 @@ MTGBlockRule * MTGBlockRule::clone() const
 {
     return NEW MTGBlockRule(*this);
 }
+
+MTGBlockRule::~MTGBlockRule()
+{
+    SAFE_DELETE(blockAbility);
+}
+
 //
 // Attacker chooses blockers order
 //
@@ -1822,9 +1994,10 @@ int MTGPersistRule::receiveEvent(WEvent * event)
     {
         WEventZoneChange * e = (WEventZoneChange *) event;
         MTGCardInstance * card = e->card->previous;
-        if (card && card->basicAbilities[(int)Constants::PERSIST] && !card->counters->hasCounter(-1, -1))
+        if (!card) return 0;
+        int ok = 0;
+        if((card->basicAbilities[(int)Constants::PERSIST] && !card->counters->hasCounter(-1, -1))||(card->basicAbilities[(int)Constants::UNDYING] && !card->counters->hasCounter(1, 1)))
         {
-            int ok = 0;
             for (int i = 0; i < 2; i++)
             {
                 Player * p = game->players[i];
@@ -1833,20 +2006,24 @@ int MTGPersistRule::receiveEvent(WEvent * event)
             }
             if (!ok)
                 return 0;
+
             for (int i = 0; i < 2; i++)
             {
                 Player * p = game->players[i];
                 if (e->to == p->game->graveyard)
                 {
-					MTGCardInstance * copy = p->game->putInZone(e->card, p->game->graveyard, e->card->owner->game->temp);
+                    MTGCardInstance * copy = p->game->putInZone(e->card, p->game->graveyard, e->card->owner->game->temp);
                     if (!copy)
                     {
-                        DebugTrace("MTGRULES: couldn't move card for persist");
+                        DebugTrace("MTGRULES: couldn't move card for persist/undying");
                         return 0;
                     }
                     Spell * spell = NEW Spell(game, copy);
                     spell->resolve();
-                    spell->source->counters->addCounter(-1, -1);
+                    if(card->basicAbilities[(int)Constants::PERSIST])
+                        spell->source->counters->addCounter(-1, -1);
+                    else
+                        spell->source->counters->addCounter(1,1);
                     delete spell;
                     return 1;
                 }
@@ -2140,6 +2317,49 @@ MTGPlaneWalkerRule * MTGPlaneWalkerRule::clone() const
 {
     return NEW MTGPlaneWalkerRule(*this);
 }
+/* planeswalker damage rule */
+MTGPlaneswalkerDamage::MTGPlaneswalkerDamage(GameObserver* observer, int _id) :
+PermanentAbility(observer, _id)
+{
+}
+;
+
+int MTGPlaneswalkerDamage::receiveEvent(WEvent * event)
+{
+    
+    if (event->type == WEvent::DAMAGE)
+    {
+        WEventDamage * e = (WEventDamage *) event;
+        Damage * d = e->damage;
+        MTGCardInstance * card = dynamic_cast<MTGCardInstance*>(e->getTarget(WEvent::TARGET_TO));
+        if (d->damage > 0 && card && card->hasType(Subtypes::TYPE_PLANESWALKER))
+        {
+            int howMany = d->damage;
+            for(int k = 0;k < howMany;k++)
+            {
+                card->counters->removeCounter("loyalty",0,0);
+            }
+            d->damage = 0;
+            return 1;
+        }
+    }
+    if (WEventCounters * removel = dynamic_cast<WEventCounters*>(event))
+    {
+        if(removel->removed && removel->targetCard && removel->targetCard->hasType(Subtypes::TYPE_PLANESWALKER))
+            if(!removel->targetCard->counters->hasCounter("loyalty",0,0))
+            {
+                removel->targetCard->bury();
+                return 1;
+            }
+
+    }
+    return 0;
+}
+
+MTGPlaneswalkerDamage * MTGPlaneswalkerDamage::clone() const
+{
+    return NEW MTGPlaneswalkerDamage(*this);
+}
 
 /* Lifelink */
 MTGLifelinkRule::MTGLifelinkRule(GameObserver* observer, int _id) :
