@@ -16,6 +16,7 @@ SUPPORT_OBJECT_ANALYTICS(ManaCost)
 ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstance * c)
 {
     ManaCost * manaCost;
+    GameObserver* g = c?c->getObserver():NULL;
     if (_manaCost)
     {
         manaCost = _manaCost;
@@ -24,6 +25,7 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
     {
         manaCost = NEW ManaCost();
     }
+    manaCost->xColor = -1;
     int state = 0;
     size_t start = 0;
     size_t end = 0;
@@ -75,9 +77,8 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                 }
                 else
                 {
-
                     //Parse target for extraCosts
-                    TargetChooserFactory tcf;
+                    TargetChooserFactory tcf(g);
                     TargetChooser * tc = NULL;
                     size_t target_start = value.find("(");
                     size_t target_end = value.find(")");
@@ -90,9 +91,29 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                     //switch on the first letter. If two costs share their first letter, add an "if" within the switch
                     switch (value[0])
                     {
+                    case 'X':
                     case 'x':
-                        manaCost->x();
+                        if(value == "x" || value == "X")
+                            manaCost->x();
+                        else
+                        {
+                            vector<string>colorSplit = parseBetween(value,"x:"," ",false);
+                            if(colorSplit.size())
+                            {
+                                int color = -1;
+                                const string ColorStrings[] = { Constants::kManaColorless, Constants::kManaGreen, Constants::kManaBlue, Constants::kManaRed, Constants::kManaBlack, Constants::kManaWhite };
+                                for (unsigned int i = 0; i < sizeof(ColorStrings)/sizeof(ColorStrings[0]); ++i)
+                                {
+                                    if (s.find(ColorStrings[i]) != string::npos)
+                                    {
+                                        color = i;
+                                    }
+                                }
+                                manaCost->specificX(color);
+                            }
+                        }
                         break;
+
                     case 't': //Tap
                         if (value == "t")
                         {
@@ -143,10 +164,12 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                         manaCost->addExtraCost(NEW MillCost(tc));
                         break;
                     case 'n': //return unblocked attacker cost
-                        TargetChooserFactory tcf;
-                        tc = tcf.createTargetChooser("creature|myBattlefield", c);
-                        manaCost->addExtraCost(NEW Ninja(tc));
-                        break;
+                        {
+                            TargetChooserFactory tcf(g);
+                            tc = tcf.createTargetChooser("creature|myBattlefield", c);
+                            manaCost->addExtraCost(NEW Ninja(tc));
+                            break;
+                        }
                     case 'p' :
                         {
                             SAFE_DELETE(tc);
@@ -157,38 +180,55 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                             break;
                         }
                     case 'q':
+                        if(value == "q")
                         {
                             manaCost->addExtraCost(NEW UnTapCost);
+                        }
+                        else
+                        {
+                            manaCost->addExtraCost(NEW UnTapTargetCost(tc));
+                        }
+                        break;
+                    case 'c': //Counters or cycle
+                        {
+                            if(value == "chosencolor")
+                            {
+                                if(c)
+                                manaCost->add(c->chooseacolor, 1);
+                            }
+                            else if(value == "cycle")
+                            {
+                                manaCost->addExtraCost(NEW CycleCost(tc));
+                            }
+                            else
+                            {
+                                size_t counter_start = value.find("(");
+                                size_t counter_end = value.find(")", counter_start);
+                                AbilityFactory abf(g);
+                                string counterString = value.substr(counter_start + 1, counter_end - counter_start - 1);
+                                Counter * counter = abf.parseCounter(counterString, c);
+                                size_t separator = value.find(",", counter_start);
+                                size_t separator2 = string::npos;
+                                if (separator != string::npos)
+                                {
+                                    separator2 = value.find(",", counter_end + 1);
+                                }
+                                SAFE_DELETE(tc);
+                                size_t target_start = string::npos;
+                                if (separator2 != string::npos)
+                                {
+                                    target_start = value.find(",", counter_end + 1);
+                                }
+                                size_t target_end = value.length();
+                                if (target_start != string::npos && target_end != string::npos)
+                                {
+                                    string target = value.substr(target_start + 1, target_end - 1 - target_start);
+                                    tc = tcf.createTargetChooser(target, c);
+                                }
+                                manaCost->addExtraCost(NEW CounterCost(counter, tc));
+                            }
                             break;
                         }
-                    case 'c': //Counters
-                    {
-                        size_t counter_start = value.find("(");
-                        size_t counter_end = value.find(")", counter_start);
-                        AbilityFactory abf;
-                        string counterString = value.substr(counter_start + 1, counter_end - counter_start - 1);
-                        Counter * counter = abf.parseCounter(counterString, c);
-                        size_t separator = value.find(",", counter_start);
-                        size_t separator2 = string::npos;
-                        if (separator != string::npos)
-                        {
-                            separator2 = value.find(",", separator + 1);
-                        }
-                        SAFE_DELETE(tc);
-                        size_t target_start = string::npos;
-                        if (separator2 != string::npos)
-                        {
-                            target_start = value.find(",", separator2 + 1);
-                        }
-                        size_t target_end = counter_end;
-                        if (target_start != string::npos && target_end != string::npos)
-                        {
-                            string target = value.substr(target_start + 1, target_end - 1 - target_start);
-                            tc = tcf.createTargetChooser(target, c);
-                        }
-                        manaCost->addExtraCost(NEW CounterCost(counter, tc));
-                        break;
-                    }
                     default: //uncolored cost and hybrid costs
                     {
                         int intvalue = atoi(value.c_str());
@@ -206,7 +246,7 @@ ManaCost * ManaCost::parseManaCost(string s, ManaCost * _manaCost, MTGCardInstan
                                 }
                                 else
                                 {
-                                    for (int j = 0; j < Constants::MTG_NB_COLORS; j++)
+                                    for (int j = 0; j < Constants::NB_Colors; j++)
                                     {
                                         if (c == Constants::MTGColorChars[j])
                                         {
@@ -243,7 +283,7 @@ ManaCost::ManaCost()
     init();
 }
 
-ManaCost::ManaCost(int8_t _cost[], int nb_elems)
+ManaCost::ManaCost(vector<int8_t>& _cost, int nb_elems)
 {
     init();
     for (int i = 0; i < nb_elems; i++)
@@ -260,13 +300,15 @@ ManaCost::ManaCost(ManaCost * manaCost)
     init();
     if ( !manaCost ) 
         return;
-    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
         cost[i] = manaCost->getCost(i);
     }
     hybrids = manaCost->hybrids;
 
     kicker = NEW ManaCost( manaCost->kicker );
+    if(kicker)
+    kicker->isMulti = manaCost->isMulti;
     Retrace = NEW ManaCost( manaCost->Retrace );
     BuyBack = NEW ManaCost( manaCost->BuyBack );
     alternative = NEW ManaCost( manaCost->alternative );
@@ -274,8 +316,8 @@ ManaCost::ManaCost(ManaCost * manaCost)
     morph = NEW ManaCost( manaCost->morph );
     suspend = NEW ManaCost( manaCost->suspend );
 
-    // TODO: Need to figure out if a deep copy is necessary
-    extraCosts = manaCost->extraCosts;
+    extraCosts = manaCost->extraCosts ? manaCost->extraCosts->clone() : NULL;
+    xColor = manaCost->xColor;
 }
 
 // Copy Constructor 
@@ -285,9 +327,9 @@ ManaCost::ManaCost(const ManaCost& manaCost)
     : InstanceCounter<ManaCost>(manaCost)
 #endif
 {
-    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
-        cost[i] = manaCost.cost[i];      
+        cost.push_back(manaCost.cost[i]);      
     }
  
     hybrids = manaCost.hybrids;
@@ -301,8 +343,8 @@ ManaCost::ManaCost(const ManaCost& manaCost)
     morph = NEW ManaCost( manaCost.morph );
     suspend = NEW ManaCost( manaCost.suspend );
     
-    // TODO: Need to figure out if a deep copy is necessary
-    extraCosts = manaCost.extraCosts;
+    extraCosts = manaCost.extraCosts ? manaCost.extraCosts->clone() : NULL;
+    xColor = manaCost.xColor;
 }
 
 // operator=
@@ -310,7 +352,7 @@ ManaCost & ManaCost::operator= (const ManaCost & manaCost)
 {
     if ( this != &manaCost )
     {
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        for (int i = 0; i < Constants::NB_Colors; i++)
             cost[i] = manaCost.cost[i];
 
         hybrids = manaCost.hybrids;
@@ -322,6 +364,7 @@ ManaCost & ManaCost::operator= (const ManaCost & manaCost)
         FlashBack = manaCost.FlashBack;
         morph = manaCost.morph;
         suspend = manaCost.suspend;
+        xColor = manaCost.xColor;
     }
     return *this;
 }
@@ -336,25 +379,81 @@ ManaCost::~ManaCost()
     SAFE_DELETE(Retrace);
     SAFE_DELETE(morph);
     SAFE_DELETE(suspend);
+
+    cost.erase(cost.begin() ,cost.end());
 }
 
 void ManaCost::x()
 {
-    cost[Constants::MTG_NB_COLORS] = 1;
+    if (cost.size() <= (size_t)Constants::NB_Colors)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return;
+    }
+
+    cost[Constants::NB_Colors] = 1;
 }
 
 int ManaCost::hasX()
 {
-    return cost[Constants::MTG_NB_COLORS];
+    if (cost.size() <= (size_t)Constants::NB_Colors)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return 0;
+    }
+    if(xColor > 0)
+        return 0;
+
+    return cost[Constants::NB_Colors];
+}
+
+void ManaCost::specificX(int color)
+{
+    if (cost.size() <= (size_t)Constants::NB_Colors)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return;
+    }
+    xColor = color;
+    cost[Constants::NB_Colors] = 1;
+}
+int ManaCost::hasSpecificX()
+{
+    if (cost.size() <= (size_t)Constants::NB_Colors)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return 0;
+    }
+    if(xColor > 0)
+        return cost[Constants::NB_Colors];
+    return 0;
+}
+
+int ManaCost::hasAnotherCost()
+{
+    if (cost.size() <= (size_t)Constants::NB_Colors)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return 0;
+    }
+    int result = 0;
+    if(kicker)
+        result = 1;
+    //kicker is the only one ai knows for now, later hasAnotherCost() can be used to determine other cost types.
+    return result;
 }
 
 void ManaCost::init()
 {
     int i;
-    for (i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    
+    cost.erase(cost.begin() ,cost.end());
+    
+    for (i = 0; i <= Constants::NB_Colors; i++)
     {
-        cost[i] = 0;
+        cost.push_back(0);
     }
+    
     extraCosts = NULL;
     kicker = NULL;
     alternative = NULL;
@@ -363,15 +462,20 @@ void ManaCost::init()
     Retrace = NULL;
     morph = NULL;
     suspend = NULL;
+    isMulti = false;
 }
 
-void ManaCost::reinit()
+void ManaCost::resetCosts()
 {
     int i;
-    for (i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    
+    cost.erase(cost.begin() ,cost.end());
+
+    for (i = 0; i <= Constants::NB_Colors; i++)
     {
-        cost[i] = 0;
+        cost.push_back(0);
     }
+    
     SAFE_DELETE(extraCosts);
     SAFE_DELETE(kicker);
     SAFE_DELETE(alternative);
@@ -386,9 +490,12 @@ void ManaCost::copy(ManaCost * _manaCost)
 {
     if (!_manaCost)
         return;
-    for (unsigned int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+
+    cost.erase(cost.begin() ,cost.end());
+
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
-        cost[i] = _manaCost->getCost(i);
+        cost.push_back(_manaCost->getCost(i));
     }
 
     hybrids = _manaCost->hybrids;
@@ -404,6 +511,7 @@ void ManaCost::copy(ManaCost * _manaCost)
     {
         kicker = NEW ManaCost();
         kicker->copy(_manaCost->kicker);
+        kicker->isMulti = _manaCost->kicker->isMulti;
     }
     SAFE_DELETE(alternative);
     if (_manaCost->alternative)
@@ -441,10 +549,16 @@ void ManaCost::copy(ManaCost * _manaCost)
         suspend = NEW ManaCost();
         suspend->copy(_manaCost->suspend);
     }
+    xColor = _manaCost->xColor;
 }
 
 int ManaCost::getCost(int color)
 {
+    if (cost.size() <= (size_t)color)
+    {
+        DebugTrace("Seems ManaCost was not properly initialized");
+        return 0;
+    }
     return cost[color];
 }
 
@@ -455,9 +569,20 @@ ManaCostHybrid * ManaCost::getHybridCost(unsigned int i)
     return &hybrids[i];
 }
 
+ExtraCost * ManaCost::getExtraCost(unsigned int i)
+{
+    if(extraCosts && extraCosts->costs.size())
+	{
+        if (extraCosts->costs.size() <= i)
+            return NULL;
+        return extraCosts->costs[i];
+    }
+    return NULL;
+}
+
 int ManaCost::hasColor(int color)
 {
-    if (cost[color])
+    if (getCost(color))
         return 1;
     for (size_t i = 0; i < hybrids.size(); i++)
     {
@@ -479,7 +604,7 @@ int ManaCost::isNull()
 int ManaCost::getConvertedCost()
 {
     int result = 0;
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for ( int i = 0; i < Constants::NB_Colors; i++)
     {
         result += cost[i];
     }
@@ -520,7 +645,7 @@ int ManaCost::add(ManaCost * _cost)
 {
     if (!_cost)
         return 0;
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for ( int i = 0; i < Constants::NB_Colors; i++)
     {
         cost[i] += _cost->getCost(i);
     }
@@ -534,7 +659,7 @@ int ManaCost::remove(ManaCost * _cost)
 {
     if (!_cost)
         return 0;
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for ( int i = 0; i < Constants::NB_Colors; i++)
     {
         int8_t toRemove = min(cost[i], (int8_t)_cost->getCost(i)); //we don't want to be negative
         cost[i] -= toRemove;
@@ -563,6 +688,20 @@ int ManaCost::addExtraCost(ExtraCost * _cost)
     return 1;
 }
 
+int ManaCost::addExtraCosts(ExtraCosts *_ecost)
+{
+    if(!_ecost)
+    {
+        extraCosts = NULL;
+        return 1;
+    }
+    if (!extraCosts)
+        extraCosts = NEW ExtraCosts();
+    for(size_t i = 0; i < _ecost->costs.size(); i++)
+        extraCosts->costs.push_back(_ecost->costs[i]->clone());
+    return 1;
+}
+
 int ManaCost::isExtraPaymentSet()
 {
     if (!extraCosts)
@@ -597,7 +736,7 @@ int ManaCost::pay(ManaCost * _cost)
     ManaCost * toPay = NEW ManaCost();
     toPay->copy(_cost);
     ManaCost * diff = Diff(toPay);
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         cost[i] = diff->getCost(i);
     }
@@ -622,7 +761,7 @@ int ManaCost::canAfford(ManaCost * _cost)
 
 int ManaCost::isPositive()
 {
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
 
         if (cost[i] < 0)
@@ -634,7 +773,7 @@ int ManaCost::isPositive()
 
 }
 
-void ManaCost::randomDiffHybrids(ManaCost * _cost, int8_t diff[])
+void ManaCost::randomDiffHybrids(ManaCost * _cost, std::vector<int8_t>& diff)
 {
     for (size_t i = 0; i < _cost->hybrids.size(); i++)
     {
@@ -646,7 +785,7 @@ void ManaCost::randomDiffHybrids(ManaCost * _cost, int8_t diff[])
 /**
     starting from the end of the array (diff) 
 */
-int ManaCost::tryToPayHybrids(std::vector<ManaCostHybrid>& _hybrids, int _nbhybrids, int8_t diff[])
+int ManaCost::tryToPayHybrids(std::vector<ManaCostHybrid>& _hybrids, int _nbhybrids, std::vector<int8_t>& diff)
 {
     if (!_nbhybrids)
         return 1;
@@ -674,9 +813,13 @@ int ManaCost::tryToPayHybrids(std::vector<ManaCostHybrid>& _hybrids, int _nbhybr
 //compute the difference between two mana costs
 ManaCost * ManaCost::Diff(ManaCost * _cost)
 {
-    int8_t diff[(Constants::MTG_NB_COLORS + 1) * 2];
-    diff[Constants::MTG_NB_COLORS * 2] = Constants::MTG_NB_COLORS;
-    for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    if (!_cost) 
+        return NEW ManaCost(*this); //diff with null is equivalent to diff with 0
+
+    vector<int8_t> diff;
+    diff.resize((Constants::NB_Colors + 1) * 2);
+    diff[Constants::NB_Colors * 2] = Constants::NB_Colors;
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         diff[i * 2] = i;
         diff[i * 2 + 1] = cost[i] - _cost->getCost(i);
@@ -689,7 +832,7 @@ ManaCost * ManaCost::Diff(ManaCost * _cost)
     int colorless_idx = Constants::MTG_COLOR_ARTIFACT * 2 + 1;
     if (diff[colorless_idx] < 0)
     {
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        for (int i = 0; i < Constants::NB_Colors; i++)
         {
             if (diff[i * 2 + 1] > 0)
             {
@@ -711,18 +854,28 @@ ManaCost * ManaCost::Diff(ManaCost * _cost)
     //Cost X
     if (_cost->hasX())
     {
-        diff[Constants::MTG_NB_COLORS * 2 + 1] = 0;
-        for (int i = 0; i < Constants::MTG_NB_COLORS; i++)
+        diff[Constants::NB_Colors * 2 + 1] = 0;
+        for (int i = 0; i < Constants::NB_Colors; i++)
         {
             if (diff[i * 2 + 1] > 0)
             {
-                diff[Constants::MTG_NB_COLORS * 2 + 1] += diff[i * 2 + 1];
+                diff[Constants::NB_Colors * 2 + 1] += diff[i * 2 + 1];
                 diff[i * 2 + 1] = 0;
             }
         }
     }
+    //cost x where x is specific.
+    if (_cost->hasSpecificX())
+    {
+        diff[Constants::NB_Colors * 2 + 1] = 0;
+        if (diff[_cost->xColor * 2 + 1] > 0)
+        {
+            diff[Constants::NB_Colors * 2 + 1] += diff[_cost->xColor * 2 + 1];
+            diff[_cost->xColor * 2 + 1] = 0;
+        }
+    }
 
-    ManaCost * result = NEW ManaCost(diff, Constants::MTG_NB_COLORS + 1);
+    ManaCost * result = NEW ManaCost(diff, Constants::NB_Colors + 1);
     return result;
 
 }
@@ -730,7 +883,7 @@ ManaCost * ManaCost::Diff(ManaCost * _cost)
 string ManaCost::toString()
 {
     ostringstream oss;
-    for (int i = 0; i <= Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i <= Constants::NB_Colors; i++)
     {
         if (cost[i])
         {
@@ -752,9 +905,12 @@ string ManaCost::toString()
 #ifdef WIN32
 void ManaCost::Dump()
 {
+    //if(this->getConvertedCost())//uncomment when this is far too loud and clutters your output making other traces pointless.
+    //{
     DebugTrace( "\n===ManaCost===" );
     DebugTrace( this->toString() );
     DebugTrace( "\n=============" );
+    //}
 }
 
 #endif
@@ -774,11 +930,11 @@ ostream& operator<<(ostream& out, ManaCost m)
     return out << m.toString();
 }
 
-void ManaPool::init()
+void ManaPool::Empty()
 {
-    ManaCost::init();
+    init();
     WEvent * e = NEW WEventEmptyManaPool(this);
-    GameObserver::GetInstance()->receiveEvent(e);
+    player->getObserver()->receiveEvent(e);
 }
 
 ManaPool::ManaPool(Player * player) :
@@ -797,7 +953,7 @@ int ManaPool::remove(int color, int value)
     for (int i = 0; i < value; ++i)
     {
         WEvent * e = NEW WEventConsumeMana(color, this);
-        GameObserver::GetInstance()->receiveEvent(e);
+        player->getObserver()->receiveEvent(e);
     }
     return result;
 }
@@ -808,7 +964,7 @@ int ManaPool::add(int color, int value, MTGCardInstance * source)
     for (int i = 0; i < value; ++i)
     {
         WEvent * e = NEW WEventEngageMana(color, source, this);
-        GameObserver::GetInstance()->receiveEvent(e);
+        player->getObserver()->receiveEvent(e);
     }
     return result;
 }
@@ -818,12 +974,12 @@ int ManaPool::add(ManaCost * _cost, MTGCardInstance * source)
     if (!_cost)
         return 0;
     int result = ManaCost::add(_cost);
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         for (int j = 0; j < _cost->getCost(i); j++)
         {
             WEvent * e = NEW WEventEngageMana(i, source, this);
-            GameObserver::GetInstance()->receiveEvent(e);
+            player->getObserver()->receiveEvent(e);
         }
     }
     return result;
@@ -831,22 +987,23 @@ int ManaPool::add(ManaCost * _cost, MTGCardInstance * source)
 
 int ManaPool::pay(ManaCost * _cost)
 {
-    int current[Constants::MTG_NB_COLORS];
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    vector<int> current;
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
-        current[i] = cost[i];
+        current.push_back(cost[i]);
     }
 
     int result = ManaCost::pay(_cost);
-    for (unsigned int i = 0; i < Constants::MTG_NB_COLORS; i++)
+    for (int i = 0; i < Constants::NB_Colors; i++)
     {
         int value = current[i] - cost[i];
         for (int j = 0; j < value; j++)
         {
             WEvent * e = NEW WEventConsumeMana(i, this);
-            GameObserver::GetInstance()->receiveEvent(e);
+            player->getObserver()->receiveEvent(e);
 
         }
     }
+    current.erase(current.begin(),current.end());
     return result;
 }
