@@ -9,10 +9,15 @@
 namespace
 {
     const float kPoleWidth = 7;
-    const float kVerticalMargin = 16;
+    const float kVerticalMargin = 20;
     const float kHorizontalMargin = 30;
-    const float kLineHeight = 20;
 
+    
+#ifdef TOUCH_ENABLED
+    const float kLineHeight = 30;
+#else
+    const float kLineHeight = 20;
+#endif
     //For the "Classic" Theme
     const float kSpadeHeightOffset = 4;
     const float kSpadeWidthOffset = 9;
@@ -96,7 +101,6 @@ void SimpleMenu::drawVertPole(float x, float y, float height)
 {
     JRenderer* renderer = JRenderer::GetInstance();
 
-
     float xOffset = (spadeR->mWidth - kPoleWidth) / 2;
     float topYOffset = xOffset;
     float bottomYOffset = xOffset;
@@ -130,11 +134,15 @@ void SimpleMenu::Render()
 
         for (int i = 0; i < mCount; ++i)
         {
-            float width = (static_cast<SimpleMenuItem*> (mObjects[i]))->GetWidth();
+            float width = (static_cast<SimpleMenuItem*> (mObjects[i]))->GetEnlargedWidth() + 15;
             if (mWidth < width) mWidth = width;
         }
+
+        float scaleFactor = titleFont->GetScale();
+        titleFont->SetScale(SCALE_NORMAL);
         if ((!title.empty()) && (mWidth < titleFont->GetStringWidth(title.c_str()))) 
 			mWidth = titleFont->GetStringWidth(title.c_str());
+         titleFont->SetScale(scaleFactor);
         mWidth += 2 * kHorizontalMargin;
 
         if (mCenterHorizontal)
@@ -160,44 +168,136 @@ void SimpleMenu::Render()
     float height = mHeight;
     if (timeOpen < 1) height *= timeOpen > 0 ? timeOpen : -timeOpen;
 
-    renderer->FillRect(mX, mY, mWidth, height, ARGB(180,0,0,0));
+    float heightPadding = kLineHeight/2; // this to reduce the bottom padding of the menu
+    renderer->FillRect(mX, mY, mWidth, height - heightPadding, ARGB(180,0,0,0));
 
     renderer->SetTexBlend(BLEND_SRC_ALPHA, BLEND_ONE_MINUS_SRC_ALPHA);
-    drawVertPole(mX, mY - 16, height + 32);
-    drawVertPole(mX + mWidth, mY - 16, height + 32);
-    drawHorzPole(mX - 16, mY, mWidth + 32);
-    drawHorzPole(mX - 25, mY + height, mWidth + 50);
+    
+    drawVertPole(mX, mY, height - heightPadding);
+    drawVertPole(mX + mWidth, mY, height - heightPadding);
+    drawHorzPole(mX, mY, mWidth);
+    drawHorzPole(mX, mY + height - heightPadding, mWidth);
+    //drawVertPole(mX, mY - 16, height + 32);
+    //drawVertPole(mX + mWidth, mY - 16, height + 32);
+    //drawHorzPole(mX - 16, mY, mWidth + 32);
+    //drawHorzPole(mX - 25, mY + height, mWidth + 50);
 
     renderer->SetTexBlend(BLEND_SRC_ALPHA, BLEND_ONE);
     stars->Render();
     renderer->SetTexBlend(BLEND_SRC_ALPHA, BLEND_ONE_MINUS_SRC_ALPHA);
 
-    mFont->SetScale(1.0f);
-    if (!title.empty()) titleFont->DrawString(title.c_str(), mX + mWidth / 2, mY - 3, JGETEXT_CENTER);
+    if (!title.empty()) 
+    {
+        float scaleFactor = titleFont->GetScale();
+        titleFont->SetScale(SCALE_NORMAL);
+        titleFont->DrawString(title.c_str(), mX + mWidth / 2, mY - 3, JGETEXT_CENTER);
+        titleFont->SetScale(scaleFactor);
+    }
     for (int i = startId; i < startId + maxItems; i++)
     {
         if (i > mCount - 1) break;
-        if ((static_cast<SimpleMenuItem*> (mObjects[i]))->mY - kLineHeight * startId < mY + height - kLineHeight + 7)
+        SimpleMenuItem *currentMenuItem = static_cast<SimpleMenuItem*>(mObjects[i]);
+        float currentY = currentMenuItem->getY() - kLineHeight * startId;
+        float menuBottomEdge = mY + height - kLineHeight + 7;
+        if (currentY < menuBottomEdge)
         {
-            if (static_cast<SimpleMenuItem*> (mObjects[i])->hasFocus())
+            if (currentMenuItem->hasFocus())
             {
-                WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT)->DrawString(static_cast<SimpleMenuItem*> (mObjects[i])->desc.c_str(), mX
+                WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT)->DrawString(currentMenuItem->getDescription().c_str(), mX
                                 + mWidth + 10, mY + 15);
                 mFont->SetColor(ARGB(255,255,255,0));
             }
             else
+            {
                 mFont->SetColor(ARGB(150,255,255,255));
+            }
             (static_cast<SimpleMenuItem*> (mObjects[i]))->RenderWithOffset(-kLineHeight * startId);
         }
+        mFont->SetScale(SCALE_NORMAL);
     }
+    mFont->SetScale(SCALE_NORMAL);
 }
 
+bool SimpleMenu::CheckUserInput(JButton key)
+{
+    // a dude may have clicked somewhere, we're gonna select the closest object from where he clicked
+    // since we know we are in a menu, we just need to check one cardinality
+    int x = -1, y = -1;
+    int n = mCurr;
+    
+    if ((key == JGE_BTN_NONE) && mEngine->GetLeftClickCoordinates(x, y))
+    {
+
+        // first scan the buttons on the screen and then process the other gui elements
+        for (size_t i = 0; i < mButtons.size(); i++)
+        {
+            if (mButtons[i]->ButtonPressed())
+            {
+                mEngine->LeftClickedProcessed();
+                return true;
+            }
+        }
+        
+        if (mObjects.size())
+        {
+            float top, left;
+            float menuTopEdge =  mY + kLineHeight;
+            float menuBottomEdge = mY + mHeight - (kLineHeight/2);
+            
+            if (y < menuTopEdge)
+                n = (mCurr - 1) > 0 ? mCurr -1 : 0;
+            else if (y >= menuBottomEdge)
+                n = (mCurr + 1) < mCount ? mCurr + 1 : mCurr - 1;
+            else
+            {
+                for (int i = 0; i < mCount; i++)
+                {
+                    if (mObjects[i]->getTopLeft(top, left))
+                    {
+                        if ( (y > top) && (y <= (top + kLineHeight)) )
+                            n = i;
+                    }   
+                }
+            }
+            
+            // check bounds of n. 
+            if ( n < 0 ) 
+                n = 0;
+            if ( n >= mCount ) 
+                n = mCount - 1;
+            
+            // check to see if the user clicked 
+            if (n != mCurr && mObjects[mCurr] != NULL && mObjects[mCurr]->Leaving(JGE_BTN_DOWN))
+            {
+                mCurr = n;
+                mObjects[mCurr]->Entering();
+            }
+            // if the same object was selected process click
+            else if (n == mCurr && mObjects[mCurr] != NULL && mObjects[mCurr]->Leaving(JGE_BTN_OK))
+            {
+                mObjects[mCurr]->Entering();
+            }
+
+            mEngine->LeftClickedProcessed();
+            mEngine->ResetInput();
+            return true;
+        }
+        mEngine->LeftClickedProcessed();
+    }
+    else
+        return JGuiController::CheckUserInput(key);
+    
+    return false;
+}
+
+
 void SimpleMenu::Update(float dt)
 {
     JGuiController::Update(dt);
     if (mCurr > startId + maxItems - 1)
         startId = mCurr - maxItems + 1;
     else if (mCurr < startId) startId = mCurr;
+ 
     stars->Update(dt);
     selectionT += 3 * dt;
     selectionY += (selectionTargetY - selectionY) * 8 * dt;
@@ -225,7 +325,7 @@ void SimpleMenu::Add(int id, const char * text, string desc, bool forceFocus)
     SimpleMenuItem * smi = NEW SimpleMenuItem(this, id, fontId, text, 0, mY + kVerticalMargin + mCount * kLineHeight,
                     (mCount == 0), autoTranslate);
 
-    smi->desc = desc;
+    smi->setDescription(desc);
     JGuiController::Add(smi);
     if (mCount <= maxItems) mHeight += kLineHeight;
     if (forceFocus)
