@@ -4,6 +4,10 @@
 #include "Player.h"
 #include "Tasks.h"
 #include "AIPlayer.h"
+
+// Todo remove this dependency!
+#include "AIPlayerBaka.h"
+
 #include "Translate.h"
 #include "MTGDefinitions.h"
 #include <JRenderer.h>
@@ -160,7 +164,7 @@ void Task::LoadAIDeckNames()
         {
             found = 0;
             std::ostringstream stream;
-            stream << JGE_GET_RES("ai/baka") << "/deck" << nbDecks + 1 << ".txt";
+            stream << "ai/baka" << "/deck" << nbDecks + 1 << ".txt";
             if (fileExists(stream.str().c_str()))
             {
                 found = 1;
@@ -281,8 +285,8 @@ int TaskList::save(string _fileName)
         return -1;
     }
 
-    std::ofstream file(fileName.c_str());
-    if (file)
+    std::ofstream file;
+    if (JFileSystem::GetInstance()->openForWrite(file, fileName))
     {
 #if defined (WIN32) || defined (LINUX)
         DebugTrace("\nsaving tasks\n");
@@ -309,48 +313,39 @@ int TaskList::load(string _fileName)
     }
     if (fileName == "")
     {
-#if defined (WIN32) || defined (LINUX)
         DebugTrace("\nTaskList::load: No filename specified\n");
-#endif
         return -1;
     }
 
-    wagic::ifstream file(fileName.c_str());
-    std::string s;
-
-    if (file)
+    std::string contents;
+    if (!JFileSystem::GetInstance()->readIntoString(fileName, contents))
     {
-        while (std::getline(file, s))
-        {
-            if (!s.size()) continue;
-            if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
-            if (s[0] == '#')
-            {
-                continue;
-            }
-
-            task = Task::createFromStr(s);
-            if (task)
-            {
-                this->addTask(task);
-            }
-            else
-            {
-#if defined (WIN32) || defined (LINUX)
-                DebugTrace("\nTaskList::load: error creating task\n");
-#endif
-            }
-        }
-        file.close();
-    }
-    else
-    {
-#if defined (WIN32) || defined (LINUX)
         DebugTrace("\nTaskList::load: Failed to open file\n");
-#endif
         return -1;
     }
 
+    std::stringstream stream(contents);
+    std::string s;
+    while (std::getline(stream, s))
+    {
+        if (!s.size()) continue;
+        if (s[s.size() - 1] == '\r') s.erase(s.size() - 1); //Handle DOS files
+        if (s[0] == '#')
+        {
+            continue;
+        }
+
+        task = Task::createFromStr(s);
+        if (task)
+        {
+            this->addTask(task);
+        }
+        else
+        {
+            DebugTrace("\nTaskList::load: error creating task\n");
+        }
+    }
+
     return 1;
 }
 
@@ -436,13 +431,13 @@ void TaskList::passOneDay()
     }
 }
 
-void TaskList::getDoneTasks(Player * _p1, Player * _p2, GameApp * _app, vector<Task*>* result)
+void TaskList::getDoneTasks(GameObserver* observer, GameApp * _app, vector<Task*>* result)
 {
     result->clear();
     // TODO: Return only accepted tasks
     for (vector<Task*>::iterator it = tasks.begin(); it != tasks.end(); it++)
     {
-        if ((*it)->isDone(_p1, _p2, _app))
+        if ((*it)->isDone(observer, _app))
         {
             result->push_back(*it);
         }
@@ -524,7 +519,6 @@ void TaskList::Render()
     if (0 == tasks.size())
     {
         f->DrawString(_("There are no tasks that need to be done. Come again tomorrow.").c_str(), posX, posY);
-        posY += 20;
         return;
     }
 
@@ -620,11 +614,10 @@ string TaskWinAgainst::getShortDesc()
     return result;
 }
 
-bool TaskWinAgainst::isDone(Player * _p1, Player * _p2, GameApp * _app)
+bool TaskWinAgainst::isDone(GameObserver* observer, GameApp * _app)
 {
-    GameObserver * g = GameObserver::GetInstance();
-    AIPlayerBaka * baka = (AIPlayerBaka*) _p2;
-    return ((baka) && (!_p1->isAI()) && (_p2->isAI()) && (g->gameOver != _p1) // Human player wins
+    AIPlayerBaka * baka = (AIPlayerBaka*) observer->players[1];
+    return ((baka) && (!observer->players[0]->isAI()) && (observer->players[1]->isAI()) && (observer->didWin(observer->players[0])) // Human player wins
                     && (baka->deckId == opponent));
 }
 
@@ -675,9 +668,9 @@ string TaskSlaughter::getShortDesc()
     return buffer;
 }
 
-bool TaskSlaughter::isDone(Player * _p1, Player * _p2, GameApp * _app)
+bool TaskSlaughter::isDone(GameObserver* observer, GameApp * _app)
 {
-    return TaskWinAgainst::isDone(_p1, _p2, _app) && (_p2->life <= targetLife);
+    return TaskWinAgainst::isDone(observer, _app) && (observer->players[1]->life <= targetLife);
 }
 
 void TaskSlaughter::storeCustomAttribs()
@@ -761,10 +754,9 @@ string TaskDelay::getShortDesc()
     return buffer;
 }
 
-bool TaskDelay::isDone(Player * _p1, Player * _p2, GameApp * _app)
+bool TaskDelay::isDone(GameObserver* observer, GameApp * _app)
 {
-    GameObserver * g = GameObserver::GetInstance();
-    return TaskWinAgainst::isDone(_p1, _p2, _app) && (afterTurn ? (g->turn >= turn) : (g->turn <= turn));
+    return TaskWinAgainst::isDone(observer, _app) && (afterTurn ? (observer->turn >= turn) : (observer->turn <= turn));
 }
 
 void TaskDelay::storeCustomAttribs()
@@ -829,11 +821,10 @@ string TaskImmortal::getShortDesc()
     return buffer;
 }
 
-bool TaskImmortal::isDone(Player * _p1, Player * _p2, GameApp * _app)
+bool TaskImmortal::isDone(GameObserver *observer, GameApp * _app)
 {
-    GameObserver * g = GameObserver::GetInstance();
-    return (!_p1->isAI()) && (_p2->isAI()) && (g->gameOver != _p1) // Human player wins
-                    && (_p1->life >= targetLife);
+    return (!observer->players[0]->isAI()) && (observer->players[1]->isAI()) && (observer->didWin(observer->players[0])) // Human player wins
+                    && (observer->players[0]->life >= targetLife);
 }
 
 void TaskImmortal::storeCustomAttribs()
@@ -925,10 +916,10 @@ string TaskMassiveBurial::getShortDesc()
     return buffer;
 }
 
-bool TaskMassiveBurial::isDone(Player * _p1, Player * _p2, GameApp * _app)
+bool TaskMassiveBurial::isDone(GameObserver* observer, GameApp * _app)
 {
     int countColor = 0;
-    vector<MTGCardInstance *> cards = _p2->game->graveyard->cards;
+    vector<MTGCardInstance *> cards = observer->players[1]->game->graveyard->cards;
 
     for (vector<MTGCardInstance *>::iterator it = cards.begin(); it != cards.end(); it++)
     {
@@ -959,7 +950,7 @@ void TaskMassiveBurial::restoreCustomAttribs()
 
 void TaskMassiveBurial::randomize()
 {
-    color = rand() % (Constants::MTG_NB_COLORS - 1) + 1;
+    color = rand() % (Constants::NB_Colors - 1) + 1;
     bodyCount = 5 + ((Constants::MTG_COLOR_LAND == color) ? rand() % 10 : rand() % 20);
     Task::randomize();
 }
@@ -1018,11 +1009,10 @@ string TaskWisdom::getShortDesc()
     return buffer;
 }
 
-bool TaskWisdom::isDone(Player * _p1, Player * _p2, GameApp * _app)
+bool TaskWisdom::isDone(GameObserver* observer, GameApp * _app)
 {
-    GameObserver * g = GameObserver::GetInstance();
     int countColor = 0;
-    vector<MTGCardInstance *> cards = _p1->game->hand->cards;
+    vector<MTGCardInstance *> cards = observer->players[0]->game->hand->cards;
 
     for (vector<MTGCardInstance *>::iterator it = cards.begin(); it != cards.end(); it++)
     {
@@ -1032,7 +1022,7 @@ bool TaskWisdom::isDone(Player * _p1, Player * _p2, GameApp * _app)
         }
     }
 
-    return (!_p1->isAI()) && (_p2->isAI()) && (g->gameOver != _p1) // Human player wins
+    return (!observer->players[0]->isAI()) && (observer->players[1]->isAI()) && (observer->didWin(observer->players[0])) // Human player wins
                     && (countColor >= cardCount);
 }
 
@@ -1054,7 +1044,7 @@ void TaskWisdom::restoreCustomAttribs()
 
 void TaskWisdom::randomize()
 {
-    color = rand() % (Constants::MTG_NB_COLORS - 1) + 1;
+    color = rand() % (Constants::NB_Colors - 1) + 1;
     cardCount = 2 + ((Constants::MTG_COLOR_LAND == color) ? rand() % 5 : rand() % 7);
     Task::randomize();
 }
@@ -1100,11 +1090,10 @@ string TaskPacifism::getShortDesc()
     return buffer;
 }
 
-bool TaskPacifism::isDone(Player * _p1, Player * _p2, GameApp * _app)
+bool TaskPacifism::isDone(GameObserver* observer, GameApp * _app)
 {
-    GameObserver * g = GameObserver::GetInstance();
-    return (!_p1->isAI()) && (_p2->isAI()) && (g->gameOver != _p1) // Human player wins
-                    && (_p2->life >= lifeSlashCardMin) && ((int) _p2->game->library->cards.size() >= lifeSlashCardMin);
+    return (!observer->players[0]->isAI()) && (observer->players[1]->isAI()) && (observer->didWin(observer->players[0])) // Human player wins
+                    && (observer->players[1]->life >= lifeSlashCardMin) && ((int) observer->players[1]->game->library->cards.size() >= lifeSlashCardMin);
 }
 
 void TaskPacifism::storeCustomAttribs()
@@ -1159,11 +1148,9 @@ void TaskPacifism::randomize()
  return buffer;
  }
 
- bool TaskXX::isDone(Player * _p1, Player * _p2, GameApp * _app) {
- GameObserver * g = GameObserver::GetInstance();
+ bool TaskXX::isDone(GameObserver* observer, GameApp * _app) {
  // TODO: Implement
- return (!_p1->isAI()) && (_p2->isAI()) && (g->gameOver != _p1) // Human player wins
- && ;
+ return (!observer->players[0]->isAI()) && (observer->players[1]->isAI()) && (observer->gameOver != _p1) // Human player wins
  }
 
  void TaskXX::storeCustomAttribs() {
