@@ -4,6 +4,7 @@
 #include "Counters.h"
 #include "MTGCardInstance.h"
 #include "CardDescriptor.h"
+#include "AllAbilities.h"
 
 ThisDescriptor::~ThisDescriptor()
 {
@@ -31,7 +32,7 @@ int ThisDescriptor::matchValue(int value)
     return 0;
 }
 
-ThisDescriptor * ThisDescriptorFactory::createThisDescriptor(string s)
+ThisDescriptor * ThisDescriptorFactory::createThisDescriptor(GameObserver* observer, string s)
 {
     size_t found;
 
@@ -116,7 +117,7 @@ ThisDescriptor * ThisDescriptorFactory::createThisDescriptor(string s)
         size_t start = s.find("{");
         size_t end = s.find("}");
         string counterString = s.substr(start + 1, end - start - 1);
-        AbilityFactory abf;
+        AbilityFactory abf(observer);
         Counter * counter = abf.parseCounter(counterString, NULL);
         if (counter)
         {
@@ -253,6 +254,19 @@ ThisDescriptor * ThisDescriptorFactory::createThisDescriptor(string s)
         return NULL;
     }
     
+        //this creature has 2 of the same weapons in its children vector
+    found = s.find("dualwielding");
+    if (found != string::npos)
+    {
+        ThisDualWield * td = NEW ThisDualWield(criterion);
+        if (td)
+        {
+            td->comparisonMode = mode;
+            return td;
+        }
+        return NULL;
+    }
+
     //controller life
     found = s.find("opponentlife");
     if (found != string::npos)
@@ -318,9 +332,59 @@ ThisDescriptor * ThisDescriptorFactory::createThisDescriptor(string s)
         return NULL;
     }
 
+    //this Variable
+    vector<string>splitVComparison = parseBetween(s,"variable{","}",false);
+    if (splitVComparison.size())
+    {
+        ThisVariable * td = NEW ThisVariable(criterion,splitVComparison[1]);
+        if (td)
+        {
+            td->comparisonMode = mode;
+            return td;
+        }
+        return NULL;
+    }
+
+    vector<string>splitTargetComparison = parseBetween(s,"cantargetcard(",")",false);
+    if (splitTargetComparison.size())
+    {
+        TargetChooserFactory tf(observer);
+        TargetChooser * tcc = tf.createTargetChooser(splitTargetComparison[1],NULL);
+        ThisTargetCompare * td = NEW ThisTargetCompare(tcc);
+        if (td)
+        {
+            return td;
+        }
+        return NULL;
+    }
+
     return NULL;
 }
 
+ThisTargetCompare::ThisTargetCompare(TargetChooser * _tcc)
+{
+    targetComp = _tcc;
+}
+
+int ThisTargetCompare::match(MTGCardInstance * card)
+{
+    if(targetComp->canTarget(card))
+        return 1;
+    return 0;
+}
+
+ThisTargetCompare::~ThisTargetCompare()
+{
+    SAFE_DELETE(targetComp);
+}
+
+ThisTargetCompare* ThisTargetCompare::clone() const 
+{
+    ThisTargetCompare * a =  NEW ThisTargetCompare(*this);
+
+    return a;
+}
+
 ThisCounter::ThisCounter(Counter * _counter)
 {
     counter = _counter;
@@ -369,6 +433,13 @@ ThisCounter::~ThisCounter()
     SAFE_DELETE(counter);
 }
 
+ThisCounter* ThisCounter::clone() const 
+{
+    ThisCounter * a =  NEW ThisCounter(*this);
+    a->counter = NEW Counter(NULL, counter->name.c_str(), counter->power, counter->toughness);
+    return a;
+}
+
 ThisOpponentlife::ThisOpponentlife(int olife)
 {
     comparisonCriterion = olife;
@@ -379,6 +450,11 @@ int ThisOpponentlife::match(MTGCardInstance * card)
     return matchValue(card->controller()->opponent()->life);
 }
 
+ThisOpponentlife* ThisOpponentlife::clone() const 
+{
+    return NEW ThisOpponentlife(*this);
+}
+
 ThisControllerlife::ThisControllerlife(int life)
 {
     comparisonCriterion = life;
@@ -389,6 +465,11 @@ int ThisControllerlife::match(MTGCardInstance * card)
     return matchValue(card->controller()->life);
 }
 
+ThisControllerlife* ThisControllerlife::clone() const 
+{
+    return NEW ThisControllerlife(*this);
+}
+
 ThisPower::ThisPower(int power)
 {
     comparisonCriterion = power;
@@ -399,6 +480,11 @@ int ThisPower::match(MTGCardInstance * card)
     return matchValue(card->power);
 }
 
+ThisPower* ThisPower::clone() const 
+{
+    return NEW ThisPower(*this);
+}
+
 ThisEquip::ThisEquip(int equipment)
 {
     comparisonCriterion = equipment;
@@ -408,6 +494,11 @@ int ThisEquip::match(MTGCardInstance * card)
     return matchValue(card->equipment);
 }
 
+ThisEquip* ThisEquip::clone() const 
+{
+    return NEW ThisEquip(*this);
+}
+
 ThisAuras::ThisAuras(int auras)
 {
     comparisonCriterion = auras;
@@ -417,6 +508,11 @@ int ThisAuras::match(MTGCardInstance * card)
     return matchValue(card->auras);
 }
 
+ThisAuras* ThisAuras::clone() const 
+{
+    return NEW ThisAuras(*this);
+}
+
 ThisOpponentDamageAmount::ThisOpponentDamageAmount(int damagecount)
 {
     comparisonCriterion = damagecount;
@@ -426,6 +522,11 @@ int ThisOpponentDamageAmount::match(MTGCardInstance * card)
     return matchValue(card->controller()->opponent()->damageCount);
 }
 
+ThisOpponentDamageAmount* ThisOpponentDamageAmount::clone() const 
+{
+    return NEW ThisOpponentDamageAmount(*this);
+}
+
 ThisUntapped::ThisUntapped(int untapped)
 {
     comparisonCriterion = untapped;
@@ -435,6 +536,11 @@ int ThisUntapped::match(MTGCardInstance * card)
     return matchValue(!card->isTapped());
 }
 
+ThisUntapped* ThisUntapped::clone() const 
+{
+    return NEW ThisUntapped(*this);
+}
+
 ThisTapped::ThisTapped(int tapped)
 {
     comparisonCriterion = tapped;
@@ -444,6 +550,11 @@ int ThisTapped::match(MTGCardInstance * card)
     return matchValue(card->isTapped());
 }
 
+ThisTapped* ThisTapped::clone() const 
+{
+    return NEW ThisTapped(*this);
+}
+
 ThisAttacked::ThisAttacked(int attack)
 {
 
@@ -456,6 +567,11 @@ int ThisAttacked::match(MTGCardInstance * card)
     return matchValue(card->didattacked);
 }
 
+ThisAttacked* ThisAttacked::clone() const 
+{
+    return NEW ThisAttacked(*this);
+}
+
 ThisBlocked::ThisBlocked(int block)
 {
 
@@ -468,6 +584,11 @@ int ThisBlocked::match(MTGCardInstance * card)
     return matchValue(card->didblocked);
 }
 
+ThisBlocked* ThisBlocked::clone() const 
+{
+    return NEW ThisBlocked(*this);
+}
+
 ThisNotBlocked::ThisNotBlocked(int unblocked)
 {
 
@@ -480,6 +601,11 @@ int ThisNotBlocked::match(MTGCardInstance * card)
     return matchValue(card->notblocked);
 }
 
+ThisNotBlocked* ThisNotBlocked::clone() const 
+{
+    return NEW ThisNotBlocked(*this);
+}
+
 ThisDamaged::ThisDamaged(int wasDealtDamage)
 {
 
@@ -494,6 +620,30 @@ result = 1;
     return matchValue(result);
 }
 
+ThisDamaged* ThisDamaged::clone() const 
+{
+    return NEW ThisDamaged(*this);
+}
+
+ThisDualWield::ThisDualWield(int dualWield)
+{
+
+    comparisonCriterion = dualWield;
+}
+
+int ThisDualWield::match(MTGCardInstance * card)
+{
+int result = 0;
+if(card->isDualWielding)
+result = 1;
+    return matchValue(result);
+}
+
+ThisDualWield* ThisDualWield::clone() const 
+{
+    return NEW ThisDualWield(*this);
+}
+
 ThisToughness::ThisToughness(int toughness)
 {
     comparisonCriterion = toughness;
@@ -504,6 +654,11 @@ int ThisToughness::match(MTGCardInstance * card)
     return matchValue(card->toughness);
 }
 
+ThisToughness* ThisToughness::clone() const 
+{
+    return NEW ThisToughness(*this);
+}
+
 ThisCounterAny::ThisCounterAny(int nb)
 {
     comparisonCriterion = nb;
@@ -519,6 +674,11 @@ int ThisCounterAny::match(MTGCardInstance * card)
     return matchValue(result);
 }
 
+ThisCounterAny * ThisCounterAny::clone() const 
+{
+    return NEW ThisCounterAny(*this);
+}
+
 ThisX::ThisX(int x)
 {
     comparisonCriterion = x;
@@ -528,3 +688,29 @@ int ThisX::match(MTGCardInstance * card)
 {
     return matchValue(card->X);
 }
+
+ThisX * ThisX::clone() const 
+{
+    return NEW ThisX(*this);
+}
+
+//
+ThisVariable::ThisVariable(int comp,string _vWord)
+{
+    vWord = _vWord;
+    comparisonCriterion = comp;
+}
+
+int ThisVariable::match(MTGCardInstance * card)
+{
+    int result = 0;
+    WParsedInt * res = NEW WParsedInt(vWord,NULL,card);
+    result = res->getValue();
+    SAFE_DELETE(res);
+    return matchValue(result);
+}
+
+ThisVariable * ThisVariable::clone() const 
+{
+    return NEW ThisVariable(*this);
+}
\ No newline at end of file
