@@ -197,8 +197,12 @@ PIXEL_TYPE WDecoStyled::getColor(int type)
 void WGuiHeader::Render()
 {
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
+    // save the current scaling factor.  We don't want the lists to change font size these lists should stay the same no matter what
+    float currentScale = mFont->GetScale();
+    mFont->SetScale(SCALE_NORMAL);
     mFont->SetColor(getColor(WGuiColor::TEXT));
     mFont->DrawString(_(displayValue).c_str(), x + width / 2, y, JGETEXT_CENTER);
+    mFont->SetScale(currentScale);
 }
 
 bool WGuiMenu::Leaving(JButton key)
@@ -273,7 +277,7 @@ bool WGuiMenu::yieldFocus()
 
 //WGuiList
 WGuiList::WGuiList(string name, WSyncable * syncme) :
-    WGuiMenu(JGE_BTN_DOWN, JGE_BTN_UP, false, syncme)
+    WGuiMenu(JGE_BTN_DOWN, JGE_BTN_UP, false, syncme), startWindow(-1), endWindow(-1)
 {
     failMsg = "NO OPTIONS AVAILABLE";
     width = SCREEN_WIDTH - 10;
@@ -300,13 +304,17 @@ void WGuiList::Render()
     int adjustedCurrent = 0;
     int start = 0, nowPos = 0, vHeight = 0;
     int nbitems = (int) items.size();
-
+    
     //List is empty.
     if (!items.size() && failMsg != "")
     {
         WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
+        // save the current scaling factor.  We don't want the lists to change font size these lists should stay the same no matter what
+        float fontScaleFactor = mFont->GetScale();
+        mFont->SetScale(SCALE_NORMAL);
         mFont->SetColor(getColor(WGuiColor::TEXT_FAIL));
         mFont->DrawString(_(failMsg).c_str(), x + width / 2, y, JGETEXT_RIGHT);
+        mFont->SetScale(fontScaleFactor);
         return;
     }
 
@@ -360,10 +368,11 @@ void WGuiList::Render()
     //Render items.
     if (start >= 0)
     {
+        int pos;
         //Render current underlay.
         if (currentItem >= 0 && currentItem < nbitems && items[currentItem]->Visible()) items[currentItem]->Underlay();
 
-        for (int pos = 0; pos < nbitems; pos++)
+        for (pos = 0; pos < nbitems; pos++)
         {
             if (!items[pos]->Visible()) continue;
 
@@ -386,6 +395,9 @@ void WGuiList::Render()
             break;
         }
 
+        startWindow = start;
+        endWindow = pos;
+
         //Draw scrollbar
         if (listHeight > SCREEN_HEIGHT && listSelectable > 1)
         {
@@ -418,6 +430,43 @@ void WGuiList::ButtonPressed(int controllerId, int controlId)
     it->ButtonPressed(controllerId, controlId);
 }
 
+bool WGuiList::CheckUserInput(JButton key)
+{
+    JGE * mEngine = JGE::GetInstance();
+    int i, j;
+
+    if ((key == JGE_BTN_OK) && mEngine->GetLeftClickCoordinates(i, j))
+    {   // a dude clicked somwhere, we're gonna select the closest object from where he clicked
+        int n = currentItem;
+        unsigned int distance2;
+        unsigned int minDistance2 = -1;
+        int begin = (startWindow == -1) ? 0 : startWindow;
+        int end = (endWindow == -1) ? items.size() : endWindow;
+        for(int k = begin; k < end; k++)
+        {
+          WGuiBase* pItem = (items[k]);
+          distance2 = static_cast<unsigned int>((pItem->getY() - j) * (pItem->getY() - j) + (pItem->getX() - i) * (pItem->getX() - i));
+          if (distance2 < minDistance2 && pItem->Selectable())
+          {
+              minDistance2 = distance2;
+              n = k;
+          }
+        }
+
+        if (n != currentItem && items[n]->Selectable())
+        {
+            setSelected(n);
+            mEngine->LeftClickedProcessed();
+            if (sync) syncMove();
+            return true;
+        }
+    }
+
+//    mEngine->LeftClickedProcessed();
+    return WGuiMenu::CheckUserInput(key);
+}
+
+
 string WDecoEnum::lookupVal(int value)
 {
 
@@ -503,7 +552,7 @@ void WDecoConfirm::Entering(JButton key)
 
     SAFE_DELETE(confirmMenu);
     mState = OP_CONFIRMED;
-    confirmMenu = NEW SimpleMenu(444, listener, Fonts::MENU_FONT, 50, 170);
+    confirmMenu = NEW SimpleMenu(JGE::GetInstance(), 444, listener, Fonts::MENU_FONT, 50, 170);
     confirmMenu->Add(1, confirm.c_str());
     confirmMenu->Add(2, cancel.c_str());
 }
@@ -860,13 +909,12 @@ bool WGuiSplit::yieldFocus()
 }
 
 //WGuiMenu
-WGuiMenu::WGuiMenu(JButton next = JGE_BTN_RIGHT, JButton prev = JGE_BTN_LEFT, bool m, WSyncable * syncme) :
-    WGuiItem("")
+WGuiMenu::WGuiMenu(JButton next, JButton prev, bool dPad, WSyncable * syncme) : WGuiItem("")
 {
     buttonNext = next;
     buttonPrev = prev;
     currentItem = -1;
-    mDPad = m;
+    mDPad = dPad;
     sync = syncme;
     held = JGE_BTN_NONE;
 }
@@ -934,7 +982,7 @@ bool WGuiMenu::CheckUserInput(JButton key)
         unsigned int minDistance2 = -1;
         for(size_t k = 0; k < items.size(); k++)
         {
-          WGuiItem* pItem = (WGuiItem*)items[k];
+          WGuiBase* pItem = items[k];
           distance2 = static_cast<unsigned int>((pItem->getY() - j) * (pItem->getY() - j) + (pItem->getX() - i) * (pItem->getX() - i));
           if (distance2 < minDistance2 && pItem->Selectable())
           {
@@ -1061,9 +1109,7 @@ bool WGuiMenu::nextItem()
 
     while (potential < nbitems - 1 && items[potential]->Selectable() == false)
         potential++;
-    if (potential == nbitems || !items[potential]->Selectable())
-        potential = -1;
-    else if (potential != currentItem && (!now || now->Leaving(buttonNext)))
+    if (potential != currentItem && (!now || now->Leaving(buttonNext)))
     {
         currentItem = potential;
         items[currentItem]->Entering(buttonNext);
@@ -1092,9 +1138,9 @@ bool WGuiMenu::prevItem()
 
     while (potential > 0 && items[potential]->Selectable() == false)
         potential--;
-    if (potential < 0 || !items[potential]->Selectable())
-        potential = -1;
-    else if (potential != currentItem && (!now || now->Leaving(buttonNext)))
+
+    if ( (!(potential < 0 || !items[potential]->Selectable()))
+          && (potential != currentItem && (!now || now->Leaving(buttonNext))))
     {
         currentItem = potential;
         items[currentItem]->Entering(buttonPrev);
@@ -1254,7 +1300,7 @@ void WGuiAward::Underlay()
     char buf[1024];
     JQuadPtr trophy;
 
-    string n = Options::getName(id);
+    string n = id ? Options::getName(id) : textId;
     if (n.size())
     {
         sprintf(buf, "trophy_%s.png", n.c_str()); //Trophy specific to the award
@@ -1277,7 +1323,7 @@ void WGuiAward::Underlay()
 }
 void WGuiAward::Render()
 {
-    GameOptionAward * goa = dynamic_cast<GameOptionAward*> (&options[id]);
+    GameOptionAward * goa = id ? dynamic_cast<GameOptionAward*> (&options[id]) : dynamic_cast<GameOptionAward*> (&options[textId]);
 
     if (!goa) return;
 
@@ -1320,6 +1366,17 @@ WGuiAward::WGuiAward(int _id, string name, string _text, string _details) :
     height = 60;
     details = _details;
 }
+
+WGuiAward::WGuiAward(string _id, string name, string _text, string _details) :
+    WGuiItem(name)
+{
+    id = 0;
+    textId = _id;
+    text = _text;
+    height = 60;
+    details = _details;
+}
+
 WGuiAward::~WGuiAward()
 {
     GameOptionAward * goa = dynamic_cast<GameOptionAward*> (&options[id]);
@@ -1328,7 +1385,7 @@ WGuiAward::~WGuiAward()
 bool WGuiAward::Visible()
 {
     //WGuiAward is only visible when it's tied to an already achieved award.
-    GameOptionAward * goa = dynamic_cast<GameOptionAward*> (&options[id]);
+    GameOptionAward * goa = id ? dynamic_cast<GameOptionAward*> (&options[id]) : dynamic_cast<GameOptionAward*> (&options[textId]);
     if (!goa || !goa->number) return false;
     return true;
 }
@@ -1550,7 +1607,7 @@ WDistort::WDistort(float x1, float y1, float x2, float y2, float x3, float y3, f
 
 void WGuiListRow::Render()
 {
-    int start = 0, nowPos = 0, vHeight = 0;
+    int start = 0, nowPos = 0;
     int nbitems = (int) items.size();
 
     //List is empty.
@@ -1576,7 +1633,6 @@ void WGuiListRow::Render()
         }
     }
 
-    vHeight = 0;
     nowPos = 4;
     float nowVPos = 4;
     float tallestRow = 0;
@@ -1640,7 +1696,7 @@ bool WGuiFilters::Finish(bool emptyset)
         {
             WCFilterFactory * wc = WCFilterFactory::GetInstance();
             WCardFilter * f = wc->Construct(src);
-            if (recolorTo > -1 && recolorTo < Constants::MTG_NB_COLORS)
+            if (recolorTo > -1 && recolorTo < Constants::NB_Colors)
             {
                 f = NEW WCFilterAND(f, NEW WCFilterColor(recolorTo));
             }
@@ -1648,7 +1704,7 @@ bool WGuiFilters::Finish(bool emptyset)
         }
         else
         {
-            if (recolorTo > -1 && recolorTo < Constants::MTG_NB_COLORS)
+            if (recolorTo > -1 && recolorTo < Constants::NB_Colors)
             {
                 WCardFilter * f = NEW WCFilterColor(recolorTo);
                 source->addFilter(f);
@@ -1798,6 +1854,7 @@ void WGuiFilters::Render()
 bool WGuiFilters::CheckUserInput(JButton key)
 {
     if (subMenu && !subMenu->isClosed() && subMenu->CheckUserInput(key)) return true;
+    
     if (key == JGE_BTN_SEC)
     {//|| key == JGE_BTN_MENU){
         //TODO Pop up a "Are you sure?" dialog.
@@ -1919,7 +1976,8 @@ void WGuiFilterItem::updateValue()
         SAFE_DELETE(mParent->subMenu);
         mState = STATE_CHOOSE_TYPE;
         SAFE_DELETE(mParent->subMenu);
-        mParent->subMenu = NEW SimpleMenu(-1234, this, Fonts::MENU_FONT, 20, 20, "Filter By...", 10);
+        mParent->subMenu = NEW SimpleMenu(JGE::GetInstance(), -1234, this, Fonts::MENU_FONT, 20, 20, "Filter By...", 6);
+            
         if (mParent->isAvailable(FILTER_SET))
         {
             mParent->subMenu->Add(FILTER_SET, "Set");
@@ -1987,7 +2045,7 @@ void WGuiFilterItem::updateValue()
         SAFE_DELETE(mParent->subMenu);
         mParent->clearArgs();
         mState = STATE_CHOOSE_VAL;
-        mParent->subMenu = NEW SimpleMenu(-1234, this, Fonts::MENU_FONT, 20, 20, "Filter:");
+        mParent->subMenu = NEW SimpleMenu(JGE::GetInstance(), -1234, this, Fonts::MAIN_FONT, 20, 20, "Filter:");
         if (filterType == FILTER_TYPE)
         {
             mParent->addArg("Artifact", "t:Artifact;");
@@ -2002,6 +2060,7 @@ void WGuiFilterItem::updateValue()
             mParent->addArg("Legendary", "t:Legendary;");
             mParent->addArg("Sorcery", "t:Sorcery;");
             mParent->addArg("Tribal", "t:Tribal;");
+            mParent->addArg("Planeswalker", "t:Planeswalker;");
 
         }
         else if (filterType == FILTER_SUBTYPE)
@@ -2009,7 +2068,7 @@ void WGuiFilterItem::updateValue()
             vector<string> stlist;
             for (int i = Subtypes::LAST_TYPE + 1;; i++)
             {
-                string s = Subtypes::subtypesList->find(i);
+                string s = MTGAllCards::findType(i);
                 if (s == "") break;
                 if (s.find(" ") != string::npos) continue;
                 if (s == "Nothing")
@@ -2295,7 +2354,7 @@ WGuiBase::CONFIRM_TYPE WGuiKeyBinder::needsConfirm()
             confirmationString = ss.str();
 
             // Then create the menu.
-            confirmMenu = NEW SimpleMenu(0, this, Fonts::MENU_FONT, 40, 130, "Conflict");
+            confirmMenu = NEW SimpleMenu(JGE::GetInstance(), 0, this, Fonts::MENU_FONT, 40, 130, "Conflict");
             confirmMenu->Add(1, _("Cancel and return to the options menu").c_str());
             confirmMenu->Add(2, _("This is okay, validate and save").c_str());
             return CONFIRM_NEED;
@@ -2303,6 +2362,7 @@ WGuiBase::CONFIRM_TYPE WGuiKeyBinder::needsConfirm()
     }
 
     // Check whether any button has no key associated to it.
+#if (!defined IOS)
     confirmingButton = JGE_BTN_NONE;
     for (signed int i = (sizeof(btnToCheck) / sizeof(btnToCheck[0])) - 1; i >= 0; --i)
     {
@@ -2322,12 +2382,13 @@ WGuiBase::CONFIRM_TYPE WGuiKeyBinder::needsConfirm()
         confirmationString = s;
 
         confirmingButton = btnToCheck[i];
-        confirmMenu = NEW SimpleMenu(1, this, Fonts::MENU_FONT, 40, 130, "Binding missing");
+        confirmMenu = NEW SimpleMenu(JGE::GetInstance(), 1, this, Fonts::MENU_FONT, 40, 130, "Binding missing");
         confirmMenu->Add(1, _("Cancel and return to the options menu").c_str());
         confirmMenu->Add(2, _("This is okay, validate and save").c_str());
         return CONFIRM_NEED;
     }
-
+#endif // IOS
+    
     return CONFIRM_OK;
 }
 
